<?php
/**
 * Send a requested AUDIO FILE or PLAYLIST to user from a redirected /ext/audio request
 * this script is used for file access from embeded or external audio players
 *
 * 3 kind of audio file can be processed :
 *	- audio files (.mp3, .wav...)
 *	- existing playlist (.m3u files)
 *	- PHP playlist (used by audio explorer scripts), named *playlist*.m3u[.xml]
 *   - single mp3 playlist (used by audio explorer scripts),named *singleMP3*.mp3.xml, used to bypass flash player limitation (cannot read somme characters like "&"...)
 * if a .xml extension is added on a m3u or a mp3 file, the actual output is converted to an XML playlist file designed for being red by flash player.
 *
 * All (mime) video files can be processed. Only simple "streaming" is supported (no playlist or XML stuff...)
 *
 * URI must be formated like "/external/sess_$id/[audio/$bitrate/]urlencoded($filename)
 * where "$id" is an active session id, and "urlencoded(filename)" is url-encoded requested
 * file name (must include absolute path, and not relative path)
 *
 * If audio/$bitrate/ is specified, output file is reencoded with FFMPEG encoder
 *
 * If $filename is an existing m3u file, an modified m3u is sent to user, replacing song's filenames by
 * reformated filenames (/external/sess_$id/...)
 *
 * If $fileName is *playlist*.m3u or *playlist*.m3u.xml,
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

//cfDbgUrl();


define('SESSION_NAME_PREFIX','sess_');
define('ASF_HEADER_LENGTH',444);

$resampledAudioFile=false;
$bitrate=false;

// Set to true to remove Content-Type header (debug)
$debugRemoveContentHeader=false;

/**
 * @desc Reencode to WMA and stream to output
 *
 * @param string $completeFilename
 * @param integer $bitrate
 */

function reencodeWMA($completeFilename, $bitrate=64){
	global $debugRemoveContentHeader;
	global $bitrate;

	/**
	 * Replace file size and number of packets in ASF header
	 *
	 * @param string $data
	 */
	function processASFHeader($data,$totalLength=-1){
		static $buffer='';
		static $bufferProcessed=0;


		$echo='';

		if($totalLength==-1) $_ENV['outputLength']+=strlen($data); else $_ENV['outputLength']=$totalLength;
		cfDebugUpdateBuffer($_ENV['outputLength']);
		if($bufferProcessed) {
			if($totalLength>=$_ENV['wmaFileSize']) {
				$echo.=substr($data,0,strlen($data)-($totalLength-$_ENV['wmaFileSize']));
				cfDebugUpdateBuffer('coupe');
				return $echo;
				//return true;
			}
			else $echo.=$data;
			//cfDebugUpdateBuffer(strlen($data).' -> '.number_format($_ENV['outputLength'],null,'.',' '));
			return $echo;
			// Limit sent data to announced file size
		}
		else{
			$buffer.=$data;
			// Process buffer
			if(strlen($buffer)>=ASF_HEADER_LENGTH){
				cfDebugUpdateBuffer('Process ASF header');
				$buffer=$_ENV['baseHeader'].substr($buffer,ASF_HEADER_LENGTH);

				// Replace number of packets
				$buffer[434]=$buffer[86]=$_ENV['wmaPackets'][0];
				$buffer[435]=$buffer[87]=$_ENV['wmaPackets'][1];

				// Replace file size
				$pfs=pack('V',$_ENV['wmaFileSize']);
				$buffer[70]=$pfs[0];
				$buffer[71]=$pfs[1];
				$buffer[72]=$pfs[2];
				$buffer[73]=$pfs[3];
				$pfs=pack('V',$_ENV['wmaFileSize']-394);
				$buffer[410]=$pfs[0];
				$buffer[411]=$pfs[1];
				$buffer[412]=$pfs[2];
				$buffer[413]=$pfs[3];

				// Replace stuff by other stuff...
//							$buffer[118]=chr(2);
// 							Streaming genome ?
				$buffer[94]=ord(96);
				$buffer[95]=ord(66);
				$buffer[96]=ord(41);
				$buffer[97]=ord(102);
/*
				$buffer[102]=ord(64);
				$buffer[103]=ord(9);
				$buffer[104]=ord(58);
				$buffer[105]=ord(109);
*/
				$echo.=$buffer;
				$bufferProcessed=true;
				$buffer='';
			}
			return $echo;
		}
	}

	/**
	 * @dest estimate WMA reencoded file size and number of packets
	 *
	 * @param string $cfn: source file
	 * @param integer $br: output bitrate (32,64,128)
	 * @return array (filesize, number of packets)
	 */
	function estimatedReencodedSize($cfn,$br){
		static $ratio=array('32'=>4700,'64'=>8700,'128'=>18000);

		// Try to get file length
		$ai=efGetAudioInfo($cfn,false,array('vbrParsing'=>1));
		if(!isset($ai['length'])) return array(false,false);

		// Transform ms to s
		$length=$ai['length']/1000;

		if(!isset($ratio[$br])) return array(false,false);
		//return (array(1818044,floor((1818044-444)/3200)));
		return (array(ASF_HEADER_LENGTH+floor($ratio[$br]*$length),floor($ratio[$br]*$length/3200)));
	}

	cfPingUpdate();
	if(!$debugRemoveContentHeader) header('Content-Type: audio/x-ms-wma'); // any audio format reencoded to wma
	header('Pragma: features="broadcast"');
	header('Pragma: timeout=3000');
	//if(!strpos($_SERVER['HTTP_USER_AGENT'],'pera')) header('Content-Type: audio/x-ms-wma'); else header('Content-Type: text/html');

	// Parse received Pragma
	$inHeaders=getallheaders();
	if(isset($inHeaders['Pragma'])){
		$inPragma=cfExplode2Levels(',','=',$inHeaders['Pragma']);
		cfDebugUpdateBuffer($inHeaders['Pragma']);
	}

	// Set audio quality
	if(!$bitrate || $bitrate=='original') $bitrate=64;
	if($bitrate==128) $audioArgs='-ab 128000 -ac 1';
	elseif ($bitrate==64) $audioArgs='-ab 64000 -ac 1';
	elseif ($bitrate==32) $audioArgs='-ab 32000 -ac 1';
	else $audioArgs='';

	// Generate base ASF header
	$cmdLine='"'.cfFfmpegExe().'" -y -i "'.$completeFilename.'" -t 2 -acodec wmav2 '.$audioArgs.' -f asf -';
	cfFfmpegLog($cmdLine,'WMA audio encoding');

	$s2=cfStreamProc($cmdLine,array('stdout'=>'return'));
	$_ENV['baseHeader']=substr($s2,0,ASF_HEADER_LENGTH);

	$cmdLine=str_replace('-t 2 ','',$cmdLine);
	cfDebugUpdateBuffer('Cmd line: '.$cmdLine);

	// Client only want ASF header
	if(isset($inHeaders['max-duration']) && $inHeaders['max-duration']=='0'){
		header('Content-Length: '.strlen($_ENV['baseHeader']));
		cfDebugUpdateBuffer('+++++++++++++++++++');
		die($_ENV['baseHeader']);
	}

	// Estimate reencoded file size and number of WMA packets
	//$ai=efGetAudioInfo($completeFilename,false,array('vbrParsing'=>1));
	list($estimatedFileSize, $estimatedBlocks)=estimatedReencodedSize($completeFilename,$bitrate);

	$_ENV['wmaPackets']=pack('v',$estimatedBlocks);
	$_ENV['wmaFileSize']=$estimatedFileSize;
	$_ENV['outputLength']=0;
	header('Content-Length: '.$estimatedFileSize);
	cfDebugUpdateBuffer('est file size : '.$estimatedFileSize);

	// Stream file
	cfStreamProc($cmdLine,array('stdout'=>'processASFHeader'));
	if($_ENV['outputLength']<$estimatedFileSize && connection_aborted()) {
		echo str_pad('',$estimatedFileSize-$_ENV['outputLength'],chr(0));
		cfDebugUpdateBuffer('PAD with '.($estimatedFileSize-$_ENV['outputLength']).' NULL chars');
	}
	if(connection_aborted()) cfDebugUpdateBuffer('Connection aborted'); else cfDebugUpdateBuffer('All streamed');
	exit;
}

/**
 * @desc Return a converted m3u or xml formated string from an existing m3u file
 *  - from m3u file :
 *		File names of original m3u file are replaced by extAccess formated URL
 *
 *		if $xmlOutput is true, output is XML formated so it can be red by flash audio player
 *		else output is formated like a normal m3u file.
 *
 * @param string $inFilename : complete file name of m3u file
 * @param string $bitrate : max output bitrate
 * @param boolean $xmlOutput : true if output is XML formated, false if m3u formated
 * @return string XML File
 */
function audioConvertM3U($m3uFilename, $bitrate, $xmlOutput){
	// Playlist header
	if($xmlOutput) $output='<?xml version="1.0" encoding="utf-8"?>'."\n".'<playlist volume="'.((cfUGetVar('audioVolume')!==false)?cfUGetVar('audioVolume'):'80').'">'."\n"; else $output="#EXTM3U\n";

	// Used to indicate a flash player playlist has been used and so only mp3 format is allowed (WA for mobiles where native format would be prefered over mp3)
	if(isset($_SESSION['activeResourceId'])) cfRSetVar('flashPlaylistUsed',$xmlOutput);

	$nb=0; // number of songs in playlist

	// Internal playlist (explorer/music resource) or temporary playlist (musicDB, play artist or album)
	if($m3uFilename=='*playlist*.m3u' || $m3uFilename=='*tmp*playlist*.m3u' || $m3uFilename=='*empty*playlist*.m3u'){

		// If filename = *playlist*.m3u, generates a m3u file pointing on cfRGetVar('playlist') or cfRGetVar('playlistShuffled') array of songs
		if($m3uFilename=='*tmp*playlist*.m3u') $playlist=cfRGetVar('playlistTmp'); // Temp (artist or album) playlist
		elseif($m3uFilename=='*empty*playlist*.m3u') $playlist=array(); // Empty playlist, used not to display XML error in player, called by url /audio/emptyPlaylist.xml
		else if(cfRGetVar('shufflePlaylist')) $playlist=cfRGetVar('playlistShuffled'); // Shuffled playlist
		else $playlist=cfRGetVar('playlist'); // Regular playlist

		foreach ($playlist as $song){
			// external ref (returned by flash player to browser every time a new song is played)
			if(!isset($song['externalRef'])) $song['externalRef']=cfBase64_url_encode(cfResourceRelativePath($song['completeFileName']));
			// resampling
			if($bitrate) $externalFileName=cfExtAudio(($song['completeFileName']), "audio/".$bitrate."/", 'absolute','b64');
			else $externalFileName=cfExtAudio(($song['completeFileName']),'','absolute','b64');
			// Create XML node
			if($xmlOutput) $output.='<audio externalRef="'.$song['externalRef'].'" songLabel="'.cfXMLEncodeProperty(cfUTF8Encode($song['label'],false,false)).'" file="'.$externalFileName."\" />\n";
			// Or M3U lines
			else $output.= '#EXTINF:'.$song['label']."\n".$externalFileName."\n";
			$nb++;
		}
		cfLog('extAccess.php: audio playlist generated from internal playlist. '.$nb.' records generated',LOG_DBG);
	}
	// Single mp3 file
	elseif($m3uFilename=='*singleMP3*.m3u'){

		// resampling
		if($bitrate) $externalFileName=cfExtAudio(cfRGetVar('singleMP3'), "audio/".$bitrate."/", 'absolute','b64');
		else $externalFileName=cfExtAudio(cfRGetVar('singleMP3'),'','absolute','b64');

		// label
		if(cfRGetVar('singleMP3Label')) $label=cfRGetVar('singleMP3Label');
		else $label=cfUTF8Encode(cfFileWithoutExtension(basename(cfRGetVar('singleMP3'))));

		if($xmlOutput) $output.='<audio songLabel="'.cfXMLEncodeProperty($label).'" file="'.$externalFileName."\" />\n";
		else $output.= '#EXTINF:'.cfUTF8Decode($label)."\n".$externalFileName."\n";

		cfLog('extAccess.php: single mp3 playlist generated',LOG_DBG);
	}
	// else (existing m3u file)
	else{
		if(($handle = fopen ($m3uFilename, "r"))){
			$label=false;
			while (!feof ($handle)) {
				$m3uFile = trim(fgets($handle, 4096));
				if(substr($m3uFile,0,8)=="#EXTINF:"){
					$label=substr($m3uFile,8);
					$label=substr($label,strpos($label,',')+1);
				}
				elseif($m3uFile && substr($m3uFile,0,1)!="#"){
					$m3uFile=str_replace('\\','/',$m3uFile);

					// If file name is an absolute path, use it
					if(substr($m3uFile,0,1)=='/' || substr($m3uFile,1,2)==':/'){
						// Add drive letter if filename starts with "/" under windows
						if(substr($m3uFile,0,1)=='/' && substr($m3uFile,1,1)!=='/' && WEnv::host()->isWindows()) $completeFilename=substr($m3uFilename,0,1).':'.$m3uFile;
						else $completeFilename=$m3uFile;
					}
					// else prepend playlist file's directory
					else
						$completeFilename=cfDirName($m3uFilename).'/'.$m3uFile;
					if(file_exists($completeFilename) && efFileType($completeFilename)=='audio'){
					if($bitrate) $externalFileName=cfExtAudio($completeFilename, 'audio/'.$bitrate.'/',(($xmlOutput)?'relative':'relative'));
					else $externalFileName=cfExtAudio(($completeFilename),'',(($xmlOutput)?'relative':'relative'));

					$externalRef=cfBase64_url_encode(cfResourceRelativePath($completeFilename));
					if($xmlOutput)
						$output.='<audio '.(($label)?'songLabel="'.cfXMLEncodeProperty(cfUTF8Encode($label,false,false)).'" ':'').'file="'.$externalFileName.'"  externalRef="'.$externalRef.'"'."/>\n";
					else
						$output.= '#EXTINF:'.$label."\n".cfGGetVar('hostName').$externalFileName."\n";
					$label=false;
					$nb++;
					}
				}
			}
			fclose ($handle);
		}
	cfLog('extAccess.php: audio playlist generated from existing playlist. '.$nb.' records added',LOG_DBG);
	}
	if($xmlOutput) $output.='</playlist>';

	cfDebugUpdateBuffer($output,0);

	return $output;
}

/**
 * @desc Send an empty XML playlist to flash XML audio player
 */
function audioSendEmptyXMLPlaylist(){
	WHeaders::noCache();
	WHeaders::contentType('xml');
	echo audioConvertM3U('*empty*playlist*.m3u',false,true);
	exit;
}

/**
 * @desc Check published audio access rights and send xml playlist and flash reencoded files to output
 *
 * @param array $uri: parsed URI ("previewImg"=>1 to generate a snapshot)
 */
function audioPublished($uri){
	require_once(INCLUDE_DIR.'explorerFunctions.php');
	global $debugRemoveContentHeader;

	if(cfMGetVar('debugAsync')) cfDebugSetBuffer('audio',array("TYPE"=>"published audio")+$uri);

	$id=$uri['dlToken'];
	$tokenList=efTokensRead();

	// If link is not published audio or streamed direct link, exit
	if(!(@$token=$tokenList[$id])) cfLog('invalid published video id ('.$id.')',LOG_DBG);
	if($token['type']!=='publishAudio' && ($token['type']!=='directLink' || @$token['streaming']!=='audio')) cfLog('invalid published audio id ('.$id.')',LOG_DBG);

	if(!isset($uri['file'])) die();

	$files=array();

	// Generate playlist from directory content
	if(is_dir($token['filename'])){
		foreach (cfGlob($token['filename'].'/*.*') as $cfn){
			if(efFileType($cfn)=='audio') $files[]=array('cfn'=>$cfn);
		}
	}
	// Generate playlist from list of files
	else{
		foreach (explode('|',$token['filename']) as $cfn){
			if(file_exists($cfn)) $files[]=array('cfn'=>$cfn);
		}
	}

	/**
	 * Playlist
	 */
	if($uri['file']=='playlist.xml'){
		// Increase number of downloads and save (except for directLinks for which downloads are counted on actual audio file download
		// in order not to have a negative offset of 1 on limited number of downloads)
		if($token['type']!=='directLink') {
			$tokenList[$id]['limitNb']--;
			efTokensWrite($tokenList);
		}

		// Get songs id3 info
		foreach ($files as $k=>$v) {
			$files[$k]['ai']=efGetAudioInfo($files[$k]['cfn']);
			$files[$k]['uri']=str_replace('playlist.xml',$k.'.mp3',$_SERVER['REQUEST_URI']);
		}

		header('Content-type: text/xml');
		require_once(INCLUDE_DIR.'fileInfoFunctions.php');
		$xmlPlaylist=fiArrayToPlaylist($files,'xml','URI');
		cfDebugUpdateBuffer($xmlPlaylist);
		die($xmlPlaylist);
	}
	/**
	 * Streamed file
	 */
	else{
		// Increase number of viewed times
		if($token['type']=='directLink') {
			$tokenList[$id]['limitNb']--;
			efTokensWrite($tokenList);
		}
	}

	/**
	 * Reencoded (or not) audio files
	 */
	$k=cfFileWithoutExtension($uri['file']);
	if(!isset($files[$k])) die();
	$cfn=$files[$k]['cfn'];

	cfDebugUpdateBuffer('Play file: '.$cfn);

	$bitrate=@$token['maxOutputBitRate'];

	if(!$debugRemoveContentHeader) header('Content-type: audio/mpeg');

	// Reencode to mp3 and send to ouptut
	if($bitrate!='original' || cfFileExtension($cfn)!='mp3'){
		// Audio quality
		if(!$bitrate || $bitrate=='original') $audioArgs='';
		elseif ($bitrate==32) $audioArgs='-ar 11025 -ab '.$bitrate.' -ac 1';
		else $audioArgs='-ar 44100 -ab '.$bitrate.' -ac 1';

		// ffmpeg command line
		$cmdLine='"'.cfFfmpegExe().'" -i "'.$completeFilename.'" '.$audioArgs.' -f mp3'/* ' -map_metadata :'*/.' -';
		cfFfmpegLog($cmdLine,'MP3 audio encoding');

		// Reencode
		cfStreamProc($cmdLine,array('stdout'=>'stdout'),array('speedLimit'=>0));
	}
	// Send raw mp3 file to output
	else {
		header('Content-length: '.filesize($cfn));
		echo file_get_contents($cfn);
	}
	exit;
}

/**
 * MP3 ID3 header processing (not used)
 *
 * @param string $data: in-data
 * @return string processed data
 */
function audioJoinedMP3Process($data){
	global $trackNumber; // Track number in playlist (0 for 1st track), used to differentiate 1st track from others
	global $totalPlaylistLength; // Total playlist length (ms)
	static $headerProcessed=false; // Set to true once file's header has been processed. Reset by setting $data===-1
	static $tmpData=''; // Temporary buffer, in case data length is not sufficient to be processed

	// Indicate we're switching to a new file
	if($data===-1) {
		$headerProcessed=false;
		return;
	}


	// Retreive audioJoinedMP3Process temporary buffer
	if($tmpData) $data=$tmpData.$data;
	$tmpData='';

	// If header has already been processed for this file, just dump data to output
	if($headerProcessed) return $data;

	$dataLength=strlen($data);

	// If there is not enough data to process ID3 header, store it and proceed
	if($dataLength<11){
		$tmpData=$data;
		return '';
	}

	// Get ID3 header
	$header=substr($data,0,10);
	$flag = 0;
	// Bytes 3 & 4: ID3 Version
	$version = ($majorVersion=ord($header[3])).'.'.ord($header[4]);

	// Byte 5: flag???
	$flag = ord($header[5]);

	// Compute complete tag data length
	$totalLength=($flag & 64)?-10:0;
	$nb=0;
	for($i=0;$i<4;$i++)	$bytes[$i] = ord($header[6+$i]);
	for($i=3;$i>=0;$i--) for($o=0;$o<7;$o++){
		if(($bytes[$i] & 1<<$o)==1<<$o) $totalLength+=1<<$nb;
		$nb++;
	}

	$headerProcessed=true;
	// 1st played track, add TLEN tag
	if($trackNumber==0){
		// Create (new) length tag
		$TLENTag="TLEN\0\0\0".chr(strlen((string)$totalPlaylistLength))."\0\0\0".$totalPlaylistLength;

		// Pack new total tag data length
		$totalLength+=strlen($TLENTag);
		$a=$totalLength%128;$totalLength=($totalLength-$a)/128;
		$b=$totalLength%128;$totalLength=($totalLength-$b)/128;
		$c=$totalLength;

		// New header
		$header=substr($data,0,7).chr($c).chr($b).chr($a);

		// We're done for this file
		$headerProcessed=true;

		return $data;
		// Send newly created track length tag
		return $header.$TLENTag.substr($data,10);
	}
	// Other tracks: remove header & ID3 tags
	else{
		// Header not yet entirely in buffer: continue
		if($dataLength<$totalLength){
			$tmpData=$data;
			return '';
		}
		// We're done for this file
		$headerProcessed=true;

		// Header entirely in $data: remove,
		return substr($data,$totalLength);
	}
}

/**
 * @desc Concatenate and stream content of playlist, with mp3 format
 *
 */
function audioStreamJoinedFilesPlaylist($m3uFilename){
	global $trackNumber; // Track number in playlist (0 for 1st track), used to differentiate 1st track from others
	global $totalPlaylistLength; // Total playlist length (ms)

	/**
	 * @desc Process ID3 header of joined mp3s files. Those files are reencoded by ffmpeg so header is quite simple and stable
	 * 		1st file of joined playlist: add TLEN (track length) ID3 tag
	 * 		other files: remove ID3 tag (all files should have same bitrate & encoding)
	 *
	 * @param mixed $data: data to be processed, -1 to indicate we're switching to a new file
	 */

	require_once(INCLUDE_DIR.'explorerFunctions.php');

	wSession_write_close();

	// Get output bitrate
	global $bitrate;
	// Device bitrate limit
	$bitrate=cfBitrateMin(cfBGetVar('maxAudioBitrate'),$bitrate);
	// Resource bitrate limit
	$bitrate=cfBitrateMin(cfRGetVar('maxOutputBitRate'),$bitrate);

	// Get speed limit
	$speedLimit=WEnv::user()->getVar('downloadSpeedLimit');

	// Retrieve playlist
	if($m3uFilename=='*tmp*playlist*.m3u') $playlist=cfRGetVar('playlistTmp'); // Temp (artist or album) playlist
	else if(cfRGetVar('shufflePlaylist')) $playlist=cfRGetVar('playlistShuffled'); // Shuffled playlist
	else $playlist=cfRGetVar('playlist'); // Regular playlist


	// Compute total file size (actually add all file size, so may be false if reencoding). Content-Length is mandatory for Android
	$totalPlaylistLength=0;
	$contentSize=0;

	// Check if there is a function for getting actual filenames out of external names (for musicDB resources)
	if(cfRGetVar('extAccessFunction')) $extAccessFunction=create_function('$completeFilename', cfRGetVar('extAccessFunction'));

	foreach ($playlist as $k=>$v) {
		$cfn=$v['completeFileName'];

		// Transform external filename into actual filename
		if(isset($extAccessFunction)){
			$cfn=$extAccessFunction($cfn);
			$playlist[$k]['completeFileName']=$cfn;
		}
		if(!cfFileRights($cfn)){
			unset($playlist[$k]);
			continue;
		}
		@$contentSize+=filesize($cfn);
		@$nbFiles++;
		//$ai=efGetAudioInfo($cfn,false);
		//$totalPlaylistLength+=(@$ai['length'])?$ai['length']:300000; /* arbitrary 300 sec length if not found*/
	}

	// Set ffmpeg audio chanels, bitrate and frequency parameters
	if(!$bitrate || $bitrate=='original') $audioArgs='-aq 41000 -ab 64 -ac 2';
	elseif($bitrate==64) $audioArgs='-ar 22050 -ab '.$bitrate.' -ac 2';
	elseif($bitrate==32) $audioArgs='-ar 22050 -ab '.$bitrate.' -ac 1';
	else $audioArgs='-ar 44100 -ab '.($bitrate/2).' -ac 2';


	// HTTP headers
	WHeaders::contentType('.'.cfBGetVar('audioFormat'));
	WHeaders::noCache();
	header('Content-Length:'.$contentSize);

	//WHeaders::contentRange($contentSize);

	// Guess requested track# from start offset
	list($from,$to)=WHeadersIn::getRange();
	$requestedTrackNumber=(floor($nbFiles*(int)$from/$contentSize));

	/**
	 * Stream audio files
	 */
	$trackNumber=0;
	foreach ($playlist as $v){
		$cfn=@$v['completeFileName'];

		if($trackNumber>=$requestedTrackNumber){
			// if browser uses mp3/m4v (android)
			if(cfBGetVar('audioFormat')=='m4v') $acodec='-f m4v -acodec libmp3lame';
			// else use plain mp3
			else  $acodec='-f mp3';

			// Reencode to mp3/m4v with ffmpeg (1st track full-speed to satisfy range-requests, other ones at music's speed)
			$cl='"'.cfFfmpegExe().'" '.((($trackNumber-$requestedTrackNumber)>0)?'-re':'').' -i "'.$cfn.'" '.$audioArgs.' '.$acodec.' -';
			cfFfmpegLog($cl,'MP3 audio encoding');

			// Debug
			cfDebugUpdateBuffer('Start streaming reencoded file: '.$cl);

			// Reencoding speed limit
			$streamOptions=array('speedLimit'=>$speedLimit);

			// Reencoding range (removed, works better without)
			//$streamOptions+=array('range'=>'auto');

			// Reencode
			//cfStreamProc($cl,array('stdout'=>'audioJoinedMP3Process'),$streamOptions,$s,$o);
			cfStreamProc($cl,array('stdout'=>'stdout'),$streamOptions,$s,$o);
		}
		$trackNumber++;
	}

	exit;
}

require(INCLUDE_DIR.'explorerFunctions.php');
require(INCLUDE_DIR.'mime_type.php');

if(!isset($_SESSION)) $_SESSION=array();

//Get URI and verify is starts with "/audio/"
$uri=$_SERVER['REQUEST_URI'];

$outputCodec='mp3';

// Published playlist
$parsedURI=cfParseExtURI();
if(isset($parsedURI['dlToken'])) audioPublished($parsedURI);

// Verify that URI starts by '/audio/'
if(substr($uri,0,7)!='/audio/') cfHTTPError(404);

// Special: empty XML playlist for flash player
if(cfCmpRight($uri,'/emptyPlaylist.xml')) audioSendEmptyXMLPlaylist();

$uri=stripcslashes(rawurldecode(substr($uri,7)));
cfLog('/ext/audio.php: file requested : '.$uri,LOG_DBG);



// verify that passed session ID exists, without starting session
$passedsessionId=substr($uri,2,strpos($uri,"/")-2); $found=false;
$uri=substr($uri,strlen($passedsessionId)+3);
if(!($handle = opendir(wSession_save_path()))) cfHTTPError(403);
while ((($file = @readdir($handle))!==false) && !($found)) {if(is_file(wSession_save_path()."/".$file) && $passedsessionId==$file)	 $found=true;}
closedir($handle);

// If passed session not found, display 404 and exit
if(!$found) cfHTTPError(404);

// START SESSION
wSession_id(substr($passedsessionId,strlen(SESSION_NAME_PREFIX)));
wSession_start();

// Set active resource id
if(substr($uri,0,6)!='resId.') cfHTTPError(404);
$resId=substr($uri,6,strpos($uri,'/')-6);
if(!is_numeric($resId) || !isset($_SESSION['res'][$resId]) || !is_array($_SESSION['res'][$resId]))  cfHTTPError(403);
$_SESSION['activeResourceId']=$resId;
$uri=substr($uri,strpos($uri,'/')+1);


// verifie that client's IP match with session's IP
if($_SERVER['REMOTE_ADDR']!=$_SESSION['accountIP'] && cfGGetVar('disableClientSessionIPControl')!='true' && !isset($_SERVER['HTTPS'])) {
	cfLog('extAccess.php: passed session id does not match with IP => 404',LOG_ERROR);
	cfHTTPError(403);
}
/*
// unencode filename
if(cfUTF8Encode(cfUTF8Decode($uri,true,false,false,2048),false,true)!=$uri) // if encoding-decoding doesn't work then string is already utf8-encoded
	$completeFilename=$uri;
else
	$completeFilename=cfUTF8Decode($uri,true,true,false,2048);
*/

// If a resampled audio file is asked for, get bitrate and file name
if(substr($uri,0,6)=='audio/' || (cfRGetVar('outputBitRate') && cfRGetVar('outputBitRate')!='original') || isset($otherReason)){
  	$resampledAudioFile=true;

  	if(substr($uri,0,6)=='audio/'){
		$uri=substr($uri,6);
		$bitrate=substr($uri,0,strpos($uri,"/"));
		$uri=substr($uri,strpos($uri,"/")+1);
  	}
  	else $bitrate=cfRGetVar('outputBitRate');

	// Reencoding for WMP mobile
	if(cfCmpRight($uri,WMP_MOBILE_AUDIO_EXT) && !cfRGetVar('flashPlaylistUsed')){
		$uri=substr($uri,0,-strlen(WMP_MOBILE_AUDIO_EXT)); // Remove fake extension
		$outputCodec='wmpMobile';
	}

	// Reencoding for ogg HTML5 player (OGG_AUDIO_EXT = convert.ogg)
	elseif(cfCmpRight($uri,OGG_AUDIO_EXT)){
		$uri=substr($uri,0,-strlen(OGG_AUDIO_EXT)); // Remove fake extension
		$outputCodec='ogg';
	}

	// Reencoding for mp3 HTML5 player (MP3_AUDIO_EXT = convert.mp3)
	elseif(cfCmpRight($uri,MP3_AUDIO_EXT)){
		$uri=substr($uri,0,-strlen(MP3_AUDIO_EXT)); // Remove fake extension
		$outputCodec='mp3';
	}

	// Reencoding to browser's audio format
	elseif(cfBGetVar('audioFormat') && !cfRGetVar('flashPlaylistUsed')) $outputCodec=cfBGetVar('audioFormat');

	// Or to mp3 (default for non-mobile browsers and for flash player)
	else $outputCodec='mp3';

	// No resampling if original bitrate on requested format
	if($bitrate=='original' && cfFileExtension(cfUnprotectFilename($uri))==$outputCodec) $resampledAudioFile=false;
	cfLog('extAccess.php: resampled audio file requested. Bitrate='.$bitrate,LOG_DBG);
}

// If no bitrate is passed in the url, look at resource-level
if(!$bitrate) $bitrate=cfRGetVar('outputBitRate');

// Replace resource-base-path-relative filename by actual filename (note: don't check file existance as it may be a playlist)
$completeFilename=cfFilenameResolve(cfUnprotectFilename($uri),false);

// DEBUG
if(cfMGetVar('debugAsync')) cfDebugSetBuffer('audio',array('CFN'=>$completeFilename,'$uri'=>$uri));

// Verify that requested bitrate is not superior to resource's max allowed bitrate
if(!is_numeric($bitrate)) $bitrate='original';
if(($maxBitrate=cfRGetVar('maxOutputBitRate')) && $maxBitrate!='original'){
	$bitrate=cfBitrateMin($bitrate,$maxBitrate);
	if($bitrate!=='original') {
		// If requested source audio was requested but a max bitrate is detected, guess format
		if(!$resampledAudioFile){
			if(cfBGetVar('wmp') && !cfBGetVar('flash')) $outputCodec='wmpMobile';
			elseif(cfBGetVar('audioFormat')) $outputCodec=cfBGetVar('audioFormat'); // So far mp3 or m4v
			else $outputCodec='mp3';
			$resampledAudioFile=true;
		}
	}
}

// Remove rnd part for playlists, singleMp3 and singleVideo
if(cfCmpRight($completeFilename,'*playlist*.m3u')
	|| cfCmpRight($completeFilename,'*playlist*.m3u.xml')
	|| cfCmpRight($completeFilename,'*singleMP3*.m3u')
	|| cfCmpRight($completeFilename,'*singleMP3*.m3u.xml')
	|| cfCmpRight($completeFilename,'*singleVideo*.xml')) {
	$completeFilename=substr($completeFilename,strpos($completeFilename,'*'));
}


// If file extension is ".m3u.xml", file is a playlist that need to be converted into XML format requested by FLASH audio player
if(cfFileExtension($completeFilename)=='xml'){
	$completeFilename=cfFileWithoutExtension($completeFilename);
	$xmlOutput=true;
}
else $xmlOutput=false;

// If file is a m3u file, convert it's content to URL links
// Warning: m3u now stands for any playlist format: it may be m3u, xml or joinedfiles (in this case, this is set by (mobile) browser capabilities)
if(cfFileExtension($completeFilename)=='m3u'){
	// If playlist is "joinedfiles", i.e. no playlist, just concatenated mp3 (except if it's an xml playlist, which may occur if android browser actually supports flash)
	if (cfBGetVar('playlistFormat')=='joinedfiles' && !$xmlOutput) audioStreamJoinedFilesPlaylist($completeFilename);

	$convertedM3UFile=audioConvertM3U($completeFilename, $bitrate, $xmlOutput); // generate playlist
	if($completeFilename=='*tmp*playlist*.m3u') $completeFilename='tmpplaylist.m3u';
	if($completeFilename=='*playlist*.m3u') $completeFilename='playlist.m3u';
	if($completeFilename=='*singleMP3*.m3u') $completeFilename='singleMP3.m3u';
	// restore xml file extension
	if($xmlOutput) $completeFilename.='.xml';
}
else{
	// Resource specific file source and rights function : bypass common file's existence and access rights
	// and (may) replace given file name by real file name
	if(cfRGetVar('extAccessFunction')) {

		$extAccessFunction = create_function('$completeFilename', cfRGetVar('extAccessFunction'));
		if(!($completeFilename=$extAccessFunction($completeFilename))) cfHTTPError(404);
	}
	// Check file's existence and access rights
	else{
		// If file doesn't exist, Display 404 and exit
		if(!file_exists($completeFilename))	cfHTTPError(404);

		// If file download not allowed, Display 404 and exit
		if(!cfFileRights($completeFilename,'download')) cfHTTPError(403);

		// If file is not an audio or video file, Display 404 and exit
		if($mt=mimeType(cfFileExtension($completeFilename))) $type=substr($mt,0,strpos($mt,"/")); else $type=false;
		if($type!='audio' && $type!='video') cfHTTPError(403);
	}
}


// Check if a file is non-mp3 with flash player (except ogg vorbis that may be requested by HTML5 player)
if(!$resampledAudioFile && cfFileExtension($completeFilename)!='m3u' && cfFileExtension($completeFilename)!='mp3'){
	if((cfRGetVar('audioPlayer')=='flash' && $outputCodec!='ogg') || (cfIsMobile() && cfFileExtension($completeFilename)!='aac')){
		$resampledAudioFile=true;
		$bitrate='original';
		if(cfBGetVar('wmpMobile') && !cfBGetVar('flash')) $outputCodec='wmpMobile'; else $outputCodec='mp3';
	}
}


// Browser caching
$cacheFile = (cfCmpLeft($completeFilename,'*playlist*') || cfCmpLeft($completeFilename,'*singleMP3*') || cfCmpLeft($completeFilename,'*singleVideo*'))?false:true;
if(cfBGetVar('name')=='Wii') $cacheFile=false;

// Check if a range request can be satified
$acceptRange=(($resampledAudioFile || cfFileExtension($completeFilename)=='m3u' || cfFileExtension($completeFilename)=='xml')?false:true);
if(cfBGetVar('OS')=='iOS') $acceptRange=true; // Force accept range to true on iOS devices

// Check if client did a range request
$clientHeaders=getallheaders();
$range=((isset($clientHeaders['Range']) && strtolower(substr($clientHeaders['Range'],0,6)=='bytes='))?trim(substr($clientHeaders['Range'],6)):$range=false);

/***********************************************************************************************************************************
 * Range request
 **********************************************************************************************************************************/
if($range && strpos($range,'-')!==false && $range!='0-' && !$resampledAudioFile/* iPhone resampled content: 416 is sent by cfRangeAccessibleOutputSend function */){
	// Reject range requests on resampled files or m3u or xml
	if(!$acceptRange){
		header('HTTP/1.1 416 Requested Range Not Satisfiable');
		$range=false;
	}
	else{
		$range=trim($range);
		$fileSize=cfFileSize($completeFilename);

		// From
		$from=substr($range,0,strpos($range,'-'));
		if(!strlen($from)) $from=0;
		if(!is_numeric($from) || $from>$fileSize) $rangeErr=true;

		// To
		$rto=substr($range,strpos($range,'-')+1);
		if(!strlen($rto)) {
			$to = $fileSize-1;
		}
		else $to=$rto;
		if($to+1>$fileSize || $to<$from) $rangeErr=true;
		$content_size = 1 + $to - $from;

		cfDebugUpdateBuffer('Range requested: '.$from.'-'.$to);

		// Incorrect range
		if(isset($rangeErr)){
			header('HTTP/1.1 416 Requested Range Not Satisfiable');
			$range=false;
		}
		else{
			header('HTTP/1.1 206 Partial Content');
			header("Content-Range: bytes $from-$to/$fileSize");
			$contentLength=$content_size;
			$range=array('from'=>$from,'to'=>1+$to);
		}

		// iPhone optimization (unused)
		if($from==0 && $to==1 && $resampledAudioFile && $outputCodec=='mp3') {
			cfDebugUpdateBuffer('Send "ID"');
			die('ID');
		}
	}
}
else $range=false;

// Set content-length if possible
if(!$range){
	if(cfFileExtension($completeFilename)=='m3u' || cfFileExtension($completeFilename)=='xml') $contentLength=strlen($convertedM3UFile);
	elseif (!$resampledAudioFile) $contentLength=cfFileSize($completeFilename);
}


/***********************************************************************************************************************************
 * HEADERS
 **********************************************************************************************************************************/

// Mime type
if ($resampledAudioFile && cfFileExtension($completeFilename)!='m3u' && cfFileExtension($completeFilename)!='xml') {
	if(($outputCodec=='mp3' || $outputCodec=='m4v' || $outputCodec=='ogg') && !$debugRemoveContentHeader) WHeaders::contentType($outputCodec);
}
// m3u or xml playlist or other audio format
else
	if(!$debugRemoveContentHeader) header('Content-Type: '.mimeType($completeFilename));

// Cache
if($cacheFile){
	header("Pragma:"); // Do not remove, required by IE in SSL mode
	header('Cache-Control: private');
	header('ETag: "'.md5($completeFilename).'"');
	header('Last-Modified: '.gmdate("D, d M Y H:i:s",@filemtime($completeFilename)).' GMT');
	header('Expires: '.gmdate("D, d M Y H:i:s",time()+3600*24*100).' GMT');
}
// No cache
else{
	if(cfGetBrowser()=='ie') header("Pragma:"); // Do not remove, required by IE in SSL mode
	else header("Pragma: no-cache");
	header("Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0");
	header("Expires: Thu, 01 Jan 1970 01:00:00 GMT");
}

// Other
header('X-Powered-By:');
header('Content-Description: file transfert');

// Set filename. Android & iPhone don't like this header...
if(cfBGetVar('OS')!=='Android' && cfBGetVar('OS')!=='iOS') header('Content-Disposition: inline; filename="'.cfUTF8Encode(basename($completeFilename),false,false).(($outputCodec=='wmpMobile')?WMP_MOBILE_AUDIO_EXT:'').'"');

if($acceptRange) header("Accept-Ranges: bytes"); else header("Accept-Ranges: none");

// Content-length
if(isset($contentLength)) header('Content-Length:'.$contentLength);
elseif (cfBGetVar('name')=='Android') header('Content-Length:'.filesize($completeFilename)); // Android and iPhone require a content length

cfLog('/ext/audio.php: file download initiated:'.$completeFilename,LOG_DBG);



/*
 * SEND FILE TO OUTPUT
 */

// XML or M3U playlist : send to output
if(cfFileExtension($completeFilename)=='m3u' || cfFileExtension($completeFilename)=='xml'){
	echo $convertedM3UFile;
}
else{
	// Close session so file download don't block other scripts from same user
	// Note : no possible log beyond this point
	wSession_write_close();

	// Log (history) access to file
	cfLogContentAccess($completeFilename);

	// Set download speed limit
	$speedLimit=WEnv::user()->getVar('downloadSpeedLimit');

	// Extend disconnection time for standalone media players
	if(cfBGetVar('standaloneMediaPlayer')) cfPingUpdate('force',600);

	// Resampled / converted mp3
	if($resampledAudioFile){
		set_time_limit(0);

		// Audio bitrate must be set in bps and not kbps for ogg encoding
		$oggFix=1;
		if($outputCodec=='ogg') $oggFix=1000;


		// Audio quality
		if(!$bitrate || $bitrate=='original') $audioArgs='-aq 0';
		elseif($bitrate==32) $audioArgs='-ar 22050 -ab '.$bitrate*$oggFix.' -ac 1';
		elseif($bitrate==64) $audioArgs='-ar 22050 -ab '.$bitrate*$oggFix.' -ac 2';
		else $audioArgs='-ar 44100 -ab '.($bitrate/2)*$oggFix.' -ac 2';

		// Stream resampled file
		if($outputCodec=='mp3' || $outputCodec=='m4v' || $outputCodec=='ogg'){
			// ID3 reencoding
			if($outputCodec=='mp3') $acodec='-f mp3';
			else if($outputCodec=='m4v') $acodec='-f m4v -acodec libmp3lame';
			else if($outputCodec=='ogg') $acodec='-f ogg -acodec libvorbis';
			$threads='-threads 4 ';
			$cmdLine='"'.cfFfmpegExe().'" -i "'.$completeFilename.'" '.$threads.$audioArgs.' '.$acodec/*.' -map_metadata :'*/.' -';

			cfFfmpegLog($cmdLine,$outputCodec.' audio encoding');

			// Debug
			cfDebugUpdateBuffer('Start streaming reencoded '.$outputCodec.': '.$cmdLine);

			// Reencoding speed limit
			$streamOptions=array('speedLimit'=>$speedLimit);

			// Reencoding range
			if($range) $streamOptions+=array('range'=>$range);

			// iPhone special: pre-generate file and send as range request-compatible
			if(cfBGetVar('OS')=='iOS'){
				/*
			    if($range) header('HTTP/1.1 206 Partial Content');
				//header("Content-Length: 999999999");
			    header("Accept-Ranges: bytes");
			    if($range){
				    list($from,$to)=explode('-',$range);
				    header("Content-Range: bytes $range/999999999");
				    header("Content-Length: ".(1+$to-$from));
			    }
			    */
				cfRangeAccessibleOutputStart();
				cfStreamProc($cmdLine,array('stdout'=>'stdout'));
				cfRangeAccessibleOutputSend();
				exit;
			}
			// Seems to be required by latest versions of flash player
			if(strpos(cfArrayItem(getallheaders(),'Referer'),'player.swf')) header("Content-Length: 999999999");

			// Reencode
			cfStreamProc($cmdLine,array('stdout'=>'stdout'),$streamOptions);

			// Debug
			cfDebugUpdateBuffer('DONE STREAMING MP3');
		}

		// Stream WMP mobile format resampled file
		elseif ($outputCodec=='wmpMobile')	reencodeWMA($completeFilename, $bitrate);
	}
	// Stream raw file
	else {
		// Debug
		cfDebugUpdateBuffer('Start streaming raw file, speedlimit:'.(int)$speedLimit);

		// Send file to output
		cfStreamFile($completeFilename,false,'auto','auto');

		cfDebugUpdateBuffer('Done');
	}
}
?>