<?php
/**
 * Concacts management and display functions
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2013 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 2.2.0N
 */


/**
 * @desc Load contacts from file
 */
function ctLoadAll(){
	$_ENV['contacts']=cfParse_ini_file(cfAppDataDir().FILE_CONTACTS,true,false);
	if(!$_ENV['contacts']) $_ENV['contacts']=array();
}

/**
 * @desc Save contacts from file
 */
function ctSaveAll(){
	cfWriteIniFile($_ENV['contacts'], cfAppDataDir().FILE_CONTACTS, true);
	cfServerSendCommand('refreshContacts');
}

/**
 * @desc Create a new contact
 *
 * @param string $accountName
 */
function ctAdd($accountName){
	$_ENV['contacts'][$accountName]=array("type"=>"reg");
}

/**
 * @desc Remove an existing contact
 *
 * @param string $accountName
 */
function ctRemove($accountName){
	unset($_ENV['contacts'][$accountName]);
}

/**
 * @desc Generate HTML code for contact tree
 *
 * @return string
 */
function ctTree(){
	//cfDbg($_ENV['contacts']);
	// No contacts frame
	if(count($_ENV['contacts'])==0){
		return '<center style="position:absolute;top:50%;width:98%;margin-top:-40px"><div class="extendedInfo" style="width:50%"><br><b>'.cfCaption('contactNo').'</b><br>'.outBt(cfCaption('genAdd'),outIcon('new'),'javascript:contacts.add()').'<br>&nbsp;</div></center>';
	}


	// Contacts tree top node
	$tree=array('ico'=>outIcon('resources'),'path'=>'top','label'=>'','sub'=>array());

	// Display
	foreach ($_ENV['contacts'] as $name=>$value){
		switch (@$value['status']) {
			// Online
			case 1:
				$ico=outIcon('contactOnline');
				$label='<b>'.cfUTF8Encode($name).' </b> ('.cfCaption('contactOnLine').')';
				$link=cfSiteLink($name);
				break;
			// Offline
			case 2:
				$ico=outIcon('contactOffline');
				$label='<b>'.cfUTF8Encode($name).' </b> ('.cfCaption('contactOffLine').')';
				$link=cfSiteLink($name);
				break;
			// Unknown
			default:
				$ico=outIcon('contactUnknown');
				$label='<b>'.cfUTF8Encode($name).' </b> (?)';
				$link=(isset($value['url']))?((strtolower(substr($value['url'],0,7))=='http://')?$value['url']:'http://'.$value['url']):(($value['type']=='reg')?cfSiteLink($name):false);
		}

		// Remove button icon
		$deleteControl=outImage(outIcon('cancelVS12'),false,'onclick="javascript:contacts.remove(\''.$name.'\');"'.outToolTip(cfCaption('genRemove')),'vertical-align:middle;left:0;position:absolute;cursor:pointer');

		// Create tree node
		$tree['sub'][$name]=array(
			'ico'=>$ico,
			'label'=>$label.((@$value['status']==2)?'':'&nbsp;&nbsp;&nbsp;'.outImageIcon('go')),
			'unfolded'=>true,
			'noClick'=>(@$value['status']==2),
			'outerLabel'=>$deleteControl,
			'tag'=>$link,
			'sub'=>array());
			// Subnode(s)
			if(0){
				$tree['sub'][$type]['sub'][$subNodeId]=array('ico'=>$icoPath,'label'=>cfUTF8Encode($ini['name']),'sub'=>array());
				if($filename==cfRGetVar('selectedResource')) $tree['sub'][$type]['sub'][$filename]['selected']=true;
			}
	}
	// Create treeView
	$treeView=new treeView('contactsTree');
	$treeView->phpTree=$tree;
	$treeView->selectFunction='contacts.select';

	$newContactButton=outBt(cfCaption('genAdd'),outIcon('new'),'javascript:contacts.add()',false,false,'style="position:absolute;top:0.1em;right:0.2em"');

	return $newContactButton.$treeView->HTML('',false,false);
}

/**
 * @desc Echo HTML & Javascript code for contacts treeview
 *
 */
function ctDisplayTree(){
?>
<script type="text/javascript">
var contacts={
	'add':function(){
		var db=new DialogBox({'icon':'<?php echo outIcon('new')?>','title':'<?php echo cfCaption('genNew')?>','text':'<?php echo cfCaption('contactNew')?>','input':'','buttons':{'ok':'<?php echo cfCaption('genAdd')?>','cancel':'<?php echo cfCaption('genCancel')?>'},'mask':true},'contacts.addProceed');
		db.show();
	},
	'addProceed':function(ret){
		if(ret.button=='ok') wl.asr.send({'addContact':ret.input});
	},
	'select':function(n){
		W.open(n.tag(),"<?php echo APPLICATION_NAME;?>","width=800,height=600,location=yes,resizable=yes,toolbar=yes,menubar=yes");
	},
	'remove':function(name,event){
		stopPropagation(event)
		wl.asr.send({'removeContact':name});
	}
}

</script>
<?php
	echo '<div id="contactsOuterDiv">';
	echo ctTree();
	echo '</div>';
}


/**
 * @esc Process async commands, update view
 *
 */
function ctProcessCommands(){
	if(isset($_POST['addContact'])){
		$name=$_POST['addContact'];
		if(strlen($name)<5 || strlen($name)>15) return;
		for($i=0;$i<strlen($name);$i++) if(strpos('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-.',substr($name,$i,1))===false) return;
		ctAdd($_POST['addContact']);
		ctSaveAll();
	}
	else if(isset($_POST['removeContact'])){
		if(isset($_ENV['contacts'][$_POST['removeContact']])){
			ctRemove($_POST['removeContact']);
			ctSaveAll();
		}
	}
	else return;

	// Reload page to apply changes
	echo cfAsyncHeader();
	echo cfAsyncJSaction('D.location.reload();');
	die(cfAsyncFooter());
}

ctLoadAll();
?>