<?php
/**
 * Logged user session initialisation
 *
 * load resource data, language data, protected dirs...
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/**
 * @desc Rotate log files if over a given size
 *
 * @param string $filename: log file name (file must be into log directory)
 * @param int $maxsize: maximum log size
 */
function ifRotateLog($filename, $maxsize){
	$cfn=cfAppLogDir().'/'.$filename;
	if(!is_file($cfn) || !$maxsize) return;

	if(filesize($cfn)>$maxsize){
		@unlink($cfn.'.1');
		@rename($cfn,$cfn.'.1');
	}
}

/**
 * Check that required PHP modules have been loaded. Echo errors within <errors> tags if any problem
 * @return true if all OK, false if any library has not been loaded
 *
 */
function ifCheckModules(){
	$lExt=(get_loaded_extensions());

	$modules=array(
		'curl'=>'curl_exec',
		'iconv'=>'iconv',
		'exif'=>'exif_read_data',
//		'gd2'=>'imagecolorallocatealpha',
		'mcrypt'=>'mcrypt_decrypt',
//		'mysql'=>'mysql_create_db',
		'openssl'=>'openssl_csr_new',
//		'ssh2'=>'ssh2_scp_send',
		'id3'=>'id3_get_tag'
	);
	$qt=array(
		'weezo'=>'customResizeJPG'
		);


      $module = 'weezo';
      /*
      $functions = get_extension_funcs($module);
      echo "<br>Weezo Functions available :<br>\n";
      foreach($functions as $func) echo $func."<br>\n";
		*/

	$modules+=$qt;

	$err='';
	foreach ($modules as $module=>$function) if(!array_search($module,$lExt) && !function_exists($function)){
		$err.='Error: missing ';
		$filename=(WEnv::config()->getVar('OSFamily')=='windows')?'php_'.$module.'.dll':$module.'.so';
		$err.=$filename.' module ';
		if(!file_exists(cfAppPHPDir().'/ext/'.$filename))
			$err.=' (file could not be found in "'.cfAppPHPDir().'/ext/'.$filename.'").';
		$err.="<br>\n";
	}
	if($err){
		echo '<errors>'.$err.'</errors>';
		return false;
	}
	return true;
}

function ifDiagnostic(){

	echo '<html><head><title>Diagnostic</title></head>';
	echo '<body style="font-family:Tahoma, Arial; font-size:10px">';
	echo '<b style="font-size:120%">Weezo dev diag</b><br>'; flush();


	// MVars test
	echo '<br><b>In-memory vars management: </b>';
	if(function_exists('customSetData')) echo ' custom';
	if(function_exists('apc_fetch')) echo ' APC';
	if(function_exists('apc_fetch')) echo ' XCache';
	echo '<br><br>';

	if(function_exists('customSetData')){
		echo 'Custom in-memory test: ';
		customSetData('foo',42);
		if(customGetData('foo')==42) echo "OK"; else echo "NOK";
	}
	if(function_exists('apc_get')){
		echo 'APC in-memory test: ';
		apt_set('foo',42);
		if(apt_get('foo')==42) echo "OK"; else echo "NOK";
	}
	if(function_exists('xcache_get')){
		echo 'XCache in-memory test: ';
		xcache_set('foo',42);
		if(xcache_get('foo')==42) echo "OK"; else echo "NOK";
	}
	echo '<br><br>';

	// Initialization deadlock check
	echo "<b>Initialization in progress: </b>".((cfMGetVar('weezoInitInProgress'))?'true':'false').'<br>';

	echo "<b>Server initialized: </b>".((cfMIssetVar('weezoGeneral'))?'true':'false').'<br>';
	echo '<br><b>Loaded Modules</b><br>';
	flush();

	// Check loaded modules
	ifCheckModules();


	// Check weezo functions
	$module = 'weezo';
	$functions = get_extension_funcs($module);
	echo "<br>Weezo Functions available :<br>\n";
	foreach($functions as $func) echo $func."<br>\n";


	echo '<br><b>PHP Info</b><br>';
	phpinfo();
	flush();
}

/**
 * @desc Kill CMD consol launched by Weezo (to set path env) as it's not anymore usefull
 * 		 This function is useless out of windows OS
 *
 */
function ifKillParentCmd(){
	if(!function_exists('customParentPIDs')) return;

	$myPID=getmypid();
	foreach(cfProcParentPID() as $child=>$tmp){
		list($parent,$exeName)=explode(':',$tmp);
		$proc[$child]=array('exe'=>$exeName,'parent'=>$parent);
		if($child==$myPID) $parentPID=$parent;
	}
	if($parent=@$proc[$proc[$parentPID]['parent']]['exe']=='cmd.exe') cfTerminateProcess($proc[$parentPID]['parent'],false);
}

/**
 * @desc Statistics reporting
 *
 */
function ifStatsReport(){
	// Stats class
	class stats{
		static $stats=array();

		function add($k,$v=null){
			if($v===null) $v=cfGGetVar($k);
			if($v===true) $v='true';
			elseif($v===false) $v='false';
			$this->stats[$k]=$v;
		}

		function get(){
			return $this->stats;
		}
	}
	$s=new stats();

	// General
	$s->add('theme');
	$s->add('serverPort');
	$s->add('webPagesCompressionLevel');
	$s->add('uploadMaxFilesize');
	$s->add('skin');
	$s->add('noPopup');
	$s->add('startWithWindows');
	$s->add('UPnPEnabled');
	$s->add('UPnPFailure');
	$s->add('UPnPFailure');
	$s->add('UPnPFailure');
	$s->add('loginMessage',((cfGGetVar('loginMessage'))?true:false));

	// user stats
	$userHint=$userPassword=$users=$userTheme=$userHidden=0;
	$userNbResources=0;
	foreach (cfMGetVar('weezoUsers') as $k=>$user) if(!@$user['invisible']){
		$users++;
		if(@$user['hidden']) $userHidden++;
		if(@$user['password']) $userPassword++;
		if(@$user['theme']) $userTheme++;
		$i=0;
		while (isset($user['resource'.$i])) $i++;
		$userNbResources+=$i;
	}
	$s->add('users',$users);
	$s->add('userHint',$userHint);
	$s->add('userPassword',$userPassword);
	$s->add('userTheme',$userTheme);
	$s->add('userHidden',$userHidden);
	$s->add('userNbResources',$userNbResources);

	// Resources stats
	$res=array();
	foreach (cfMGetVar('weezoResourcesList') as $id=>$filename){
		$r=cfMGetVar('weezoResData'.$id);
		@$res[$r['subType']]++;
	}
	foreach ($res as $subtype=>$nb) $s->add('res'.$subtype,$nb);

	// Send report to dev server
	cfSocketHTTPRequest('http://dev.weezo.net/report.php?config='+base64_encode(serialize($s->get())),3);
}

/**
 * @desc Return an array containing resource definition
 *
 * @param string $type
 * @param string $subType
 */
function ifResourceDefinition($type,$subType){
	$baseFile=false;
	$res=array('type'=>$type,'subType'=>$subType);
	$desriberFile=cfAppDocRoot().'/res/'.$type.'/'.$subType.'/describer.ini';

	// Base file
	if(file_exists($desriberFile)) {
		// Look for describer.ini file, and parse it
		$res['describer']=cfParse_ini_file($desriberFile);

		// Look for base file in describer
		if(isset($res['describer']['baseFile']) && file_exists(cfAppDocRoot().'/res/'.$res['type'].'/'.$res['subType'].'/'.$res['describer']['baseFile'])) $baseFile=$res['describer']['baseFile'];
	}
	// if base file not found in describer, set it to index.php
	if(!$baseFile) {
		if(file_exists(cfAppDocRoot().'/res/'.$res['type'].'/'.$res['subType'].'/index.php')) $baseFile='index.php'; else return false;
	} // If no base file found, exit
	$res['baseFile']=$baseFile;

	// Script path
	$res['resourceScriptPath']='/res/'.$res['type'].'/'.$res['subType'];

	// resource icons

	/**
	 * 32x32 icon
	 */
	if(isset($res['describer']['iconFile'])){
		// 1st look for png
		if(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/'.$res['describer']['iconFile'].'.png'))
			$res['resourceIcon']=$res['resourceScriptPath'].'/'.$res['describer']['iconFile'].'.png';
		// then for gif
		elseif (file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/'.$res['describer']['iconFile'].'.gif'))
			$res['resourceIcon']=$res['resourceScriptPath'].'/'.$res['describer']['iconFile'].'.gif';
		// then for jpg
		elseif (file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/'.$res['describer']['iconFile'].'.jpg'))
			$res['resourceIcon']=$res['resourceScriptPath'].'/'.$res['describer']['iconFile'].'.jpg';
	}
	// if 32x32 resource icon not yet found
	if(!isset($res['resourceIcon'])){
		// look for resourceIcon.png
		if(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/resourceIcon.png'))
			$res['resourceIcon']=$res['resourceScriptPath'].'/resourceIcon.png';
		// then look for resourceIcon.gif
		elseif(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/resourceIcon.gif'))
			$res['resourceIcon']=$res['resourceScriptPath'].'/resourceIcon.gif';
		// then look for resourceIcon.jpg
		elseif(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/resourceIcon.jpg'))
			$res['resourceIcon']=$res['resourceScriptPath'].'/resourceIcon.jpg';
		// then look in upperdir for resourceIcon.gif
		elseif(file_exists(cfAppDocRoot().'/res/'.$res['type'].'/resourceIcon.gif'))
			$res['resourceIcon']='/res/'.$res['type'].'/resourceIcon.gif';
		else $res['resourceIcon']=false;
	}

	/**
	 * 16x16 icon
	 */
	if(isset($res['describer']['iconSmallFile'])){
		// 1st look for png
		if(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/'.$res['describer']['iconSmallFile'].'.png'))
			$res['resourceIconSmall']=$res['resourceScriptPath'].'/'.$res['describer']['iconSmallFile'].'.png';
		// then for gif
		elseif (file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/'.$res['describer']['iconSmallFile'].'.gif'))
			$res['resourceIconSmall']=$res['resourceScriptPath'].'/'.$res['describer']['iconSmallFile'].'.gif';
		// then for jpg
		elseif (file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/'.$res['describer']['iconSmallFile'].'.jpg'))
			$res['resourceIconSmall']=$res['resourceScriptPath'].'/'.$res['describer']['iconSmallFile'].'.jpg';
	}
	// if 16x16 resource icon not yet found
	if(!isset($res['resourceIconSmall'])){
		// look for resourceIconSmall.png
		if(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/resourceIconSmall.png'))
			$res['resourceIconSmall']=$res['resourceScriptPath'].'/resourceIconSmall.png';
		// then look for resourceIconSmall.gif
		elseif(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/resourceIconSmall.gif'))
			$res['resourceIconSmall']=$res['resourceScriptPath'].'/resourceIconSmall.gif';
		// then look for resourceIconSmall.jpg
		elseif(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/resourceIconSmall.jpg'))
			$res['resourceIconSmall']=$res['resourceScriptPath'].'/resourceIconSmall.jpg';
		// then look in upperdir for resourceIconSmall.png
		elseif(file_exists(cfAppDocRoot().'/res/'.$res['type'].'/resourceIconSmall.png'))
			$res['resourceIconSmall']='/res/'.$res['type'].'/resourceIconSmall.png';
		// then look in upperdir for resourceIconSmall.gif
		elseif(file_exists(cfAppDocRoot().'/res/'.$res['type'].'/resourceIconSmall.gif'))
			$res['resourceIconSmall']='/res/'.$res['type'].'/resourceIconSmall.gif';
		else $res['resourceIconSmall']=false;
	}

	/**
	 * 16x16 user interface simple icon
	 */
	if(file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/uiResourceIconSmall.png')) $res['uiResourceIconSmall']=$res['resourceScriptPath'].'/uiResourceIconSmall.png';

	// resourceTypeLabel
	if(isset($res['describer'][cfGGetVar('language')]['name'])) $res['resourceTypeLabel']=$res['describer'][cfGGetVar('language')]['name'];
	elseif(isset($res['describer']['en']['name'])) $res['resourceTypeLabel']=$res['describer']['en']['name'];
	else $res['resourceTypeLabel']=cfUTF8Decode(cfCaption('res'.ucfirst($res['type'])));

	// resourcePreviewImage
	if(isset($res['describer']['previewImage'])) $res['resourcePreviewImage']=$res['resourceScriptPath'].'/'.$res['describer']['previewImage'];
	elseif (file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/preview.jpg')) $res['resourcePreviewImage']=$res['resourceScriptPath'].'/preview.jpg';
	else $res['resourcePreviewImage']=false;

	// resourceDescription
	if(cfCaptionExist($res['type'].'_'.$res['subType'].'-description')) $res['resourceDescription']=cfCaption($res['type'].'_'.$res['subType'].'-description');
	elseif(isset($res['describer'][cfGGetVar('language')]['description'])) $res['resourceDescription']=$res['describer'][cfGGetVar('language')]['description'];
	elseif(isset($res['describer']['en']['description'])) $res['resourceDescription']=$res['describer']['en']['description'];
	else $res['resourceDescription']=cfCaption('genNoDescription',false,false,false,true);

	// resourceConfigFile
	if(isset($res['describer']['configFile'])) $res['resourceConfigFile']=$res['describer']['configFile'];
	elseif (file_exists(cfAppDocRoot().$res['resourceScriptPath'].'/config.php')) $res['resourceConfigFile']='config.php';
	else $res['resourceConfigFile']=false;

	// Separate sessions as default option for misc extensions
	if(!isset($res['describer']['resourceStartsSession']) && $res['type']=='misc') $res['describer']['resourceStartsSession']=true;

	// resourceStartsSession: if true, close weezo session before begining of resource script
	if(isset($res['describer']['resourceStartsSession'])) $res['resourceStartsSession']=$res['describer']['resourceStartsSession'];
	else $res['resourceStartsSession']=false;

	// separateWindow: if true, open resource in a separate window
	if(isset($res['describer']['separateWindow'])) $res['separateWindow']=$res['describer']['separateWindow'];

	// UI config file: if set, open this script into UI (or remote admin) config window
	if(isset($res['describer']['configFile'])) $res['configFile']=$res['describer']['configFile'];

	// remote config file: if set, create a button in window title bar to go to this administration page (for administrator only)
	if(isset($res['describer']['remoteConfigFile'])) $res['remoteConfigFile']=$res['describer']['remoteConfigFile'];

	// singleInstance: if set, allow creation of a single instance of this resource
	if(isset($res['describer']['singleInstance'])) $res['singleInstance']=$res['describer']['singleInstance'];

	return $res;
}

/**
 * @desc Load resources definitions into weezoResourcesDefinitionstypesubtype memory arrays
 *
 */
function ifResourcesDefinitions(){
	$res=array();
	foreach (cfGlob(cfAppDocRoot().'/res/*') as $cfn0) if(is_dir($cfn0)){
		$type=basename($cfn0);
		foreach (cfGlob($cfn0.'/*') as $cfn) if(is_dir($cfn)){
			$subType=basename($cfn);
			// Load resource definition
			if($r=ifResourceDefinition($type,$subType))	{
				$res[$type][$subType]='weezoResourceDefinition'.strtolower($type).strtolower($subType);
				cfMSetVar($res[$type][$subType],$r);
			}
		}
	}
	cfMSetVar('weezoResourcesDefinitions',$res);
}

/**
 * @return bool : result
 * @param string : $resourceFileName : name of .res resource file located in data path
 * @desc load resource in array : from data path / $resourceFileName
 *
 * 	common resources parameters:
 *
 * version : version of file
 * resourceName : name of resource (filename without extension)
 * name : "label" of resource
 * type : type of resource
 * subType : sub type of resource
 * baseFile : main script file name (without path)
 * resourceScriptPath : path (from document root) to resource script(s)
 * resourceDataDir : path to resource data directory
 * resourceIcon : path (from document root) to resource's icon
 * resourceIconSmall : path (from document root) to 16x16 resource's icon
 * resourceJsLink : javascript link to main resource script
 * resourceBaseURL : link to main resource script
 * resourceTypeLabel : label of resource as specified in resource's desciber file or, if not found, resource's type label
 * resourcePreviewImage (optional) : path (from document root) to resource's preview image
 * resourceDescription (optional) : path (from document root) to resource's description (read from describer.ini file)
 * resourceStartsSession : true if resource doesn't need a wSession_start from security.php script
 * resourceConfigFile : (optional) resource configuration script name
*/
function ifLoadResource($resourceFileName,$res=false){
	// If file doesn't exist, (try to) remove from existing in-memory resources
	if(!file_exists(cfAppDataDir() . '/res/' . $resourceFileName)) return false;
	/*
		foreach (cfResourcesGetUser() as $id=>$data) if($data['resourceName'].'.res'==$resourceFileName) {cfMUnsetVar('weezoResData'.$data['id']); break;}
		return false;
	}
	*/
	// Parse resource file
	if(!$res) $res = cfParse_ini_file(cfAppDataDir().'/res/'.$resourceFileName);

	// Misc controls on parsed file content
	if(!$res || count($res)==0) return false;
	if(!isset($res['type']) || !isset($res['subType']) || !file_exists(cfAppDocRoot().'/res/'.$res['type'].'/'.$res['subType'])) return false;

	// Get type/subtype resource definition
	$res=array('definition'=>ifResourceDefinition($res['type'],$res['subType']))+$res;

	// resourceName (filename without extention)
	$res['resourceName']=cfFileWithoutExtension($resourceFileName);
	$res['resourceFilename']=$resourceFileName;

	// recompose subpaths of application path
	if(isset($res['path']) && substr($res['path'],0,8)=='*appDir*') $res['path']=cfAppDataRootDir().substr($res['path'],8);
	if(isset($res['bookmarksPath']) && substr($res['bookmarksPath'],0,8)=='*appDir*') $res['bookmarksPath']=cfAppDataRootDir().substr($res['bookmarksPath'],8);

	// resourceDataDir
	//if resource sub type dir doesn't exists, create it
	$dataDir=cfAppDataDir()."/res/".$res['type']."/".$res['subType'];
	if(!file_exists($dataDir)) {
		if(!file_exists(cfAppDataDir()."/res/".$res['type'])) @mkdir(cfAppDataDir()."/res/".$res['type']);
		@mkdir($dataDir);
	}
	$dataDir=cfAppDataDir()."/res/".$res['type']."/".$res['subType']."/".$res['resourceName'];
	//if(!file_exists($dataDir)) mkdir($dataDir); //if resource data dir doesn't exists, create it
	$res['resourceDataDir']=$dataDir;

	// Add timestamp of last update, used by resources to detect configuration updates
	$res['memUpdate']=time();

	// Save resource data in memory
	cfMSetVar('weezoResData'.$res['id'],$res);

	// Update resources list
	$rl=cfMGetVar('weezoResourcesList'); if(!$rl) $rl=array();
	$rl[$res['id']]=$resourceFileName;
	cfMSetVar('weezoResourcesList',$rl);

	return true;
}

/**
 * @desc Load all resources into memory
 *
 */
function ifLoadResources(){
	// Destroy previous resources data in memory
	if($r=cfMGetVar('weezoResourcesList')){
		foreach ($r as $id=>$filename) @cfMUnsetVar('weezoResData'.$id);
		unset($r);
		cfMUnsetVar('weezoResourcesList');
	}

	// Reload resources
	$retries=0;
	while($retries<5){ // Sometimes resources get lost. phenomenon has not been explained yet, so do a check at the end and retry up to 5 times
		$nb=0;
		foreach (cfGlob(cfAppDataDir().'/res/*.res') as $resourceCompleteFilename) {
			if(ifLoadResource(basename($resourceCompleteFilename))) $nb++;
		}
		// Check weezoResourcesList contains the right number of resources
		if($nb<>count(cfMGetVar('weezoResourcesList'))) $retries++; else break;
	}
}

/**
 * @desc load user ($_SESSION['user']) resources in $_SESSION['res'][resource number][field name] array
 * @return number of loaded resources
*/
function ifLoadUserResources() {
	function resIcon32(&$resourceData){
		if(@$resourceData['customIcon']){
			$cfn='/gfx/res/'.$resourceData['customIcon'].'.png';
			if(is_file(cfAppDocRoot().$cfn)) return $cfn;
		}
		return $resourceData['definition']['resourceIcon'];
	}

	$nbResources=0;
	$resourceRed=0;
	$activeItems=array(); // array containing resource's icons positions
	$addedResourcesFilenames=array(); // Array of added resources filenames, used to prevent double resource loading

	// Load all resources
	$resources=cfResourcesGetAll();

	// Standalone published resource
	if(isset($_SESSION['user']['publishToken'])){
		foreach ($resources as $id=>$resourceData) if(@$resourceData['publishToken']==$_SESSION['user']['publishToken']) {
			$_SESSION['res'][0]=array(
				'id'=>$id,
				'type'=>$resourceData['type'],
				'subType'=>$resourceData['subType'],
				'resourceJsLink'=>'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$resourceData['definition']['baseFile'].'?resId=0',
				'resourceBaseURL'=>'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$resourceData['definition']['baseFile'].'?resId=0',
				'resourceIcon'=>resIcon32($resourceData));
			return 1;
		}
		die('ifLoadUserResources error: published resource not found');
	}

	// Standalone direct resource access
	if(isset($_SESSION['user']['directResourceAccessToken'])){
		foreach ($resources as $id=>$resourceData) if(@$resourceData['directResourceAccessToken']==$_SESSION['user']['directResourceAccessToken']) {
			$_SESSION['res'][0]=array(
				'id'=>$id,
				'type'=>$resourceData['type'],
				'subType'=>$resourceData['subType'],
				'resourceJsLink'=>'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$resourceData['definition']['baseFile'].'?resId=0',
				'resourceBaseURL'=>'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$resourceData['definition']['baseFile'].'?resId=0',
				'resourceIcon'=>resIcon32($resourceData));
			return 1;
		}
		die('ifLoadUserResources error: direct resource access not found');
	}

	// Browse resources listed in user .usr file
	while(isset($_SESSION['user']['resource'.$resourceRed]['file'])){
		$temp=cfUGetVar('resource'.$resourceRed);
		// Check that resource .res file exists
		if(isset($temp['file']) && !isset($addedResourcesFilenames[$temp['file']])){
			if(isset($_SESSION['res'][$nbResources])) unset($_SESSION['res'][$nbResources]);
			// Search in all memory-loaded resources the one matching filename
			foreach($resources as $uniqueID=>$resourceData){
				// The right one...
				if(isset($resourceData['resourceName']) && $resourceData['resourceName'].'.res'==$temp['file']){

					// Last control: verify that browser supports this resource
					if(isset($resourceData['type']) && isset($resourceData['subType']) && stripos($resourceData['type'].'/'.$resourceData['subType'],cfBGetVar('unavailableResources'))===false){
						$opt=array();
						if(isset($resourceData['defaultWindowWidth'])) $opt['dWW']=$resourceData['defaultWindowWidth'];
						if(isset($resourceData['defaultWindowHeight'])) $opt['dWH']=$resourceData['defaultWindowHeight'];
						if(isset($resourceData['winInnerWidth'])) $opt['dIW']=$resourceData['winInnerWidth'];
						if(isset($resourceData['winInnerHeight'])) $opt['dIH']=$resourceData['winInnerHeight'];
						if(@$resourceData['maximizeAtFirstCall']) $opt['max']=1; // Maximize window when opened
						if(@$resourceData['separateWindow']||@$resourceData['definition']['separateWindow']) $opt['separateWindow']=1;
						if(@$_SESSION['user']['administrator'] && @$resourceData['definition']['remoteConfigFile'])
							$opt['remoteConfigFile']='/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$resourceData['definition']['remoteConfigFile'];


						$options='';foreach ($opt as $k=>$v) $options.=",'".$k."':'".$v."'"; $options='{'.substr($options,1)."}";


						// ADD RESOURCE to resources list
						// NOTE: resource creation also present in ifResetResourcesUsers()
						$_SESSION['res'][$nbResources]=array(
							// unique ID
							'id'=>$uniqueID,
							'type'=>$resourceData['type'],
							'subType'=>$resourceData['subType'],
							'resourceIcon'=>$resourceData['definition']['resourceIcon'],
							'resourceBaseURL'=>$resourceData['definition']['resourceScriptPath'].'/'.$resourceData['definition']['baseFile'].'?resId='.$nbResources,
							// resourceJsLink
							'resourceJsLink'=>'openResource(\''.$resourceData['definition']['resourceScriptPath'].'/'.$resourceData['definition']['baseFile'].'\',\''.$nbResources."','".
							str_replace('\\',"\\\\",str_replace("'","`",str_replace('"',"&quot;",cfUTF8Encode($resourceData['name']))))."','".
							resIcon32($resourceData)."',".$options.')');

						// Set background icons position according to iconPosition data
						$activeItems[$uniqueID]=array('name'=>'resIcon_'.$uniqueID, 'type'=>'icon', 'x'=>null, 'y'=>null, 'resourceFilename'=>$temp['file']);
						if(isset($temp['iconPosition'])) {
							@list($x,$y)=@explode('x', trim($temp['iconPosition']));
							if($x && $y && is_numeric($x) && is_numeric($y))
								$activeItems[$uniqueID]['x']=$x;
								$activeItems[$uniqueID]['y']=$y;
						}

						$addedResourcesFilenames[$temp['file']]=1;
						$nbResources++;
						break;
					}
				}
			}
		}
		$resourceRed++;
	}
	cfGSetVar('activeItems',$activeItems);
	if($nbResources==0) cfLog('login.php: No resource loaded !',LOG_ERROR); else cfLog('login.php: user configuration file loaded. '.$nbResources.' resource(s) loaded', LOG_DBG);
	return $nbResources;
}

/**
 * @desc load general.ini and language files into memory
 * 			Called at first script call after apache start-up, and every time general.ini file is modified
 *
 */
function ifLoadGeneral(){
	$dir=cfAppDataDir();
	$_ENV['weezoGeneral']=array();
	if(file_exists($dir.'/general.ini')) $_ENV['weezoGeneral']=cfParse_ini_file($dir."/general.ini", true);
	if(!count($_ENV['weezoGeneral'])) {
		echo "Internal Server Error : missing ini files";
		cfLog('Internal Server Error : missing ini files',LOG_INF);
		exit;
	}
	cfLog('initFunctions.php: general.ini loaded',LOG_DBG);

	// Store User Interface language in a separate var so it can be easily differentiated from browser language stored into 'language' G var
	$_ENV['weezoGeneral']['languageUI']=@$_ENV['weezoGeneral']['language'];

	// Add system directories to protected dir
	$_ENV['weezoGeneral']['protectedDirs']=array();
	if(isset($_ENV['weezoGeneral']['protectSensitiveFolders']) && $_ENV['weezoGeneral']['protectSensitiveFolders']  && strtolower($_ENV['weezoGeneral']['protectSensitiveFolders'])!='false'){
		if(getenv('systemroot')) $_ENV['weezoGeneral']['protectedDirs'][] = strtolower(strtr(getenv('systemroot'),"\\","/"));
		$_ENV['weezoGeneral']['protectedDirs'][] = strtolower(substr($_SERVER['DOCUMENT_ROOT'],0,strlen($_SERVER['DOCUMENT_ROOT'])));
		$_ENV['weezoGeneral']['protectedDirs'][] = strtolower(substr($_SERVER['DOCUMENT_ROOT'],0,strlen($_SERVER['DOCUMENT_ROOT'])-3)."data");
		$_ENV['weezoGeneral']['protectedDirs'][] = strtolower(substr($_SERVER['DOCUMENT_ROOT'],0,strlen($_SERVER['DOCUMENT_ROOT'])-3)."bin");
		$_ENV['weezoGeneral']['protectedDirs'][] = strtolower(substr($_SERVER['DOCUMENT_ROOT'],0,strlen($_SERVER['DOCUMENT_ROOT'])-3)."Apache");
		$_ENV['weezoGeneral']['protectedDirs'][] = strtolower(substr($_SERVER['DOCUMENT_ROOT'],0,strlen($_SERVER['DOCUMENT_ROOT'])-3)."PHP");
	}

	// Compatibility with V<2.0: transform [userX] arrays into users comma separated list
	$i=0;
	if(!@$_ENV['weezoGeneral']['users']) while (isset($_ENV['weezoGeneral']['user'.$i]['file'])){
		$files[]=$_ENV['weezoGeneral']['user'.$i]['file'];
		unset($_ENV['weezoGeneral']['user'.$i]);
		$i++;
	}
	if($i) $_ENV['weezoGeneral']['users']=implode(',',$files);


	// Store data in memory
	cfMSetVar('weezoGeneral',$_ENV['weezoGeneral']);

	// Store separately debugAsync value for performances purposes
	cfMSetVar('debugAsync',@$_ENV['weezoGeneral']['debugAsync']);

	// Set system codepage (used for utf8 encoding/decoding)
	if(!isset($_ENV['weezoGeneral']['systemCodepage'])) $_ENV['weezoGeneral']['systemCodepage']='1252';
	cfMSetVar('systemCodepage','windows-'.$_ENV['weezoGeneral']['systemCodepage']);
}

/**
 * @desc load .usr GROUP files into memory
 * 		 Called at first script call after apache start-up, and every time an user file is modified
 *
 */
function ifLoadUsers(){
	$files=array();
	$adminFound=false;
	$usersOrder='';

	// Get a filename=>id resources array
	$resourcesList=cfArraySwapKeysValues(cfMGetVar('weezoResourcesList'));

	// New users storage method (V2.0): get ordered users list from "users" var in general.ini. List of users filenames, comma separated
	if(cfGGetVar('users')) foreach (explode(',',cfGGetVar('users')) as $filename) $files[]=trim($filename);

	// Add all (unordered) user files from group directory
	foreach (cfGlob(cfAppDataDir().'/groups/*.usr') as $cfn) if(!in_array(basename($cfn),$files)) $files[]=basename($cfn);

	// Get default user data
	$defaultData=@cfParse_ini_file(cfAppBinDir().'/default/user.ini');
	if(!$defaultData) $defaultData=array();

	// Load users from files
	foreach ($files as $filename)  if(file_exists(cfAppDataDir().'/groups/'.$filename)) {
		$usr=cfParse_ini_file(cfAppDataDir().'/groups/'.$filename);

		if(!isset($usr['id'])) continue;

		// Set default data
		foreach ($defaultData as $k=>$v) if(!isset($usr[$k])) $usr[$k]=$v;

		// Verify that all resources exists and are not affected twice to this user
		$inexistingFound=0;
		$addedResourcesFilenames=array();
		for($i=0;isset($usr['resource'.$i]);$i++){
			// If an inexisting resource is found, or resource is already added
			if(!isset($resourcesList[$usr['resource'.$i]['file']]) || isset($addedResourcesFilenames[$usr['resource'.$i]['file']])){
				$inexistingFound=true;
				// move all following resources one step down
				for($o=$i+1;isset($usr['resource'.$o]);$o++)	$usr['resource'.($o-1)]=$usr['resource'.$o];
				unset($usr['resource'.($o-1)]);
			}
			if(isset($usr['resource'.$i])) $addedResourcesFilenames[$usr['resource'.$i]['file']]=1;
		}
		$usr['configFilename']=$filename;
		$users[$usr['id']]=$usr;
		if(@$usr['administrator']) $adminFound=true;
		$usersOrder.=(($usersOrder)?',':'').$filename;

		// If an inexisting resource has been removed, commit to (user) file
		if($inexistingFound) cfWriteIniFile($usr,cfAppDataDir().'/groups/'.$filename);
	}

	// If no administrator found and application already configured, regen an admin
	if(!$adminFound && cfGGetVar('applicationConfigured')){
		$admin=new WUserConfig();
		$admin->setVar('name',cfUTF8Decode(cfCaption('userAdministrator')));
		$admin->setVar('administrator',true);
		$admin->setVar('authenticationMethod','password');
		$admin->setVar('password','foo');
		$admin->save();
		$usersOrder.=(($usersOrder)?',':'').$admin->filename();
	}

	// Add user used for standalone resource connections
	$users['standalone']=array(
		'id'=>'standalone',
		'name'=>cfUTF8Decode(cfCaption('publishResourceLabel')),
		'invisible'=>true,
		'administrator'=>false,
		'activated'=>true,
		'accountType'=>'anonymous',
		'icon'=>'share.png',
		'authenticationMethod'=>'publishToken'
	);



	// Store data in memory
	cfMSetVar('weezoUsers',$users);

	// Modified list (upgrade from V1.3.0X to V2) or problem: update general.ini
	if($usersOrder!=cfGGetVar('users')) cfGUpdateVar('users',$usersOrder);
}

/**
 * @Desc Load all subUsers into memory from .subuser files located into data/users
 * 		 Note: user presence into a group is not checked
 */
function ifLoadSubUsers(){
	foreach (cfGlob(cfAppDataDir().'/users/*.subuser') as $cfn){
		$id=cfFileWithoutExtension(basename($cfn));
		if(preg_match('/^[a-z0-9]{8,20}$/i',$id)){
			$data=cfParse_ini_file($cfn);
			cfMSetVar('subUser'.$id,$data);
		}
	}
}

/**
 * @desc load theme data into memory, from data of themes/theme_name/menu.php
 *
 */
function ifLoadThemes(){
	$themes=array();
	$dir=cfAppDocRoot().'/themes';

	$dirs=cfRecurseBrowse($dir,'return','');

	// Browse www/themes subdirs for theme files (theme.css)
	foreach ($dirs as $completeFileName){
		if(is_dir($completeFileName) && file_exists($completeFileName.'/theme.css')){
			$file=basename($completeFileName);
			$baseDir=basename(dirname($completeFileName));
			// Mobile themes correction
			if($baseDir=='mobile') {
				$file='mobile/'.$file;
				$themes[$file]=array();
				$themes[$file]['private']=true;
			}
			// Skins are processed separately
			elseif($baseDir=='skins') continue;
			else $themes[$file]=array();


			// Set default values
			$themes[$file]['output']='normal';
			$themes[$file]['frames']=true;
			$themes[$file]['screenSize']='monitor';
			$themes[$file]['winEmptyHTML']=false;
			$themes[$file]['menuFrame']=array('position'=>'no', 'size'=>'0', 'scrolling'=>'no');
			$themes[$file]['mainFrame']=array('extraHeader'=>false, 'bodyExtraHTML'=>false,'winBorderWidth'=>0,'winBorderHeight'=>0);

			// Set theme main properties (use frames, screenSize, output type)
			if(file_exists($completeFileName.'/menu.php')){
				$theme=array();
				require($completeFileName.'/menu.php');
				foreach ($theme as $key=>$value) {
					if(!is_array($value)) $themes[$file][$key]=$value;
					else foreach ($value as $key2=>$value2) {
						$themes[$file][$key][$key2]=$value2;
					}
				}
			}
			$themes[$file]['path']='/themes/'.$file;
			$themes[$file]['dir']=cfAppDocRoot().'/themes/'.$file;
		}

		// Default skin
		$file='skins/default';
		$completeFileName=$dir.'/'.$file;
		$themes[$file]=array();

		$themes[$file]['path']='/themes/'.$file;
		$themes[$file]['dir']=cfAppDocRoot().'/themes/'.$file;
		$themes[$file]['private']=true;

		// OSX skin
		$file='skins/osx';
		$completeFileName=$dir.'/'.$file;
		$themes[$file]=array();

		$themes[$file]['path']='/themes/'.$file;
		$themes[$file]['dir']=cfAppDocRoot().'/themes/'.$file;
		$themes[$file]['private']=true;

		// Aero skin
		$file='skins/aero';
		$completeFileName=$dir.'/'.$file;
		$themes[$file]=array();

		$themes[$file]['path']='/themes/'.$file;
		$themes[$file]['dir']=cfAppDocRoot().'/themes/'.$file;
		$themes[$file]['private']=true;
	}

	// Savec into memory
	cfMSetVar('weezoThemes',$themes);
}

/**
 * @desc Create an array caching icons locations, uset by outIcon function
 *
 */
function ifLoadIcons(){
	$weezoIcons=array();
	// Get all icons located into /gfx and /gfx/fi
	$icons=array_merge(
		glob(cfAppDocRoot().'/gfx/*.{gif,png}',GLOB_BRACE),
		glob(cfAppDocRoot().'/gfx/fi/med/*.{gif,png}',GLOB_BRACE),
		glob(cfAppDocRoot().'/gfx/fi/big/*.{gif,png}',GLOB_BRACE),
		glob(cfAppDocRoot().'/gfx/browsers/*.{gif,png}',GLOB_BRACE),
		glob(cfAppDocRoot().'/gfx/fi/*.{gif,png}',GLOB_BRACE));
	// Copy icons list into a iconName=>iconPath array
	foreach ($icons as $key=>$value) $ref[cfFileWithoutExtension(substr($value,strlen(cfAppDocRoot())+5))]=substr($value,strlen(cfAppDocRoot()));
	foreach ($ref as $iconName=>$value) if(isset($icons[$iconName.'.png']) && isset($icons[$iconName.'.gif'])) $ref[$iconName]=$iconName.'.png';
	unset($icons);

	// Get all icons located into themes directories
	$themes=cfMGetVar('weezoThemes');

	foreach ($themes as $themeName=>$themeValue){
		$tIcons[$themeName]=array_merge(
			glob(cfAppDocRoot().'/themes/'.$themeName.'/*.{gif,png}',GLOB_BRACE),
			glob(cfAppDocRoot().'/themes/'.$themeName.'/fi/*.{gif,png}',GLOB_BRACE),
			glob(cfAppDocRoot().'/themes/'.$themeName.'/fi/med/*.{gif,png}',GLOB_BRACE));

		// Copy icons list into a iconName=>iconPath array
		$tmp=array();
		foreach ($tIcons[$themeName] as $key=>$value) $tmp[cfFileWithoutExtension(substr($value,strlen(cfAppDocRoot().'/themes/'.$themeName.'/')))]=substr($value,strlen(cfAppDocRoot()));
		foreach ($tmp as $iconName=>$value) if(isset($tIcons[$themeName][$iconName.'.png']) && isset($tIcons[$themeName][$iconName.'.gif'])) $tmp[$iconName]=$iconName.'.png'; // Prefer png over gif
		$tIcons[$themeName]=$tmp;
		$weezoIcons[$themeName]=$tmp;
	}

	// set icons per theme
	foreach ($ref as $iconName=>$value){
		foreach ($themes as $themeName=>$themeValue){
			if(isset($tIcons[$themeName][$iconName]) && substr($tIcons[$themeName][$iconName],-3)=='png')
				$weezoIcons[$themeName][$iconName]='/themes/'.$themeName.'/'.$iconName.'.png';
			elseif(isset($tIcons[$themeName][$iconName]) && substr($tIcons[$themeName][$iconName],-3)=='gif')
				$weezoIcons[$themeName][$iconName]='/themes/'.$themeName.'/'.$iconName.'.gif';
			else
				$weezoIcons[$themeName][$iconName]=$ref[$iconName];
		}
	}

	// Replace iPhone & Wii icons by mobile icons (but keep device specific icons)
	/*
	foreach ($weezoIcons['mobile'] as $iconName=>$iconPath){
		if(!isset($weezoIcons['iPhone'][$iconName]) || substr($weezoIcons['iPhone'][$iconName],0,4)=='/gfx') $weezoIcons['iPhone'][$iconName]=$iconPath;
		if(!isset($weezoIcons['wiizo'][$iconName]) || substr($weezoIcons['wiizo'][$iconName],0,4)=='/gfx') $weezoIcons['wiizo'][$iconName]=$iconPath;
	}
	*/
	cfMSetVar('weezoIcons',$weezoIcons);


	// Build an array of per file extension icons
	$extIcons=array();
	foreach (cfGlob(cfAppDocRoot().'/gfx/ext/*.png') as $cfn) $extIcons[basename($cfn)]=1;
	cfMSetVar('extensionIcons',$extIcons);

	// Build CSS sprites map
	cfMSetVar('cssSprites',@unserialize(file_get_contents(INCLUDE_DIR.'sprites.php')));
}

/**
 * @desc Load browsers capabilities from /data/browsers.xlm file
 *
 */
function ifLoadBrowsers(){
	// Set default values (see /bin/default/browsers.txt for explanations)
	$default=array(
		'name'=>'unknown',
		'type'=>'computer',
		'output'=>'html',
		'async'=>true,
		'asyncHTTPStreaming'=>true,
		'frames'=>true,
		'screenSize'=>'monitor',
		'asyncRequest'=>'auto',
		'fastAnim'=>true,
		'flash'=>true,
		'wmp'=>true,
		'qt'=>true,
		'qtReenc'=>true,
		'externalM3U'=>true,
		'playlistFormat'=>'m3u',
		'vlc'=>true,
		'divx'=>true,
		'm3u'=>true,
		//'standaloneMediaPlayer'=>false,
		'png'=>true,
		'theme'=>false,
		'mobileTheme'=>false,
		'clientPingInterval'=>8,
		'drag'=>true,
		'imgFadeSteps'=>5,
		'download'=>true,
		'upload'=>true,
		'videotag'=>true,
		'audiotag'=>true
		);

	// Read browsers definition file
	$tmp=@cfParse_ini_file(cfAppBinDir().'/default/browsers.txt',true,false);
	$browsers=array();

	foreach ($tmp as $k=>$v){
		// Set default values
		$tmpb=$default;
		// If browser inherits from another one, get properties from inherited one (1st check if
		if(isset($v['inherit']) && isset($browsers[$v['inherit']])) {
			foreach ($browsers[$v['inherit']] as $k2=>$v2) {
				if($k2!='signatureExcluded') $tmpb[$k2]=$v2;
			}
		}
		elseif(isset($v['inherit']) && isset($tmp[$v['inherit']])) {
			foreach ($tmp[$v['inherit']] as $k2=>$v2) {
				if($k2!='signatureExcluded') $tmpb[$k2]=$v2;
			}
		}
		// Set properties
		foreach ($v as $k2=>$v2) $tmpb[$k2]=$v2;
		$browsers[$k]=$tmpb;
	}

	// Add a default browser that will match everything
	$browsers['Default']=$tmpb; $browsers['Default']['signature']='.';

	cfMSetVar('weezoBrowsers',$browsers);
}

/**
 * @desc Load weezo.net account registration info
 *
 */
function ifLoadRegInfo(){
	 cfMSetVar('weezoRegInfo',(file_exists(cfAppDataDir().'/reg.ini'))?cfParse_ini_file(cfAppDataDir().'/reg.ini'):false);
}

/**
 * @desc Load sticky notes into weezoSNotes memory array
 *
 */
function ifLoadSNotes(){
	$maxId=0;
	// Load from file
	$weezoSNotes=(file_exists(cfAppDataDir().'/stickyNotes.ini'))?cfParse_ini_file(cfAppDataDir().'/stickyNotes.ini'):array();
	// Check values
	foreach ($weezoSNotes as $id=>$sNote) if(is_array($sNote)){
		if(!is_numeric($id) || !isset($sNote['text'])){unset($weezoSNotes[$id]);continue;}
		$maxId=max($maxId,$id);
		if(!isset($sNote['x'])) $weezoSNotes[$id]['x']=rand(0,1);
		if(!isset($sNote['y'])) $weezoSNotes[$id]['y']=rand(0,1);
		if(!isset($sNote['color'])) $weezoSNotes[$id]['color']='EE99B7';
		if(!isset($sNote['author'])) $weezoSNotes[$id]['author']='---';
		if(!isset($sNote['recipient'])) $weezoSNotes[$id]['recipient']='all';
	}
	if(!isset($weezoSNotes['maxId'])) $weezoSNotes['maxId']=$maxId;
	cfMSetVar('weezoSNotes',$weezoSNotes);
}

/**
 * @desc Load constants into memory from bin/default/constants.txt
 *
 */
function ifLoadConstants(){
	foreach (cfParse_ini_file(cfAppBinDir().'/default/constants.txt') as $key=>$value) cfMSetVar('__CONSTANT__'.$key,$value);
}

/**
 * @desc Create if needed files info cache database
 * @return bool result
 *
 */
function ifSetFilesCacheDB(){
	require_once(INCLUDE_DIR.'databaseFunctions.php');
	if(cfGGetVar('cacheFilesData') && !file_exists(W_FILES_DATABASE_FILENAME)){
		$db=dbOpenDB(W_FILES_DATABASE_FILENAME);
		if(!$db) return false;
		dbExec('CREATE TABLE dirs (id INTEGER PRIMARY KEY ASC, path KEY)',$db);
		dbExec('CREATE TABLE files (dirId KEY ASC, filename TEXT KEY, mtime INTEGER, data TEXT, UNIQUE(dirId,filename))',$db);
		dbClose($db);
	}
	return true;
}

/**
 * @desc Reset resources / users associations for all opened sessions
 * 		Called on new resource association, resource deletion, user creation/deletion/activation/deactivation
 *
 * @param bool $resetUsers: true to reload in-memory users
 * @param bool $resetResources: true to reload in-memory resources
 */
function ifResetResourcesUsers($resetUsers=false,$resetResources=false){
	$debug=false;

	// Reset in-memory resources from files
	if($resetResources) ifLoadResources();
	$resources=cfResourcesGetAll();

	// Reset in-memory users from files
	if($resetUsers) ifLoadUsers();
	$users=cfMGetVar('weezoUsers');

	// Stop current session so session file can be read
	wSession_write_close();

	/**
	 * Foreach opened session
	 */
	foreach (glob(wSession_save_path().'/sess_*') as $completeFilename){
		// Get session data
		$processedSess=@unserialize(file_get_contents($completeFilename));

		// If it's an user session
		if(isset($processedSess['user']['name']) && isset($processedSess['accountIP']) && isset($processedSess['connectionTime'])){

			// Get session's user actual config
			$userConfig=@$users[$processedSess['userId']];

			// Check that user is still existing / activated
			if(!$userConfig // user not defined anymore
			|| !@$userConfig['activated'] // user not activated anymore
			|| !cfIPFilterCheck($processedSess,$processedSess['accountIP'])) // user's ip not allowed anymore for this group (note: general-defined IP ban/access is controled at every user's request so no need to recheck here)
			{
				// Destroy user file
				@unlink($completeFilename);

				// Proceed with next user file
				continue;
			}

			// Used to see if an user's data must be saved
			$processedSessModified=false;

			// Verify currently bound resources are still existing and bound to user
			$maxSessShortID=0;
			if($processedSess['res']) foreach ($processedSess['res'] as $sessShortID=>$sessData){
				// If resource doesnt exist anymore, remove
				if(!isset($resources[$sessData['id']])) {
					unset($processedSess['res'][$sessShortID]);
					$processedSessModified=true;
					continue;
				}
				// Check if resource is still bound to this user
				$nb=0; $found=false;
				while (isset($userConfig['resource'.$nb])) {
					if($userConfig['resource'.$nb]['file']==$resources[$sessData['id']]['resourceName'].'.res') {$found=true;break;}
					$nb++;
				}
				// If this resource has not been found for this user, remove
				if(!$found){
					unset($processedSess['res'][$sessShortID]);
					$processedSessModified=true;
					continue;
				}
				if($sessShortID>$maxSessShortID) $maxSessShortID=$sessShortID;
			}

			// Check if new resources are still bound to this user
			// Browse all user's resources
			$nb=0;
			while (isset($userConfig['resource'.$nb])) {
				$rfn=cfFileWithoutExtension($userConfig['resource'.$nb]['file']);
				$found=false;
				// Browse all bound resources to see if already bound
				foreach ($processedSess['res'] as $sessShortID=>$sessData){
					if($resources[$sessData['id']]['resourceName']==$rfn){$found=true;break;}
				}
				// If resource not already bound to user, add it
				if(!$found){
					$maxSessShortID++;
					foreach ($resources as $uniqueID=>$resourceData) if($resourceData['resourceName']==$rfn) {$uniqueID=$resourceData['id'];break;}
					$processedSess['res'][$maxSessShortID]=array(
						// unique ID
						'id'=>$uniqueID,
						'type'=>$resourceData['definition']['type'],
						'subType'=>$resourceData['definition']['subType'],
						'resourceIcon'=>$resourceData['definition']['resourceIcon'],
						'resourceBaseURL'=>$resourceData['definition']['resourceScriptPath'].'/'.$resourceData['definition']['baseFile'].'?resId='.$maxSessShortID,
						// resourceJsLink
						'resourceJsLink'=>'openResource(\''.$resourceData['definition']['resourceScriptPath'].'/'.$resourceData['definition']['baseFile'].'\',\''.$maxSessShortID."','".
						str_replace('\\',"\\\\",str_replace("'","`",str_replace('"',"&quot;",cfUTF8Encode($resourceData['name']))))."','".
						$resourceData['definition']['resourceIcon']."',".
						((isset($resourceData['defaultWindowHeight']))?$resourceData['defaultWindowHeight']:'0').",".
						((isset($resourceData['defaultWindowWidth']))?$resourceData['defaultWindowWidth']:'0')  .",".
						((isset($resourceData['winInnerWidth']))?$resourceData['winInnerWidth']:'0').",".
						((isset($resourceData['winInnerHeight']))?$resourceData['winInnerHeight']:'0').
						((@$resourceData['separateWindow']||@$resourceData['definition']['separateWindow'])?',1':',0').
						')');

					// Set background icons position according to iconPosition data
					//$activeItems[$uniqueID]=array('name'=>'resIcon_'.$uniqueID, 'type'=>'icon', 'x'=>null, 'y'=>null, 'resourceFilename'=>$temp['file']);
					$maxSessShortID++;
					$processedSessModified=true;
				}
				$nb++;
			}

			if($processedSessModified) {
				// Commit to file (3 retries, as session may lock file)
				while(file_put_contents($completeFilename,serialize($processedSess))===false && @$retries++<3) usleep(100000);
			}
		}
	}
	@wSession_start();
}

/**
 * @desc Initialize default base folders for photo/video/music/documents files
 *
 */
function ifInitializeUserDirectories(){

	if(WEnv::host()->isWindows()) $home=getenv('HOMEDRIVE').getenv('HOMEPATH');
	if(WEnv::host()->isLinux()) $home=getenv('HOME');

	// Video
	if(WEnv::host()->isLinux()) cfGUpdateVar('specialFolderMyVideo',$home.'/Videos');

	// Music
	if(WEnv::host()->isLinux()) cfGUpdateVar('specialFolderMyMusic',$home.'/Music');

	// Pictures
	if(WEnv::host()->isLinux()) cfGUpdateVar('specialFolderMyPictures',$home.'/Pictures');

	// Documents
	if(WEnv::host()->isLinux()) cfGUpdateVar('specialFolderMyDocuments',$home.'/Documents');
	// Home
	if(WEnv::host()->isLinux()) cfGUpdateVar('specialFolderHome',$home);

	// Desktop
	if(WEnv::host()->isLinux()) cfGUpdateVar('specialFolderDesktop',$home.'/Desktop');
}

/**
 * @desc load user configuration file, language file, and computes a few things...
 * 			called by login.php on session start
 * @param boolean $filterIP: true to remove users who may not be accessed by $_SERVER['REMOTE_ADDR']
 * @return boolean true if no initialization critical problem, else exit
*/
function ifInitializeSession($filterIP=true){

	// Reset logged user Id
	if(isset($_SESSION['userId'])) unset($_SESSION['userId']);

	//Load users files
	if(!isset($_ENV['users'])) $_ENV['users']=array();

	$_SESSION['res']=array();

	if(!isset($_SESSION['nbLoginAttempts'])) $_SESSION['nbLoginAttempts']=array();

	$dir=cfAppDataDir();

	$nb=0;
	// Get all activated users, whose IP restriction policy matches, using general.ini for sorting
	foreach (cfUsersConfigs() as $id=>$user){
		if(!$user->getVar('activated') || 'false'===$user->getVar('activated')) continue;
		// Check for included / excluded IPs
		if($filterIP && $user->getVar('IPFilter') && $user->getVar('IPFilter')!='no'){
			if($user->getVar('IPFilter')=='include'){
				if(!$user->getVar('IPFilterIncluded')) continue;
				else{
					$match=false;
					foreach (explode(';',$user->getVar('IPFilterIncluded')) as $ip) if(cfIPMatch($ip)) $match=true;
					if(!$match)  continue;
				}
			}
			else{
				if($user->getVar('IPFilterExcluded')){
					$match=false;
					foreach (explode(';',$user->getVar('IPFilterExcluded')) as $ip) if(cfIPMatch($ip)) {$match=true; break;}
					if($match) continue;
				}
			}
		}

		// Add user to loaded users list
		$_ENV['users'][$id]=$user->getData();
	}
	if(isset($_SESSION['temp'])) unset($_SESSION['temp']); // ?

	// Set language
	if(isset($_SERVER['HTTP_ACCEPT_LANGUAGE']) && strlen($_SERVER['HTTP_ACCEPT_LANGUAGE'])>1){
		if(cfMIssetVar('weezoLng'.substr($_SERVER['HTTP_ACCEPT_LANGUAGE'],0,2))) cfGSetVar('language',substr($_SERVER['HTTP_ACCEPT_LANGUAGE'],0,2));
	}

	// Set theme

	// Force browser theme if any
	if(cfBGetVar('theme')) cfGSetVar('theme',cfBGetVar('theme'));

	// Mobile theme
	if(cfIsMobile()) {
		if(!cfGGetVar('mobileTheme') || !file_exists(cfAppDocRoot().'/themes/mobile/'.cfGGetVar('mobileTheme'))) cfGSetVar('mobileTheme','b'); // Default
		cfGSetVar('theme','mobile/'.cfGGetVar('mobileTheme'));
		if(cfBGetVar('theme')) cfGSetVar('mobileBrowserTheme',cfBGetVar('theme'));
	}

	return true;
}

/**
 * @desc Load list of disabled features for this host OS
 */
function ifLoadOSDisabledFeatures(){
	$all=cfParse_ini_file(cfAppBinDir().'/default/osDisabledFeatures.ini',true,false);
	cfMSetVar('osDisabledFeatures',@$all[WEnv::host()->osFamily()]);
}

/**
 * Update files from previous versions
 */
function ifUpdateStuff(){
	// Create groups & users directories (upgrade from V3.0.0I-
	if(!is_dir(cfAppDataDir().'/users')) @mkdir(cfAppDataDir().'/users');
	if(!is_dir(cfAppDataDir().'/groups')) @mkdir(cfAppDataDir().'/groups');

	// Move resources into resources directory
	foreach (cfGlob(cfAppDataDir().'/*.usr') as $cfn) rename($cfn,cfAppDataDir().'/groups/'.basename($cfn));

	// Move groups into groups directory
	foreach (cfGlob(cfAppDataDir().'/*.res') as $cfn) rename($cfn,cfAppDataDir().'/res/'.basename($cfn));
}

/**
 * @desc Fix timezone error on Windows, with latest version of PHP
 */
function ifFixTimezone(){
	if(@date_default_timezone_get()=='UTC' && WEnv::host()->isWindows()){
		@list($major,$minor,$sub)=@explode('.',phpversion());
		if($major<5 || $minor<4) return;
		$shell = @new COM("WScript.Shell");
		if(!$shell) return;
		$shift= $shell->RegRead("HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\TimeZoneInformation\\ActiveTimeBias");
		if(!is_numeric($shift)) return;
		$shift*=-60;
		cfMSetVar('timezoneShift',$shift);
	}
}

/**
 * @desc initialize server data :
 * Script launched at first server query
 *
 * Load general.ini into memory
 * Start Weezo if not started
 * Create statistics database if not-existing
 * Perform updates if found
 */
function ifInitializeServer(){
	global $initStep;
	define('DEBUG_STARTUP',0); // Unused

	// Ignore user abort as function can be asynchronously called (and aborted) from Weezo.exe
	ignore_user_abort(true);

	// Lock access to all scripts while initialization is not completed
	cfMSetVar('weezoInitInProgress',1);

	// Set mailslot index, used for mailslot communication with application - do it now so server may communicate startup info with app
	if(isset($_GET['setMailslotIndex']) && is_numeric($_GET['setMailslotIndex'])) cfMSetVar('mailslotIndex',$_GET['setMailslotIndex']);

	// Send initialization step to application and log
	function ifServerInitLog($stepName){
		global $initStep;
		static $step=1;

		// Send step to app so it can be displayed on user interface
		cfServerSendCmd('initStep',array('stepName'=>$stepName,'step'=>$step++));
		$initStep=cfCaption('genStep').' '.$step.cfCaption('genSeparator').cfCaption('alertErrorTitle').' > '.$stepName;

		// Log step
		cfLog($stepName,LOG_DBG);
	}

	// Catch fatal errors
	function ifFatalErrorInitHandler(){
		global $initStep;
		$last_error= error_get_last();
		if ($last_error['type'] === E_ERROR) {
			cfMUnsetVar('weezoInitInProgress');
			cfLog($initStep,LOG_ERROR);
			die('initError:'.$initStep);
		}
	}
	register_shutdown_function('ifFatalErrorInitHandler');



	ifServerInitLog('Start server initialization');

	// Load general.ini in memory
	ifLoadGeneral();
	if(cfGGetVar('debugThreads')) cfMSetVar('debugThreads',1); // Initialize thread debugging

	// Update files from previous versions
	ifUpdateStuff();

	// Kill CMD consol launched by Weezo (to set path env) as it's not anymore usefull
	ifServerInitLog('Kill CMD');
	ifKillParentCmd();

	// Load application constants
	ifLoadConstants();

	// Workaround for PHP 5.4+ error in timezone setting
	ifFixTimezone();

	// Load list of features not supported by Weezo on this OS
	ifServerInitLog('Load OS definition');
	ifLoadOSDisabledFeatures();

	// Load resources definitions in memory
	ifServerInitLog('Load resources definitions');
	ifResourcesDefinitions();

	// Load resources configuration in memory
	ifServerInitLog('Load resources');
	ifLoadResources();

	// Load group files in memory
	ifServerInitLog('Load groups');
	ifLoadUsers();

	// Load user files in memory
	ifServerInitLog('Load users');
	ifLoadSubUsers();

	// Load languages list into memory
	// weezoLngXX memory data is set to false when language exists but has not yet been requested, and to language array on 1st request
	ifServerInitLog('Load language files');
	foreach (glob(cfAppDocRoot().'/includes/lng/*.php') as $value)	{
		if(!file_exists(str_replace('.php','.gif',$value))) continue;
		cfMSetVar('weezoLng'.cfFileWithoutExtension(basename($value)),false);
	}

	// Create if needed files info cache database
	ifServerInitLog('Initialize files cache database');
	if(!ifSetFilesCacheDB()) cfLog('Error opening/creating cache database',LOG_ERROR);

	// Load themes data in memory
	ifServerInitLog('Load themes');
	ifLoadThemes();

	// Load icons location in memory, as well as css sprites map
	ifServerInitLog('Cache icon table');
	ifLoadIcons();

	// Load browsers capabilities in memory
	ifServerInitLog('Cache browsers table');
	ifLoadBrowsers();

	// Load weezo.net account registration info
	ifServerInitLog('Load weezo.net account data');
	ifLoadRegInfo();

	// Load sticky notes
	ifServerInitLog('Load sticky notes data');
	ifLoadSNotes();

	// Rebuild connected users list (in case of server failure, as connected users list is kept in memory)
	ifServerInitLog('Rebuild connected users list');
	WEnv::usersLogged()->rebuild();

	// Initialize default base folders for photo/video/music/documents files
	ifInitializeUserDirectories();

	// (re)launch Weezo if not launched
	if(0&&!cfProcessExists('Weezo.exe')) {
		if(cfGGetVar('startWithWindows')=='service') cfExecFork('net','SW_HIDE', 'start Weezo');
		else cfExecFork(cfAppBinDir().'/weezo.exe','SW_SHOWNORMAL', '-minimized');
	}

	// Create / update statistics database
	ifServerInitLog('Initialize statistics database');
	require_once(INCLUDE_DIR.'databaseFunctions.php');
	if(!dbCreateStatsDB()) cfLog('Error opening/creating statistics database',LOG_ERROR);

	// Update interrupted transfers states
	require_once(INCLUDE_DIR.'transferFunctions.php');
	tInitialize();

	// perform updates
	if(file_exists(cfAppDocRoot().'/includes/updateScript.php')) {
		ifServerInitLog('Launch update script');
		try{include(INCLUDE_DIR.'updateScript.php');} catch (Exception $e){ifServerInitLog('Update script failed !',LOG_ERROR);}
	}
	if(!cfMIssetVar('weezoTransfersLastUpdate')) {
		cfMSetVar('weezoTransfersLastUpdate',time().'.'.microtime());
	}

	// Clear APC cache (only if APC is not used as memory data storage)
	ifServerInitLog('Clear APC cache');
	if(function_exists('apc_clear_cache') && !function_exists('customSetData')) @apc_clear_cache(); // Disabled as it hangs with weezo PHP5.3 compil

	// Clean too-long log files
	ifServerInitLog('Rotate logs');
	ifRotateLog(HTTPD_ACCESS_LOG_FILE,cfGGetVar('accessLogMaxSize'));
	ifRotateLog(HTTPD_ACCESS_LOG_FILE_UI,cfGGetVar('accessLogMaxSize'));
	ifRotateLog('httpdErrors.log',cfGGetVar('accessLogMaxSize'));
	ifRotateLog('phpErrors.txt',cfGGetVar('accessLogMaxSize'));

	// Write .htaccess to enable aliases and general-level IP redirections
	cfHtaccessMainWrite();

	// Indicate server has been initialized once
	if(!cfMGetVar('weezoInitialized'))	cfMSetVar('weezoInitialized',1);

	// Unlock scripts access as daemons and bittorrent seeder are launched scripts and must not be blocked in security.php
	cfMUnsetVar('weezoInitInProgress');

	// Initialise qt dll (linux/OSX)
	if(function_exists("customInitDll")) {
		ifServerInitLog('Initialize Qt extension');
		customInitDll(cfAppBinDir().'/phpQtExt.so');
	}

	// Launch daemons (IMPORTANT: this must be done after server initialization complete)
	ifServerInitLog('Launch daemons scripts');
	require_once(INCLUDE_DIR.'daemonFunctions.php');

	// Launch main daemon
	WDaemons::create('main',3,MAIN_DAEMON_LIFETIME);

	// Launch controls daemon
	WDaemons::create('controls',10,MAIN_DAEMON_LIFETIME);

	// Launch bittorrent seeder
	if(cfGGetVar('bittorrentEnabled')) {
		ifServerInitLog('Start bittorrent seeder');
		cfPHPAsyncScript('/bittorrentSeeder.php?action=start',1000);
	}

	// Log
	ifServerInitLog('Server initialized');
}
?>
