<?php
/**
 * Resource configuration common functions
 *
 * This script is either included by accounts.php (if accessed from a browser) or directly called by embeded browser of application
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2013 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

// Data that will be passed as a js array to detect data updates
$_ENV['rcSyncData']=array();
cfDebugTag('resConfDebugTag');

require_once(INCLUDE_DIR.'mailFunctions.php');


/**
 * @desc return direct access link
 *
 * @param string $rId: resource Id
 * @param array $resourceData: resource definition
 * @return string
 */
function rcResourceDirectAccessLink($rId,$resourceData=false){
	// Get registration data
	$regInfo=cfRegistrationData();

	// Get direct access token ID
	if($rId) {
		$resourceConfig=new WResConfig($rId);
		$directResourceAccessToken=$resourceConfig->getVar('directResourceAccessToken');
	}
	else $directResourceAccessToken=$resourceData['directResourceAccessToken'];

	// Return link
	if(cfGGetVar('directLinkLength')>2){
		if(!$regInfo) return cfExternalHostName().strtolower(substr($directResourceAccessToken,0,cfGGetVar('directLinkLength')));
		return cfSiteLink($regInfo['regName']).'/'.strtolower(substr($directResourceAccessToken,0,cfGGetVar('directLinkLength')));
	}
	else{
		if(!$regInfo) return cfExternalHostName().'?directResourceAccessToken='.$directResourceAccessToken;
		return cfSiteLink($regInfo['regName']).'?directResourceAccessToken='.$directResourceAccessToken;
	}
}

/**
 * @desc return webDAV URI
 *
 * @param string $rId: resource Id
 * @param array $resourceData: resource definition
 * @return string
 */
function rcWebDAVURI($rId,$resourceData=false){
	// Get direct access token ID
	if($rId) {
		$resourceConfig=new WResConfig($rId);
		$directResourceAccessToken=$resourceConfig->getVar('directResourceAccessToken');
	}
	else $directResourceAccessToken=$resourceData['directResourceAccessToken'];

	// Return URI
	if(cfGGetVar('directLinkLength')>2)
		return '/dav/'.strtolower(substr($directResourceAccessToken,0,cfGGetVar('directLinkLength'))).'/';
	else
		return '/dav/'.$directResourceAccessToken.'/';
}

/*
 ***************************************************************************************************************************
 * Wrappers
 ***************************************************************************************************************************
 */

/**
 * @desc Set configuration rights
 * 			Display error page if config not allowed
 * @return boolean : true if user may only modifiy non-sensitive parameters, for this session only,
 * 					or false if user has full-configuration rights
 *
 */
function rcConfigRights($limitedConfigAllowed=false){
	global $includedByResourceConfig;

	// If included by resourceConfig.php script: access control has already beed done (double check anyway)
	if(isset($includedByResourceConfig) && (cfIsInApp() || (cfUGetVar('administrator') && ($_SERVER['REMOTE_ADDR']==='127.0.0.1' || cfGGetVar('remoteAdministrationAuthorized'))))) return false;

	// If limited config allowed, initialize limited configuration environment
	if($limitedConfigAllowed) {
		rcInitLimitedConfig();
		return true;
	}

	// Else forbid access and log
	cfLog('administration : unauthorized access attempt by '.$_SERVER['REMOTE_ADDR'], LOG_INF);
	require_once(INCLUDE_DIR.'outputFunctions.php');
	outDisplayErrorPage(cfcaption('errorRemoteAdministrationNotAuthorized')); // If no access right, display error page and exit
}

/**
 * @desc Return resource name (caption)
 *
 * @param string $type
 * @param string $subType
 * @param array $rData: optional: resource definition
 * @return string UTF8 encoded caption
 */
function rcResourceCaptionUTF8($type, $subType ,&$rData=null){
	if(cfCaptionExist($type.'_'.$subType.'-name')) return cfCaption($type.'_'.$subType.'-name');
	if(cfCaptionExist(strtolower($type.'_'.$subType.'-name'))) return cfCaption(strtolower($type.'_'.$subType.'-name'));
	if($rData===null) $rData=cfMGetVar('weezoResourceDefinition'.strtolower($type).strtolower($subType));
	if(isset($rData['describer'][cfGGetVar('language')]['name'])) return $rData['describer'][cfGGetVar('language')]['name'];
	return cfUTF8Encode($subType);
}

/**
 * @desc Return resource description
 *
 * @param string $type
 * @param string $subType
 * @param array $rData: optional: resource definition
 * @return string UTF8 encoded caption
 */
function rcResourceDescriptionUTF8($type, $subType ,&$rData=null){
	if(cfCaptionExist($type.'_'.$subType.'-description')) return cfCaption($type.'_'.$subType.'-description');
	if(cfCaptionExist(strtolower($type.'_'.$subType.'-description'))) return cfCaption(strtolower($type.'_'.$subType.'-description'));
	if($rData===null) $rData=cfMGetVar('weezoResourceDefinition'.strtolower($type).strtolower($subType));
	if(isset($rData['describer'][cfGGetVar('language')]['description'])) return $rData['describer'][cfGGetVar('language')]['description'];
	return cfCaption('genNoDescription');
}

/**
 * @return  mixed array : parsed file (or false if file couldn't be opened)
 * @param string $fileName : file name (not including path)
 * @desc parse a user file
*/
function rcParseUserFile($fileName){
	$file=cfAppDataDir().'/'.$fileName;
	if(!file_exists($file)) return false;
	return cfParse_ini_file($file,true);
}
/**
 * @return  mixed array : parsed file (or false if file couldn't be opened)
 * @param string $fileName : file name (not including path)
 * @desc parse a resource file
*/
function rcParseResourceFile($fileName){
	$file=cfAppDataDir().'/'.$fileName;
	if(!file_exists($file)) return false;
	return cfParse_ini_file($file, true);
}
/**
 * @return  mixed array : parsed file (or false if file couldn't be opened)
 * @desc parse the transfers file
*/
function rcParseTransfersFile(){
	$file=cfAppDataDir().'/transfers.txt';
	if(!file_exists($file)) return false;
	return cfParse_ini_file($file,true);
}

/**
 * @return mixed: filename if success, false if user file couldn't be written
 * @param mixed array $params : complete user configuration
 * @param string $fileName : file name (not including path). If not set, try to detect filename from $params
 * @param bool $commit: true to update UI and to reload all users (should be used when a single user is updated)
 * @desc Write .usr user configuration file, from $params array of parameters
*/
function rcWriteUserFile($params, $fileName='',$commit=true){
	if(!$fileName){
		if(isset($params['configFilename'])) $fileName=$params['configFilename'];
		else {
			cfLog('administration : error saving user file, filename not found', LOG_ERROR);
			return false;
		}
	}
	// Save file
	if(!cfWriteIniFile($params,cfAppDataDir().'/groups/'.$fileName,true)){
		cfLog('administration : error saving '.$fileName.' user file', LOG_ERROR);
		return false;
	}
	//cfServerSendCommand('refreshUser file="'.$fileName.'"');
	cfLog('administration : '.$fileName.' user file modified by '.$_SERVER['REMOTE_ADDR'], LOG_DBG);

	// Reload users into memory
	require_once(INCLUDE_DIR.'initFunctions.php');
	ifLoadUsers();
	return $fileName;
}

/**
 * @desc Update several user files
 *
 * @param array $users: array of user config array
 */
function rcWriteUserFiles($users){
	if(!is_array($users) || count($users)==0) return ;

	// Save to disk
	$filenames=array();
	foreach ($users as $userConfig) $filenames[]=rcWriteUserFile($userConfig,'',false);

	// Reload all users
	ifLoadUsers();
}

/**
 * @desc return an array id=>properties of users bound to resource
 *
 * @param string $filename: resource id OR resource config filename
 * @return array
 */
function rcResourceGetUsers($filename){
	$bound=array();
	// If id passed, search matching filename
	if(!strpos($filename,'.res')){
		if(!($filename=cfArrayItem(cfMGetVar('weezoResourcesList'),$filename))) return array();
	}
	// Browse users
	foreach (cfMGetVar('weezoUsers') as $id=>$user){
		$i=0;
		// Browse user's resources
		while (isset($user['resource'.$i]['file'])) {
			// If searched resource matches
			if($user['resource'.$i]['file']==$filename){
				$bound[$id]=$user;
				break;
			}
			$i++;
		}
	}
	return $bound;
}

/**
 * @return boolean : true if success, false if user file couldn't be written
 * @param mixed array $params : complete user configuration
 * @param string $fileName : file name (not including path)
 * @param string $forceCommitToApp: true to force application resource update
 * @desc Write .res resource configuration file, from $params array of parameters
*/
function rcWriteResourceFile($params, $fileName=false, $forceCommitToApp=false){
	global $resourceNameModified;
	global $limitedConfig;
	global $originalResourceData;

	// Remove data directly linked to resource definition
	unset($params['definition']);

	$commit=true;

	// Check if config has been modified
	if(isset($originalResourceData)){
		$commit=false;
		foreach ($params as $k=>$v){
			if(!isset($originalResourceData[$k])){$commit=true;break;}
			if(is_array($v)){
				foreach ($v as $k2=>$v2){
					if($originalResourceData[$k][$k2]!==$v2) {$commit=true;break 2;}
				}
			}
			elseif ($v!==$originalResourceData[$k]){
				if($k!=='path' || str_replace('*appDir*',cfAppDataRootDir(),$v)!==$originalResourceData[$k]){$commit=true;break;}
			}
		}
	}

	if(!$fileName && isset($params['resourceFilename'])) $fileName=$params['resourceFilename'];
	elseif(!$fileName && isset($params['resourceName'])) $fileName=$params['resourceName'].'.res';


	// Limited configuration : apply changes to current resource data
	if($limitedConfig) {
		// Apply
		foreach ($params as $key=>$value) if($value!=cfRGetVar($key)) cfRSetVar($key,$value);
		//$_SESSION['res'][$_SESSION['activeResourceId']]=$params;
		// Reload opener window and close this window
		if(count($_POST)) echo '<script type="text/javascript">function onLoadFunc(){applyChanges()}</script>';
		else{
?>
<script type="text/javascript">
function onLoadFunc(){winMe.resizeInnerTo(545,dgi('limitedConfigFrame').offsetHeight+10,1)}
</script>
<?php
		}
		return true;
	}

	// Config not modified - exit
	if(!$commit) return true;

	// Normal config : commit new data to resource config file
	$newResource=!file_exists(cfAppDataDir().'/res/'.$fileName);
	if(!cfWriteIniFile($params,cfAppDataDir().'/res/'.$fileName,true)){
		cfLog('administration : error saving '.$fileName.' resource file', LOG_ERROR);
		return false;
	}

	// Update in-memory resource data
	require_once(INCLUDE_DIR.'initFunctions.php');
	ifLoadResource($fileName,$params);

	if($forceCommitToApp || $newResource || isset($resourceNameModified)) {
		// Update in application (still needed as filename modification in DB are done through app)
		cfServerSendCommand('refreshResource file="'.$fileName.'" resourceNameModified="true"');

		// Reset users-resources associations
		if($newResource) {
			require_once(INCLUDE_DIR.'initFunctions.php');
			ifResetResourcesUsers(false,false);
		}
	}

	cfLog('administration : '.$fileName.' configuration file modified by '.$_SERVER['REMOTE_ADDR'], LOG_DBG);
	return true;
}

/**
 * @desc Create a new resource
 *
 * @param string $type
 * @param string $subType
 * @param array $resourceData (optional): preset resource data
 * @return string resource configuration filename
 */
function rcCreateResource($type,$subType,$resourceData=false,&$createdResourceData=false){
	// Clean $type & $subType
	$type	=str_replace('/','',str_replace('\\','',str_replace('..','',$type)));
	$subType=str_replace('/','',str_replace('\\','',str_replace('..','',$subType)));

	// Verify it's actually a resource
	if(is_dir(cfAppDocRoot().'/res/'.$type.'/'.$subType)){
		// Generate resource file
		$rData=rcGetResourceDefinition($type,$subType);
		$tmpResource=array();
		if(is_array($resourceData)) $tmpResource=$resourceData;
		if(!isset($tmpResource['name'])) $tmpResource['name']=cfUTF8Decode($rData['name']);
		$tmpResource['type']=$type;
		$tmpResource['subType']=$subType;
		$tmpResource['id']=substr(sha1(rand().rand().rand()),0,10);
		// Get default parameters if present
		if(file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/defaultConfig.php')){
			$defaultConfig=cfParse_ini_file(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/defaultConfig.php');
			foreach ($defaultConfig as $key=>$value) if($key!='name' && $key!='type' && $key!='subType') $tmpResource[$key]=$value;
			unset($defaultConfig);
		}
		// Write resource file
		$nb=1;
		while(file_exists(cfAppDataDir().'/res/'.$type.$subType.$nb.'.res')) $nb++;
		rcWriteResourceFile($tmpResource,$type.$subType.$nb.'.res');

		// Return resource data
		$createdResourceData=$tmpResource;

		return $type.$subType.$nb.'.res';
	}
	return false;
}

/**
 * @desc Add an asynchronous javascript or HTML insertion command
 *
 * @param string $xmlNode: xml node generated by cfAsyncXMLInnerHTMLbyId(), cfAsyncXMLJSaction()...
 */
function rcAddAsyncResponse($xmlNode){
	if(!isset($_ENV['rcAsyncResponse'])) $_ENV['rcAsyncResponse']=$xmlNode;
	else $_ENV['rcAsyncResponse'].=$xmlNode;
}

/**
 * @desc : return path from server root of image located in resource directory
 * @return : string : path
 *
 * @param string $fileName : name of image (without path)
 */
function rcResourceImage($fileName){
global $resourceData;	return '/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$fileName;
}

/**
 * @desc Copy $resourceData into $_SESSION['res'][0] so functions using cfRgetVar / cfRSetVar can be used by configuration scripts
 *
 */
function rcCopyResourceDataToRVar(){
	global $resourceData;
	$_SESSION['activeResourceId']=0;
	$_SESSION['res'][0]=$resourceData;
}

/**
 * @desc Create resource data directory if needed
 *
 */
function rcCreateResourceDataDir(){
	global $resourceData;
	global $fileName;
	if($resourceData['resourceDataDirAccessAllowed']){
	$subTypeDir = cfAppDataDir()."/res/".$resourceData['type'] ."/".$resourceData['subType'];if(!file_exists($subTypeDir)) @mkdir($subTypeDir);
	$dataDir = cfAppDataDir()."/res/".$resourceData['type'] ."/".$resourceData['subType']."/". cfFileWithoutExtension($fileName);if(!file_exists($dataDir)) @mkdir($dataDir);
	}
}

/**
  * @desc set a PHP callback function that will be called on resource data change
  *		Callback function must have 2 parameters :
  		- string $dataName
  		- mixed $previousDataValue
  *
  * @param string $functionName
  */
function rcSetModifiedValuePHPFunction($functionName){
	global $modifiedValuePHPFunction;
	$modifiedValuePHPFunction=$functionName;
}

/**
 * @desc Insert / update key/value pair in current user's .usr file
 * @param array $values
 *
 */
function rcUserSaveValues($values){
	// Save only if single user profile
	if($_SESSION['user']['accountType']!='singleUser' && !$_SESSION['user']['administrator']) return;
	$needSave=false;

	// Load original user data
	$users=cfMGetVar('weezoUsers');
	$userConfig=$users[$_SESSION['user']['id']];

	// Apply new entries
	foreach ($values as $key=>$value) if(!isset($userConfig[$key]) || $userConfig[$key]!=$value){
		$userConfig[$key]=$value;
		$modified=true;
	}

	// Commit to file and inform app
	if(isset($modified)) rcWriteUserFile($userConfig, basename($_SESSION['user']['configFilename']));
}

/**
 * @desc Get main info on type/subtype resource
 *
 * @param string $type
 * @param string $subType
 * @return array (type,subType,iconFile,describer)
 */
function rcGetResourceDefinition($type, $subType){
	$res=cfMGetVar('weezoResourceDefinition'.strtolower($type).strtolower($subType));
	$res['name']=rcResourceCaptionUTF8($type,$subType,$res);
	$res['description']=rcResourceDescriptionUTF8($type,$subType,$res);
	return $res;
}

/**
 * @desc Add a resource to one or more users
 *
 * @param string $rid: resoure id OR resource config filename
 * @param mixed $uids: 'all', user id or array of users ids
 */
function rcResourceAddToUsers($rid,$uids){
	// If $rid is resource config filename, find resource id
	if(strpos($rid,'.res')){
		foreach (cfMGetVar('weezoResourcesList') as $k=>$v) if($v==$rid){
			$rid=$k;
			$rFilename=$v;
			break;
		}
		if(!isset($rFilename)) return false;
	}
	// Get resource config filename from rid (as filename is resource identifier in user description)
	elseif(!($rFilename=cfArrayItem(cfMGetVar('weezoResourcesList'),$rid))) return false;

	// Get resource definition
	$rd=cfMGetVar('weezoResData'.$rid);

	$updated=0;
	// Browse users to find concerned user
	if(!is_array($uids) && $uids!=='all') $uids=array($uids);
	foreach ($users=cfMGetVar('weezoUsers') as $uid=>$user) if($uids=='all' || @$_POST['user']===$uid || in_array($uid,$uids)) {
		$max=-1;
		$alreadyBound=0;
		// Count resources to append at end, and check this user doesn't already have this resource
		foreach ($user as $k=>$v) if(cfCmpLeft($k,'resource') && isset($v['file'])){
			// If resource already bound to user, exit
			if($v['file']==$rFilename) {
				$alreadyBound=1;
				break;
			}
			$max=max($max,substr($k,8)); // $k="resourceX"
		}
		if($alreadyBound) continue;

		// Add resource to user
		$user['resource'.($max+1)]=array('type'=>$rd['type'],'subType'=>$rd['subType'],'file'=>$rFilename);
		rcWriteUserFile($user);
		$updated++;
	}
	// If at least one user updated, rebind users / resources in memory
	if($updated){
		require_once(INCLUDE_DIR.'initFunctions.php');
		ifResetResourcesUsers();
	}
}

/**
 * @desc Get updated $resourceData items. Function must be called after last call to rcProcessData.
 * 		 Data is initialized by rcInsertScriptAndForm()
 *
 * @return array updated keys=>values
 */
function rcGetUpdatedValues(){
	global $resourceData, $originalResourceData;
	$updated=array();
	foreach ($resourceData as $k=>$v){
		if(!isset($originalResourceData[$k]) || $originalResourceData[$k]!=$v || strlen($v)!=strlen($originalResourceData[$k])) $updated[$k]=$v;
	}
	return $updated;
}

/*
 ***************************************************************************************************************************
 * Misc actions
 ***************************************************************************************************************************
 */

/**
 * @desc Unlink all thumbnails of current resource
 * @return int: number of deleted files
 *
 */
function rcThumbnailsClear($path=false){
	if(!$path){
		global $resourceData;
		global $fileName;
		$path=cfAppDataDir()."/res/".$resourceData['type'] ."/".$resourceData['subType']."/". cfFileWithoutExtension($fileName);
	}
	$nb=0;
	// Clear subdir's thumbnails
	foreach (glob($path.'/*',GLOB_ONLYDIR) as $completeFilename) $nb+=rcThumbnailsClear($completeFilename);

	// Unlink empty directories
	foreach (glob($path.'/*',GLOB_ONLYDIR) as $completeFilename) @rmdir($completeFilename);

	// Clear current dir's thumbnails
	foreach (glob($path.'/*.jpg') as $completeFilename) {
		@unlink($completeFilename);
		$nb++;
	}

	// Return number of deleted files
	return $nb;
}

/**
 * @desc Delete resource file and inform application
 *
 * @param string $fileName
 */
function rcDeleteResource($fileName){
	// Remove file
	if(!file_exists(cfAppDataDir().'/res/'.$fileName)) return true;
	@unlink(cfAppDataDir().'/res/'.$fileName);

	// Remove from memory
	$rl=cfMGetVar('weezoResourcesList');
	foreach ($rl as $rid=>$rFileName) if($rFileName==$fileName){
		@cfMUnsetVar('weezoResData'.$rid);
		unset($rl[$rid]);
		cfMSetVar('weezoResourcesList',$rl);
		break;
	}

	// Remove from users
	foreach (WEnv::userConfigs() as $userId=>$userConfig){
		if(0) $userConfig=new WUserConfig(); // Zend IDE WA
		if($userConfig->removeResource($fileName)) $userConfig->save();
	}

	// Inform app
	cfServerSendCommand('deleteResource file="'.$fileName.'"');

	// Reset users-resources associations for logged-in users
	require_once(INCLUDE_DIR.'initFunctions.php');
	ifResetResourcesUsers(false,false);
	return true;
}

/**
 * @return boolean : true if success, false if user file couldn't be deleted
 * @param string $fileName : file name (not including path)
 * @desc Delete .usr user configuration file, and inform application to remove user from list
*/
function rcDeleteUser($fileName){
	// Delete file
	@unlink(cfAppDataDir().'/groups/'.$fileName);

	// Inform application (which delete file (once again), update general.ini, and it's internal controls)
	cfServerSendCommand('userDeleted file="'.$fileName.'"');
	cfLog('administration : '.$fileName.' user file deleted by '.$_SERVER['REMOTE_ADDR'], LOG_DBG);

	// Reload users into memory
	require_once(INCLUDE_DIR.'initFunctions.php');
	ifLoadUsers();

	return true;
}


/*
 ***************************************************************************************************************************
 * New resource form
 ***************************************************************************************************************************
 */

/**
 * @desc Insert new resource popup and scripts
 *
 * @param array $allowed: array(type=>array(subtype1,subtype2,...), ...) of allowed new resource. If not set, all resources are allowed
 * @param bool $standalone: true if form is displayed in standalone webbrowser (UI)
 */
function rcNewResourceForm($allowed=false,$standalone=false){
	$resourcesDefinitions=cfMGetVar('weezoResourcesDefinitions');
	$users=cfMGetVar('weezoUsers');

	// Re-order explorer resources
	$explOrder=array('Photo1','File1','music3','Video','multimedia','music1','music2','musicWinamp');
	$expl=array(); foreach ($explOrder as $k) $expl[$k]=$resourcesDefinitions['explorer'][$k];
	foreach ($resourcesDefinitions['explorer'] as $k=>$v) if(!isset($expl[$k])) $expl[$k]=$v;
	$resourcesDefinitions['explorer']=$expl;

	// Prepare resource tree
	$tree=array('ico'=>outIcon('resources'),'path'=>'top','label'=>'','sub'=>array());

	// For each types
	foreach (array('explorer','administration','webcam','website','blog','bookmarks','tv','misc') as $typeName=>$type) if(isset($resourcesDefinitions[$type]) && (!$allowed || isset($allowed[$type]))){
		// No administration config in remote admin
		if($type=='administration' && !cfIsInApp()) continue;

		// Init type's node
		$tree['sub'][$type]=array(
			'ico'=>'/res/'.$type.'/resourceIconSmall.'.((file_exists(cfAppDocRoot().'/res/'.$type.'/resourceIconSmall.png'))?'png':'gif'),
			'label'=>'<b>'.cfCaption('res'.ucfirst($type)).'</b>',
			'unfolded'=>true,
			'sub'=>array());

		$nbSub=0;

		// Search type's resources
		if(isset($resourcesDefinitions[$type])) foreach ($resourcesDefinitions[$type] as $subType=>$memVarName) {

			// filter on displayed resources
			if($allowed && !isset($allowed[$type][strtolower($subType)])) continue;

			// Remove resources not supported on this OS
			if(WEnv::host()->disabledResource($type,$subType)) continue;

			// remove deprecated
			if(!($rData=rcGetResourceDefinition($type,$subType)) || isset($rData['deprecated']) || isset($rData['describer']['deprecated'])) continue;

			// Single instance resource: Browse all resources to see if this resource has already been created
			if(@$rData['describer']['singleInstance']) {
				$found=0;
				foreach (WEnv::resConfigs() as $id=>$resConfig) if($resConfig->definition('type')==$type && $resConfig->definition('subType')==$subType){
					continue 2;
				}
				if($found) continue;
			}

			// Set node
			$tree['sub'][$type]['sub'][$subType]=array(
				'ico'=>$rData['resourceIconSmall'],
				'label'=>$rData['name'],
				'sub'=>array()
			);
			$nbSub++;
			if(!isset($firstSelected)){
				$tree['sub'][$type]['sub'][$subType]['selected']=1;
				$firstSelected=1;
			}

			// JS Array Data
			$description[$type][$subType]=array($rData['name'],$rData['description'],$rData['resourcePreviewImage']);
		}
		// If no sub nodes, remove type-level node
		if($nbSub==0) unset($tree['sub'][$type]);
	}

	// Create treeView
	$popupTreeView=new treeView('popupResTree');
	$popupTreeView->phpTree=$tree;
	$popupTreeView->selectFunction='popupSelectRes';
	$popupTreeView->dblClickFunction='popupSelectAndValidateRes';

	// Create description JS array
	$rd='';
	foreach ($description as $type=>$stDesc){
		$rd.=(($rd=='')?'{':',').'"'.$type.'":';
		$st='';
		foreach ($stDesc as $subType=>$desc){
			$st.=(($st=='')?'{':',').'"'.$subType.'":';
			$st.='Array("'.addslashes($desc[0]).'","'.addslashes($desc[1]).'","'.addslashes($desc[2]).'")';
		}
		$rd.=$st.'}';
	}
	$rd.='}';

?>
<script type="text/javascript">
var resourcesDefinitions=<?php echo $rd;?>,
	standalone=<?php echo (int)$standalone;?>,
	resConfig,newResUseFilesList=0,selectedNode,step=1;

function popupSelectRes(node){
	var path=node.getPath().split('/');
	if(path.length==2) {
		node.toggleFold();
		return;
	}
	node.setSelected();
	selectedNode=node;

	resConfig={'newResType':path[1],'newResSubType':path[2]};

	var desc=resourcesDefinitions[path[1]][path[2]];
	dgi('popupNewResDescTitle').innerHTML=desc[0];
	dgi('popupNewResDescText').innerHTML=desc[1];
	dgi('popupNewResDescImageDiv').innerHTML='<img id="resPreviewImg" style="width:266;height:200">';
	var img=D.createElement('IMG'); img.src=desc[2]; //WA
	dgi('resPreviewImg').src=desc[2]
	btEnable('step1Next');
}
function popupSelectAndValidateRes(node){
	var path=node.getPath().split('/');
	if(path.length==2) return;
	resConfig={'newResType':path[1],'newResSubType':path[2]};
	selectedNode=node;
	step1Next();
}

function step1Next(noBack){
	if(noBack) dgi('step1Prev').style.display='none';
	var e,li=(new wl.blockItemGroup('userBI')).getItems();
	for(e in li) {
		if(resConfig['newResType']!='administration') li[e].enable();
		else {
			if(li[e].n.alt!="userAdminControl") li[e].enable(0); else li[e].select();

		}
	}

	dgi('stepLb').innerHTML=(noBack)?'':'(2/2)';
	dgi('resourceName').value=selectedNode.getLabel()
	dgi('step1Frame').style.display='none';
	dgi('step2Frame').style.display='';
	dgi('resourceName').select();
	if(simpleMode){
		dgi('usersOuterFrame').style.display='none';
		wl.UICommand('fadesize:0x150');
	}
	step=2;
}

function step2Prev(){
	dgi('stepLb').innerHTML='(1/2)';
	dgi('step1Frame').style.display='';
	dgi('step2Frame').style.display='none';
	if(simpleMode) wl.UICommand('fadesize:0x425');
	step=1;
}

function popupValidate(){
	// Add selected users
	var i,u=D.getElementsByTagName('FIELDSET'),users='';
	for(i=0;i<u.length;i++) if(u[i].getAttribute('name')=='userBI' && $(u[i]).e.className().has('biSel')) users+='/'+(u[i].id.substr(4));
	if(users) wl.objectsAdd(resConfig,{'users':users.substr(1)});
	wl.objectsAdd(resConfig,{'resourceName':dgi('resourceName').value});
	if(newResUseFilesList) wl.objectsAdd(resConfig,{'useFilesList':1});

	if(standalone) wl.UICommand('close:newResource:'+wl.arrayPack(resConfig))
	else {wl.asr.send(resConfig);popupHide();}
}

// Show step1 frame of new resource popup
function newResPopupShow(shownNodes,useFilesList,inApp){
	newResUseFilesList=useFilesList?1:0;
	var n,typeNode,firstNode=0,ids,id,subType,tn,nbSubNodes=0;

	for(typeNode in treeViews['popupResTree'].tree['sub']){
		n=new treeNode('popupResTree_top/'+typeNode);
		if(shownNodes && !shownNodes[typeNode]) {
			n.hideNode();
		}
		else {
			n.showNode();

			if((shownNodes && typeof shownNodes[typeNode])=='object'){
				ids=n.childrenNodesIds();
				for(id in ids){
					subType=id.toLowerCase();
					if(!shownNodes[typeNode][subType]) {
						tn=(new treeNode(ids[id]))
						tn.hideNode()
					}
					else{
						tn=(new treeNode(ids[id]))
						tn.setSelected();
						selectedNode=tn
						if(!firstNode)	firstNode=tn;
						nbSubNodes++;
					}
				}
			}
			if(!firstNode)	firstNode=n.firstChild()
		}
	}
	var pn=dgi('popupNewRes'),s=pn.style;
	wl.mask.show(0,1);
	s.visibility="hidden";
	s.display="";
	wl.mask.moveAbove(pn);
	s.visibility="";
	s.height="405px"; //XXX
	<?php if(!$standalone) echo '$(pn).e.center();';?>

	// Single type/subtype: go to step2
	if(nbSubNodes==1) {
		popupSelectRes(firstNode)
		step1Next(1);
	}
	else{
		pn.focus()
		popupSelectRes(firstNode)
	}
}
function popupHide(){
	if(standalone) wl.UICommand('close:')
	wl.mask.hide(1);
	dgi('popupNewRes').style.display="none";
	D.onkeydown=null;
}

$(D).e.setKeycodeListener(
	function(kc){
		if(kc==13 && step==1) step1Next();
		else if(kc==13 && step==2) popupValidate();
		if(kc==27) popupHide();
	}
)

</script>
<div id="popupNewRes" style="position:absolute;display:none">
<?php
	if($standalone){
		$class='frame1';
		$classL='frame2';
		$classR='frame1';
		$subClassR='frame3';
	}
	else {
		outShadowBefore('700px');
		$class='popup frame1';
		$classL='frame2';
		$classR='frame1';
		$subClassR='frame3';
	}
?>
<div class="<?php echo $class;?>" style="padding:5px;width:690px">
<?php
echo '<div class="frame1Header" style="font-size:150%">'.outImageIcon('new32').cfCaption('resNewResource').'<span id="stepLb" style="margin-left:2em">(1/2)</span></div>';


/**
 * 1st Step (resource type selection) frame
 */
?>
<div id="step1Frame" style="height:389px;overflow:hidden;padding:0px;margin:0px">
<?php
echo outTableTransparent(false,'table-layout:fixed');
?>
	<tr>
	<td style="width:220px;vertical-align:top" rowspan="2">
<?php
// Resources tree
echo $popupTreeView->HTML($classL,false,false,'height:369px !important;height:379px;margin-top:0px;margin-bottom:0px');

// Vertical spacer
?>
	<td style="width:0.5em" rowspan="2"></td>
<?php
// Detail frame
?>
	<td style="position:relative;vertical-align:top">
		<div class="<?php echo $classR;?>" style="margin-bottom:0.5em;margin-top:0px">
			<div id="popupNewResDescTitle" style="font-weight:bold;margin:6px;margin-bottom:10px">&nbsp;</div>
			<center style="margin-top:5px;margin-bottom:5px">
				<div id="popupNewResDescImageDiv">
					<img alt="" src="/ext/image.php" style="width:266px;height:200px;margin:10px" onload="fade(this)">
				</div>
				<div class="<?php echo $subClassR;?>" id="popupNewResDescText" style="text-align:justify;font-size:100% !important;font-size:80%;height:80px;overflow:auto">&nbsp;</div>
			</center>
		</div>
	</td></tr>
	<tr><td style="vertical-align:bottop">
	<table style="width:100%;margin-top:5px"><tr>
<?php
	// Cancel button
	echo '<td>'.outBt(cfCaption('genCancel'),outIcon('cancel'),'javascript:popupHide()').'</td>';

	// Next button
	echo '<td style="text-align:right">'.outBt(cfCaption('explorerNext'),outIcon('next'),'javascript:step1Next()','step1Next').'</td>';
?>
	</tr></table>
</td></tr></table>
</div>
<?php


/**
 * 2nd Step (resource name and groups selection) frame
 */
?>
<div id="step2Frame" style="height:379px !important; height:389px;overflow:hidden;display:none;font-size:135%;padding-top:5px">
<b style="font-size:74%"><?php echo cfCaption('newResourceChooseName');?></b><br>
<input class="textInput" id="resourceName" style="width:80%;font-size:100%"><br>
<br>
<div id="usersOuterFrame">
	<b style="font-size:74%"><?php echo cfCaption('newResourceSelectUsers');?></b><br>
	<div class="<?php echo $classL;?>" style="height:215px !important;height:225px;overflow:auto;font-size:100%" id="usersFrame">
<?php

// Display users list
foreach ($users as $id=>$user) if(!@$user['invisible']){
	$out='<div style="float:left;margin-right:1em;padding:0px"><img src="'.cfExtGFX('/gfx/icons/'.$user['icon'],40,40).'" style="width:40px;height:40px;visibility:hidden" onload="fade(this)"></div>';
	$out.=str_replace('<','&lt;',str_replace('>','&gt;',cfUTF8Encode($user['name'])));
	$out.='<br><div class="warning smallFont">'.((@$user['administrator'])?cfCaption('userAdministrator'):'&nbsp;').'</div>';

	echo outBlockItem('userBI',$out,'alt="'.((@$user['administrator'])?'userAdminControl':'').'" id="user'.$user['id'].'"','width:210px;min-width:200px;padding:5px;overflow:hidden;text-overflow:ellipsis;cursor:pointer',array('type'=>'checkbox'));
}
?>
	</div>
	<div class="extendedInfo" style="font-size:80%">
	<?php echo outImageIcon('info2').cfCaption('newResourceMayChange'); ?>
	</div>
</div>
<table style="width:100%;margin-top:10px"><tr>
<?php
// Back button
echo '<td>'.outBt('&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'.cfCaption('explorerPrevious').'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;',outIcon('backRed'),'javascript:step2Prev()','step1Prev').'</td>';
// Cancel button
echo '<td style="text-align:center">'.outBt(cfCaption('genCancel'),outIcon('cancel'),'javascript:step2Prev();popupHide()').'</td>';

// Create button
echo '<td style="text-align:right">'.outBt(cfCaption('adminResourceCreate'),outIcon('new'),'javascript:popupValidate()',false,false,'','up',100).'</td>';
?>
</tr></table>
</div>
</div>
<?php if(!$standalone) outShadowAfter(); ?>
</div>
<?php
}


/*
 ***************************************************************************************************************************
 * HTML / JS Output
 ***************************************************************************************************************************
 */

/**
 * @desc Insert Javascript function needed for save button activation, and <form> with hidden fields
 * 		This function must be called before variables processing (rcProcessData)
 *
 */
function rcInsertScriptAndForm(){
	global $resourceData;
	global $originalResourceData;
	global $fileName;

	// Store current resource data to detect updated values
	$originalResourceData=$resourceData;

	$isAdvancedConfig=!isset($resourceData['resourceFilename']);

// Javascript
?>
<noscript>
<div class="frame2" ><b class="warning"><?php echo outImage(outIcon('alertBig.png'),false,false,'float:left').cfCaption('loginErrorNoJavascript');?></b><br/><br/><br/></div><br/>
</noscript>
<script language="javascript" type="text/javascript">
var rcButtonLessConfig=<?php echo (int)@$_ENV['buttonLessConfig'];?>;
<?php
echo cfJSCaptionVar('photoCfgCEAdmin');

if(isset($resourceData['type'])){ ?>
var rcResourceFilename="<?php echo $resourceData['resourceFilename'];?>";
var rcType="<?php echo $resourceData['type'];?>";
var rcSubType="<?php echo $resourceData['subType'];?>";
var rcMultiLists={};
<?php } ?>
</script>
<?php
	// Insert scripts (directly embeded to suppress one HTTP request)
	echo cfScriptLink('rcScripts.js',true);

	// cancel form
	echo '<form action="'.$_SERVER['PHP_SELF'].'" method="GET" enctype="multipart/form-data" name="cancelForm" style="display:none">';
		if(!cfIsInApp()) echo '<input type="hidden" name="config" value="'.$fileName.'">';
		if(cfIsInApp()) echo '<input type="hidden" name="resourceConfigFilename" value="'.$fileName.'">';
		echo '<input type="hidden" name="cancel" value="true">';
	echo '</form>';
	// Submit changes form
	echo '<form id="configForm" name="configForm" action="'.$_SERVER['PHP_SELF'].'" method="POST" enctype="multipart/form-data" style="padding:0;margin:0">';
	if(!cfIsInApp()) echo '<input type="hidden" name="config" value="'.$fileName.'">';
	if(cfIsInApp()) echo '<input type="hidden" name="resourceConfigFilename" value="'.$fileName.'">';
	echo '<input type="hidden" name="commandSent" value="modify">';
	if(isset($_SESSION['activeResourceId'])) echo '<input type="hidden" name="resId" value="'.$_SESSION['activeResourceId'].'">';
}

/**
 * @desc Insert resource icon and name in configuration form
 *
 * @param array $resourceData: resource data
 * @param array $options: 'rcSharedFilesBox'=>array(): if set, insert sharedFilesBox with array's arguments
 * @param boolean $webDAVAccess: set to true to allow WebDAV access to this resource
 */
function rcInsertIconName(&$resourceData, $webDAVAccess=false,$options=array()){
	global $limitedConfig;
	global $resourceObject;

	$resourceDefinition=cfMGetVar('weezoResourceDefinition'.strtolower($resourceData['type'].$resourceData['subType']));

	/**
	 * In application: show delete button, resource icon, resource modifiable name and resource type label
	 */
	if(cfIsInApp()){
		// Delete button
		echo outBt(cfCaption('genDelete'),outIcon('del'),"javascript:parent.deleteResource('".$resourceData['resourceFilename']."')",false,false,'style="float:right"');
		// Resource icon
		echo '<a class="dropdownImage" href="javascript:rcShowIconForm(1)" style="vertical-align:bottom;float:left;margin:0 1em 5px 0">';
		echo outImage($resourceObject->icon32(),false,'id="resIconImg"','width:32px;height:32px');
		echo '</a><div style="margin-top:5px">';
		echo '<script type="text/javascript">';
		echo 'var defaultResIcon="'.$resourceData['definition']['resourceIcon'].'",'; // Store default resource icon's URL in case user select it
		echo 'defaultResIconSmall="'.$resourceData['definition']['resourceIconSmall'].'";</script>'; // Store default resource small icon's URL in case user select it
		echo '<span style="display:none">';
		rcProcessData('customIcon',dataNumeric,0,array(-1,500),"&nbsp;");
		echo '</span>';
		// Resource name
		rcProcessData('name',dataText,$resourceData['name'],array(1,50),array('<span class="frame2Header">'.cfCaption('resName').'</span><br>',50),false);
		echo '</div>';
		echo '<br clear="all">';
	}
	// Limited configuration header
	elseif ($limitedConfig){
		echo '<div class="frame1Header">';
		echo outImage($resourceDefinition['resourceIconSmall'],false,'','vertical-align:middle;margin-right:1em');
		echo cfUTF8Encode($resourceData['name']);
		echo '</div>';
		return ;
	}
	// Remote administration header
	else{
		echo outImage($resourceObject->icon16(),false,'','vertical-align:middle;margin-right:1em');
		rcProcessData('name',dataText,$resourceData['name'],array(1,50),cfCaption('resName'));
		echo '<br>';
	}

	$users=rcResourceGetUsers($resourceData['resourceFilename']);
	$allUsers=cfMGetVar('weezoUsers');


	//else echo outConfigSeparator();

	echo '<div id="resourceAccessBlock">';
	echo '<div id="resourceUsersList">';

	/**************************************
	 * Bound users (normal mode only)
	**************************************/
	if(!cfSimpleMode()){
		// Generate linked users list
		$list='';
		foreach ($users as $id=>$user) {
			if(cfIsInApp()) {
				$list.=', <span style="text-decoration:underline" class="link" onclick="UIShowUser(\''.$user['id'].'\')">'.str_replace('<','&lt;',str_replace('>','&gt;',cfUTF8Encode($user['name']))).'</span>';
				$list.='(<img src="'.outIcon('cancelVSNarrow').'" style="cursor:hand;vertical-align:middle" onclick="javascript:unbindUser(\''.$id.'\')" title="'.str_replace('"',"''",cfCaption('genRemove')).'">)';
			}
			else{
				$list.=',&nbsp;<span>'.cfUTF8Encode($user['name']).'</span>';
			}
		}


		// List unbound users (in-app only)
		$unbound='';
		if(cfIsInApp() && count($users)<count($allUsers)) foreach ($allUsers as $id=>$user) {
			// if not already bound, and not "system" user
			if(!isset($users[$id]) && !@$user['invisible'] &&
			// and not admin resource or user is admin
			   ($resourceData['type']!=='administration' || @$user['administrator'])){
				$unbound.=',"'.$user['id'].'":{"name":"'.str_replace('"','&quot;',cfUTF8Encode($user['name'])).'","icon":"'.(cfExtGFX('/gfx/icons/'.$user['icon'],16,16)).'"}';
			}
		}

		// Bind new user control (in-app only)
		if($unbound){
			$unboundControl='<span class="link" style="position:absolute;right:20px" id="showUnboundUsersBt">'.cfCaption('genAdd').outImage(outIcon('down'),false,false,'vertical-align:bottom"').'</span>';
			$unbound='createUnboundUsersCM(unboundUsers={'.substr($unbound,1).'});';
		}
		else $unboundControl='';


		// Title
		$out='<div class="frame2Header">';
		$out.=outImageIcon('users','','margin-left:2px').''.cfCaption('resSharedWith');
		$out.='</div>';

		if($list) {
			$out.='<span style="width:80%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap">';
			$out.=substr($list,1);
			$out.='</span>'; // End of overflow hidden
			$out.=$unboundControl;
		}
		else {
			$out.='<div class="">';
			$out.='<span style="width:80%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap">'.outImageIcon('warning').cfCaption('resSharedWithNone');
			$out.='</span>'; // End of overflow hidden
			$out.=$unboundControl;
			$out.='</div>';
		}


		// Check box used to indicate boud users list has been modified and so it must be async refreshed
		$out.='<input type="checkbox" name="resourceUsersListModified" style="display:none">';


	}
	else {
		$out='';
		$unbound='';
	}


	// Async response (in-app only, as in remote admin, output is buffered using ob_XXX and injected into a div
	if(cfIsAsync() && cfIsInApp()) {
		// Update only if bound users list has been modified (resourceUsersListModified checkbox is toggled by toggleUserDone js function)
		if(@$_POST['resourceUsersListModified']=='on'){
			rcAddAsyncResponse(cfAsyncXMLInnerHTMLbyId($out,'resourceUsersList'));
			$script=array();
			foreach ($users as $id=>$value) $script[]=$id;
			if($unbound) rcAddAsyncResponse(cfAsyncXMLJSaction($unbound));
			rcAddAsyncResponse(cfAsyncJSaction('rcUserRightsUpdatedUsers('.json_encode($script).')'));
		}
	}
	// Sync display
	else {
		echo $out;
		if($unbound) echo '<script type="text/javascript">'.$unbound.'</script>';
	}

	echo '</div>'; // end of #resourceUsersList



	/**************************************
	 * Direct resource access
	 **************************************/
	if($resourceData['type']!=='administration'){

		if($out!='') echo outConfigSeparator();

		if(!cfRegistrationData()) $class='extendedWarning';
		else $class='';

		$out='<div id="directResourceAccessConfig" class="'.$class.'" style="margin-bottom:15px"'.((cfRegistrationData())?'':outToolTip(str_replace('"','&quot;',cfCaption('registrationNeeded')))).'>';
		if(!cfSimpleMode()) $out.='<span style="position:absolute;right:20px;">'.outHelpButton('directResourceAccessHelp','directResourceAccess').'</span>';

		// Enable / disable direct resource access
		if(cfSimpleMode()){
			$resourceData['directResourceAccessEnabled']=true;
			$out.='<span class="frame2Header">'.cfCaption('directResourceAccess').'</span><br>';
		}
		else{
			$out.=rcProcessData('directResourceAccessEnabled',dataBoolean,false,false,'<span class="frame2Header">'.cfCaption('directResourceAccess').'</span>',false,false,array('returnResult'=>1,'disabled'=>!cfRegistrationData()));
		}

		// LAN restricted access
		$out.='<span id="directResourceAccessLanOnlyBlock" style="margin-left:7em;'.(($resourceData['directResourceAccessEnabled'])?'':'visibility:hidden').'">';
		if(!cfSimpleMode()) $out.=rcProcessData('directResourceAccessLanOnly',dataBoolean,false,false,cfCaption('IPFilterLANOnly'),false,false,array('returnResult'=>1)).'</span>';
		$out.='<div id="directResourceAccessParams" style="display:'.(($resourceData['directResourceAccessEnabled'])?'block':'none').';position:relative">';


		/**
		 * Direct access publish token (directResourceAccessToken)
		 */
		if(!@$resourceData['directResourceAccessToken']){
			// Generate token if not defined yet
			require_once(cfAppDocRoot().'/local/directLink.php');
			$false='';
			$defaultToken=dlGenerateId($false,30);
		}
		else $defaultToken='';
		$out.=rcProcessData('directResourceAccessToken',dataText,$defaultToken,array(30,30),false);

		// Direct access link
		$link=rcResourceDirectAccessLink(false,$resourceData);
		$out.='<div class="configFrame">';
		$out.='<span style="margin:0 1em 0 0;font-weight:bold">'.outImageIcon('share').cfCaption('genLink').'</span>';
		$out.='<input class="textInput" value="'.$link.'" readonly="readonly" size="33" onclick="this.select()">&nbsp;';

		// Direct resource access link mail
		$mailBody=cfUTF8Decode(cfCaption('genLink').cfCaption('genSeparator')).$link;
		if(@$resourceData['directResourceAccessPasswordHas']) {
			$password=cfStoredPasswordDecrypt($resourceData['directResourceAccessPassword']);
			$mailBody.="\n".cfUTF8Decode(cfCaption('genPassword').cfCaption('genSeparator')).$password;
		}
		else $password=false;

		// Social media sharing

		$out.=outSocialURLShare($link,array('fullText'=>$mailBody,'password'=>$password,'bwIco'=>0), APPLICATION_NAME.' - '.$resourceData['name']);
		$out.='&nbsp;&nbsp;&nbsp; ';

		// Direct access password activated
		$out.='<fieldset style="text-align:right">';
		$out.=rcProcessData('directResourceAccessPasswordHas',dataBoolean,true,false,' ',false,false,array('returnResult'=>1)).'&nbsp;';

		// Fill password if not set
		if($resourceData['directResourceAccessPasswordHas'] && !cfStoredPasswordDecrypt(@$resourceData['directResourceAccessPassword']))
			$resourceData['directResourceAccessPassword']=cfStoredPasswordCrypt(strtolower(cfGenerateID(3).'.'.cfGenerateID(3).'.'.cfGenerateID(3)));

		// Direct access password
		$out.=rcProcessData('directResourceAccessPassword',dataPassword,cfStoredPasswordCrypt(strtolower(cfGenerateID(3).'.'.cfGenerateID(3).'.'.cfGenerateID(3))),array(1,999),array(cfCaption('genPassword'),12),false,false,array('returnResult'=>1,'passwordClear'=>1,'disabled'=>!$resourceData['directResourceAccessPasswordHas'],'faded'=>!$resourceData['directResourceAccessPasswordHas']));

		$out.='</fieldset>';

		// WebDAV access
		if($webDAVAccess && !cfSimpleMode() && cfConstant('WEBDAV_ENABLED')){

			$out.='<br><div id="webDAVGroup" style="margin-top:0.5em">';

			$out.=rcProcessData('webDAVEnabled',dataBoolean,false,false,outImageIcon('webDAV').cfCaption('webDAVEnabled'),false,false,array('returnResult'=>1,'disabled'=>!cfRegistrationData()));
			$out.='<span style="position:absolute;right:20px">'.outHelpButton(('webDAVDescription'),'webDAV').'</span>';

			$out.=rcProcessData('webDAVShowDetail',dataBoolean,false,false,'<span class="link" id="webDAVDetail"'.((@$resourceData['webDAVEnabled'])?'':' style="display:none"').'><span id="webDAVShowDetailShow" style="'.((@$resourceData['webDAVShowDetail'])?'display:none':'').'">'.cfCaption('genDetails').'</span><span id="webDAVShowDetailHide" style="'.((@$resourceData['webDAVShowDetail'])?'':'display:none').'">'.cfCaption('genHide').'</span></span>',false,false,array('returnResult'=>1,'disabled'=>!cfRegistrationData(),'hideCheckbox'=>1));

			$out.='<div id="webDAVParams" style="display:'.((@$resourceData['webDAVEnabled'] && @$resourceData['webDAVShowDetail'])?'block':'none').';position:relative" class="configSubFrame">';


			// Check which port will be the primary port
			$port=cfGGetVar('serverPort');
			$altPort=cfGGetVar('alternateServerPort');
			if($port!=80 && $port!=443 && $altPort==80) $port=$altPort;

			$protocol='http://';
			$port=':'.$port;
			if(cfGGetVar('serverPort')==80) $port='';
			elseif(cfGGetVar('serverPort')==443) {$port='';$protocol='https://';}
			else $out.='<div class="warning">'.cfCaption('webDAVPortWarning').'</div>';

			// Out-of-LAN address
			$out.='<div style="position:relative">';
			$out.=outTableTransparent(false,'table-layout:fixed');

			$out.='<tr id="webDAVInternetLink" style="display:'.(($resourceData['directResourceAccessLanOnly'])?'none':'').'"><td style="width:25%">'.cfCaption('genLink').'&nbsp;('.cfCaption('genInternet').')</td><td><input type="text" class="textInput" readonly="readonly" style="width:200px" value="'.$protocol.(WEnv::host()->externalIP()).$port.rcWebDAVURI(false,$resourceData).'"></td></tr>';

			// LAN address
			$out.='<tr><td>'.cfCaption('genLink').'&nbsp;('.cfCaption('apacheStartedLocal').')</td><td><input type="text" class="textInput" readonly="readonly" style="width:200px" value="'.$protocol.WEnv::host()->lanIP().$port.rcWebDAVURI(false,$resourceData).'"></td></tr>';

			// ID
			$out.='<tr><td>'.cfCaption('genAccount').'</td><td><input type="text" class="textInput" readonly="readonly" style="width:80px" value="'.$resourceData['id'].'"></td></tr>';
			// Password
			$out.='<tr><td>'.cfCaption('genPassword').'</td><td>';
			$out.='<input id="webDAVPasswordInput"'.(($resourceData['directResourceAccessPasswordHas'])?'':' style="display:none"').' type="text" class="textInput" readonly="readonly" style="width:80px" value="'.cfStoredPasswordDecrypt($resourceData['directResourceAccessPassword']).'">';
			$out.='<span id="webDAVPasswordNo"'.((!$resourceData['directResourceAccessPasswordHas'])?'':' style="display:none"').'>('.cfCaption('passwordNo').')</span><input class="textInput" style="visibility:hidden">';
			$out.='</td></tr>';
			$out.='</table>';

			// LAN restriction
			$out.='</div>';

			$out.='</div>'; // end of #webDAVParams
			$out.='</div>'; // end of #webDAVGroup
		}


		$out.='</div>'; // end of .configFrame
		$out.='</div>'; // end of #directResourceAccessParams
		$out.='</div>'; // end of #directResourceAccessConfig

		$out.='</div>';

		//$out.=outConfigSeparator();

		echo $out;
	}
	else echo '</div>';



	// Insert shared files box for files resources
	if(isset($options['rcSharedFilesBox'])){
		rcSharedFilesBox(@$options['rcSharedFilesBox'][0],@$options['rcSharedFilesBox'][1],@$options['rcSharedFilesBox'][2]);
	}
}

/**
 * @desc Included into config script of resources requiring MySQL: if MySQL not installed, echo error message and return false
 *
 * @return boolean: false if MySQL not installed, true if installed
 */
function rcMySQLCheckInstalled(){
	if(cfMySQLInstalled()) return true;


	echo '<br><br><div class="extendedWarning"><br>';
	echo outImage(outIcon('warning'),false,false,'float:left');
	echo '<center>'.cfCaption('extMySQLRequired').'<br><br>';
	echo outBt(cfCaption('resMisc').'...',outIcon('plugins'),"javascript:wl.UICommand('pluginmanagement')");
	echo '</center>&nbsp;</div>';
	return false;
}

/**
 * @desc Insert Folder / files list selection sub-form
 *
 * @param array $fileTypeFilter: array of mime types included, false for no filter
 * @param string $defaultDirectory: default shared directory, false to use cfAppSharedDir()
 * @param array $options:
 * 			'noDiv'=>1: do not include outer div
 * 			'noSubDir'=>1: do not include sub directories into single Folder mode
 * 			'filesOnly'=>1: only files, no directories in list mode
 */
function rcSharedFilesBox($fileTypeFilter=false, $defaultDirectory=false, $options=false){
	global $limitedConfig;
	global $resourceData;

	if($limitedConfig) return;

	// Filter (audio/video/image)
	if($fileTypeFilter) {
		$fileTypeFilter=cfArrayValuesToKeys($fileTypeFilter);
		require_once(INCLUDE_DIR.'mime_type.php');
		foreach ($fileTypeFilter as $ftf){
			foreach ($_ENV['weezoMime'] as $ext=>$mime) if(cfCmpLeft($mime,$ftf)) $extFilterArray[$ext]=$ext;
		}
		$extFilter='*.'.implode(';*.',$extFilterArray);
	}
	else $extFilter='';

	// Backward compatibility: set sharedMode to folder for previously created resources
	if(!isset($resourceData['sharedMode'])) {if(isset($resourceData['path'])) $resourceData['sharedMode']='folder'; else $resourceData['sharedMode']='list';}


	if(!isset($options['noDiv'])) echo outDivFrame('frame2 highlighted',false);

	/**
	 * Share mode selection
	 */
	echo '<div style="float:right">';
	rcProcessData('sharedMode',dataList,'list',array('folder','list'),array(
		array(false,'folder',array(outIcon('folder'),cfCaption('sharedFolder').' ')),
		array(false,'list',array(outIcon('filesList'),cfCaption('filesAndFolders')))
		));
	echo '</div>';

	echo '<div class="frame2Header">';
	echo '&nbsp;'.cfCaption('genFiles');
	echo '</div>';

	/**
	 * Shared Folder
	 */
	echo '<br><div id="sharedModeFolder"'.(($resourceData['sharedMode']==='list')?' style="display:none"':'style="display:block"').'>';

	// Base folder
	rcProcessData('path',dataFolderPath,(($defaultDirectory)?$defaultDirectory:cfAppSharedDir()),((isset($options['noComputerRoot']))?'noComputerRoot':false),true);

	// Subdirectories included
	if(!isset($options['noSubDir'])) rcProcessData('subFoldersIncluded',dataBoolean,true,false,cfCaption('explorerCfgSubDirectoriesIncluded'));
	echo '</div>';

	/**
	 * List of files/folders
	 */

	// Hidden control containing items list, | separated
	echo '<div style="position:absolute;top:-1000px">';
	rcProcessData('sharedItems',dataText,'',false,'&nbsp;');
	echo '</div>';

	echo '<div id="sharedModeList" style="'.(($resourceData['sharedMode']==='folder')?'display:none"':'').'">';

	// shared items, | separated
	if(!isset($resourceData['sharedItems'])) $resourceData['sharedItems']=false;
	if($resourceData['sharedMode']=='folder') $_POST['sharedItems']=$resourceData['sharedItems']; // Prevent list modification if sharedMode is folder

	if($resourceData['sharedMode']=='list'){
		require_once(INCLUDE_DIR.'explorerFunctions.php');

		$out=''; $str='';$dir=array();$file=array();
		// Generate HTML list from string
		foreach (explode('|',$resourceData['sharedItems']) as $cfn){
			if(cfFileExtension($cfn)=='lnk') $cfn=cfShortcutResolve($cfn);
			// For existing folders
			if(is_dir($cfn)) $dir[$cfn]=efIcon($cfn);
			// For existing files
			elseif(file_exists($cfn)) {
				// Apply filter (images, audio, video)
				if($fileTypeFilter && !isset($fileTypeFilter[efFileType($cfn)])) continue;
				$file[$cfn]=efIcon($cfn);
			}
			// For inexisting files/folders
			else{
				// If file/folder belongs to an unexisting drive, don't remove as the drive may only have been dismounted
				if(substr($cfn,1,2)==':/' && ord(substr($cfn,0,1))>=66 && ord(substr($cfn,0,1))<91 && !is_dir(substr($cfn,0,3))){
					if(strpos(basename($cfn),'.')===false) $file[$cfn]=outIcon('fi/folder'); else $file[$cfn]=efIcon($cfn);
				}
			}

		}
		// list item html
		function li($cfn,$ico){
			static $nb=0;
			if(!$cfn) return '<div class="'.((($nb++)%2)?'odd':'even').'">&nbsp;</div>';

			if(!cfIsInApp()) return '<div class="'.((($nb++)%2)?'odd':'even').'" id="'.cfUTF8Encode($cfn).'">'.outImage($ico,false,'class="ico2"').'<span onmouseover="tmp=lTt(this);tooltip(this,tmp)">'.cfUTF8Encode(basename($cfn)).'</span><br clear="all"></div>';

			return '<div class="'.((($nb++)%2)?'odd':'even').'" id="'.cfUTF8Encode($cfn).'">'.outButtonSmall(false,'javascript:filesRemove(this)',outIcon('eraser'),cfCaption('genRemove'),false,'style="vertical-align:bottom"').'<img alt="" src="'.$ico.'" class="ico2"><span onmouseover="tooltip(this,lTt(this))">'.cfUTF8Encode(basename($cfn)).'</span><br clear="all"></div>';
		}
		$nb=0;

		// Add dirs
		foreach ($dir as $cfn=>$ico) {
			$out.=li($cfn,$ico);
			$str.='|'.$cfn;
			$nb++;
		}
		// Add files
		foreach ($file as $cfn=>$ico) {
			$out.=li($cfn,$ico);
			$str.='|'.$cfn;
			$nb++;
		}
		while($nb++<7) $out.=li(false,false);
		$resourceData['sharedItems']=substr($str,1);
	}

	// Drop files watermark
	echo '<div style="position:absolute;padding:0 100px 0 100px;top:0;text-align:center;top:50px;z-index:1;width:'.((WEnv::host()->isWindows())?'100%':'70%').'" class="watermark">';
	echo outImage(outIcon('dropFilesIn2'),false,false,'vertical-align:bottom;margin-left:1em').'<br>';
	echo cfCaption('dropFilesHere').'</div>';

	// Shared files & folders (drag & drop) box
	echo '<div class="lvDiv" id="sharedItemsDropBox" style="position:relative;height:160px;overflow:auto;display:block;overflow-x:hidden;z-index:2">';

	// files & folder list
	if(isset($out)){
		if(cfIsAsync() && cfIsInApp()) rcAddAsyncResponse(cfAsyncXMLInnerHTMLbyId($out,'sharedItemsDropBox'));
		else echo $out;
	}

	echo '</div>';

	// Drag & Drop and files&folders input box management
	echo '<input name="dropFileInput" style="display:none" onkeyup="filesDropped()"><input type="button" name="dropFileButton" onclick="filesDropped()" style="display:none">';

	// Add & clear all buttons (only in application UI)
	if(cfIsInApp()){
		// Add files & folders button
		echo '<table style="width:100%"><tr><td>';
		echo outBt(cfCaption('addFiles'),outIcon('addMult'),"javascript:wl.UICommand('openFiles:".$extFilter."','dropFileInput')");
		if(!@$options['filesOnly']) echo outBt(cfCaption('genDirectories'),outIcon('addFolder'),"javascript:wl.UICommand('openFolder','dropFileInput')");

		// Clear all files button
		echo '</td><td style="text-align:right">'.outBt(cfCaption('explorerClearAll'),outIcon('eraser'),'javascript:filesClearList()').'</td></tr></table>';
	}
	echo '</div>'; // #sharedModeList

	if(!isset($options['noDiv'])) echo '</div>';
?>
<br>
<br>
<script type="text/javascript">
var sharedMode="<?php echo $resourceData['sharedMode'];?>";
var extFilter=<?php if(@$extFilterArray) echo '{"'.implode('":1,"',$extFilterArray).'":1}'; else echo 'false';	?>;
</script>
<?php

}

/**
 * @desc Insert HTML stuff for start of advanced configuration (folded or unfolded depending on $resourceData['advancedConfig'] value)
 * @desc array $options: noTopSeparator=>1 : don't show top line separator
 *
 */
function rcAdvancedConfigStart($options=false){
	global $resourceData;
	global $limitedConfig;

	$_ENV['advancedConfig']=1;

	// Backward compatibility : set advanced as default for already configured resources
	if(!isset($resourceData['advancedConfig'])){
		if(isset($resourceData['resourceConfigured'])) $resourceData['advancedConfig']=true; else $resourceData['advancedConfig']=false;
	}

	if(!@$options['noTopSeparator']) echo outConfigSeparator();

	echo '<span style="display:none">';
	rcProcessData('advancedConfig',dataBoolean,false,false,'lc',false);
	echo '</span>';
	if(!$limitedConfig && $resourceData['advancedConfig']==false) $folded=true; else $folded=false;
	echo outDivFrame("advancedConfig".(($folded)?'Folded':'Unfolded'),'id="advancedConfigOuter"');


	echo '<div class="link" id="advancedConfigUnfoldBt" onclick="toggleAdvancedConfig()" style="float:right'.(($folded)?';display:none':'').'">'.cfCaption('genHide').'</div>';
	echo '<div class="link" id="advancedConfigFoldBt" onclick="toggleAdvancedConfig()" style="float:right'.(($folded)?'':';display:none').'">'.cfCaption('genShow').'</div>';
	echo '<div class="frame2Header">'.cfCaption('configAdvanced').'</div>';

	echo '<div id="advancedConfigDiv" style="display:'.(($folded)?'none':'block').'" class="configFrame">';
}

/**
 * @desc Close advanced config divs
 *
 */
function rcAdvancedConfigEnd(){
	if(!isset($_ENV['advancedConfig'])) return ;
	unset($_ENV['advancedConfig']);
	echo "</div></div>\n";
	echo outConfigSeparator();
}

/**
 * @desc Insert configuration buttons to plugin's administration page, for use in application config tab
 *
 * @param string $path: path from resource root dir to config script
 */
function rcPluginConfigurationButtons($path){
	global $resourceData;

	if(!cfIsInApp()) return;

	if($path[0]!='/') $path='/'.$path;
	$path.=((strpos($path,'?')===false)?'?':'&').'rnd='.rand(). // Add a random to ensure page reloading
	$extraPath='&weezoKey='.$_SESSION['weezoKey']; // Add key for external browser access right

	// Resource administration button
	echo '<div style="position:absolute;top:50%;left:0;width:99%;text-align:center">';
	echo outBt(cfCaption('genConfigure'),outIcon('gears'),'/res/'.$resourceData['type'].'/'.$resourceData['subType'].$path);
	echo '&nbsp;&nbsp;&nbsp;';
	echo outButton(cfCaption('genConfigure').' ('.cfCaption('musicPopupPlayer').')','/res/'.$resourceData['type'].'/'.$resourceData['subType'].$path.$extraPath,outIcon('separateWindow'),false,false,' target="_blank"');
	echo '</div>';
}

/**
 * @return string : HTML code
 * @desc return HTML code for theme selection, and theme preview link.
 * 		 Must be inserted in resource parameters form, returned (POST...) value is named 'theme'
 * @param $theme: default theme
 * @param $options: set 'noThemeControl' not to display theme control, set 'noWindowSizeControl' not to display window max/restored control
*/
function rcThemeSelectionControl($theme=false,$options=false){
	global $resourceData;
	$output='';

	// No possible selection for simple mode
	if(cfSimpleMode()) return;

	// No possible selection for single-theme browsers
	if(cfBGetVar('theme')) return;

	// Theme control
	if(!isset($options['noThemeControl'])){
		$dir=cfAppDocRoot().'/themes';
		if(!$theme){
			// Default value
			if(!isset($resourceData['theme'])) $resourceData['theme']='noTheme';
			// Process POST command
			if(isset($_POST['theme'])) $resourceData['theme']=cfUTF8Decode($_POST['theme']);
			// Set Theme
			$defaultTheme=$resourceData['theme'];
		}
		else $defaultTheme=$theme;
		// Display control
		$output = '<span style="margin-right:1em;">'.cfCaption('genTheme').'</span>';
		$output .= '<select class="textInput" name="theme" id="themeSelect" size="1" style="width:15em;margin-right:1em;" '.(($theme)?'':'onChange="eSaveBt(\'theme\',this.value)"').'>';
		// Display "no theme" item (only if $theme parameter is not set)
		if(!$theme){
			if($defaultTheme!='noTheme') $output .= '<option value="noTheme">'.cfCaption('adminNoTheme').'</option>';
			else $output .= '<option selected value="noTheme">'.cfCaption('adminNoTheme').'</option>';
		}

		// Create themes select items
		$themes=cfMGetVar('weezoThemes');

		foreach ($themes as $name=>$data) if($name!='skins/default' && $name!='skins/osx' && !isset($data['private'])){
			if($name==$defaultTheme) $output .= '<option selected value="'.cfUTF8Encode($name).'">'.cfUTF8Encode($name).'</option>'; else $output .= '<option value="'.cfUTF8Encode($name).'">'.cfUTF8Encode($name).'</option>';
		}
		$output .= '</select>';
	}

	// Maximize window control
	if(!isset($options['noWindowSizeControl'])) $output.='<span style="white-space:nowrap'.((isset($options['noThemeControl']))?'':';margin-left:2em').'">'.cfCaption('genWindow').'&nbsp;'.rcProcessData('maximizeAtFirstCall',dataBoolean,true,false,array(outImageIcon('confWinMax',outToolTip(cfCaption('winMaximize'),false,false,true)),outImageIcon('confWinNormal',outToolTip(cfCaption('winRestore'),false,false,true))),false,false,array('returnResult'=>1)).'</span>';

	return $output;
 }

/**
 * @desc Insert (Facebook) comments control
 *
 */
function rcCommentsPanelControl(){
	echo '<div style="float:right">'.outImageIcon('facebook').cfCaption('genCommentsAllowed');
	rcProcessData('comments',dataBoolean,false,false,outHelpTag(cfCaption('facebookCommentsDesc'))); // Allow/Forbid read access to files and folders
	rcProcessData('commentsPanelSate',dataNumeric,0,array(0,2),false); // Default panel state (0:folded, 1 & 2: unfolded)
	echo '</div>';
}

/**
  * Output a save and a cancel buttons dedicated to config.php scripts.
  * In app, don't display buttons as changes takes effect immediately
  *
  * @return string : HTML code
  * @param boolean $defaultCancelEnabled : set to true to enable cancel button, false to set it as disabled at page load
  */
function rcButtonSaveCancel($defaultCancelEnabled=false){
	global $limitedConfig;

	//Close advanced config divs
	if(isset($_ENV['advancedConfig'])) echo "</div></div>\n";

	$output='<center style="margin-top:0.3em'.(($_ENV['buttonLessConfig'])?';display:none':'').'">';

	// Save/apply button
	if((!cfIsInApp()) && basename($_SERVER['PHP_SELF'])=='index.php') $applyScript='save(1,1)'; // Remote configuration environment

	// Limited configuration environment or application environment
	// note: re-enable all input and select items in order to transmit them on form submition
	else $applyScript='save(0,1)';

	// Save/commit button
	$output.=outButtonDisabled(cfCaption('genSave'),'javascript:'.$applyScript,outIcon('save'),false,'saveButton','style="margin-right:5em"');

	// Browser environment: cancel close window (limited config) or reload page (remote administration)
	if(!cfIsInApp())
		$output .= outButton(cfCaption('genCancel'),(($limitedConfig)?'javascript:if(winMe) winMe.closeMe(); else window.close()':'javascript:reloadForm.submit()'),outIcon('cancel'),false,'cancelButton');
	// App: use cancelForm to reset page
	else {
		$output.=outButton(cfCaption('genCancel'),'javascript:D.cancelForm.submit();',outIcon('cancel'),false,'cancelButton',false,false,!$defaultCancelEnabled);
	}
	$output .= '</center></form>';

	// Store all resourceConfig current data in order to detect changes with javascript
	foreach ($_ENV['rcSyncData'] as $k=>$v) $_ENV['rcSyncData'][$k]=cfUTF8Encode($v);
	$output.='<script type="text/javascript">var syncData='.json_encode($_ENV['rcSyncData']).'</script>';
	if(!cfIsAsync()) echo cfDragRegisterItems('');
	return $output;
}

/**
 * @desc Load data required for "limited configuration" (ie inline configuration by end-user), and display page's head and script
 *
 */
function rcInitLimitedConfig(){
	global $resourceData;
	// Load current resource data into $resourceData used by config scripts
	$resourceData=cfRGetVar();

	$_ENV['buttonLessConfig']=false;

	// Insert page's head
	$_ENV['winCloseURL']='/res/'.cfRGetVar('type').'/'.cfRGetVar('subType').'/'.cfRGetVar('baseFile'); // Frameless close URL
	cfInsertHEAD(false);
	if(cfBGetVar('name')=='iPhone') echo '<meta name="viewport" content="width=device-width, user-scalable=1">';
?>
	<script type="text/javascript">
	function applyChanges(){
		if(parent.win(winMe.getOpenerId())) parent.win(winMe.getOpenerId()).reload();
		winMe.closeMe();
	}
	function keydown(e){
		if(e && e.which==27) winMe.closeMe();
		else if(window.event && window.event.keyCode==27) winMe.closeMe();
	}
	D.onkeydown=keydown;
	</script>
	<body onload="onLoadFunc()">
<?php
	echo cfScriptLink('wz_dragdrop.js');
	echo outDivFrame('frame1','id="limitedConfigFrame"',cfGetBrowser()=='ie'?'width:99%':'');
}

/**
 * @desc Return HTML code for a resource (config) item
 *
 * @param string $rid: id of resource
 * @param WUserConfig $uc: bound user config, if set, display all controls, else just display selectable resource
 * @return string
 */
function rcResourceBlock($rid,$uc=false,$fade=false){
	$r=cfMGetVar('weezoResData'.$rid);
	$rObject=new WResConfig($rid);
	$o='';
	// Icon

	if($fade) $o.='<img src="'.cfFilteredImage($rObject->icon32(),0,0,array(),200,20).'" style="float:left;margin-right:0.5em">';
	else $o.=outImage($rObject->icon32(),false,false,'float:left;margin-right:0.5em');

	// Name
	$o.='<div class="uiUsersResTitle">'.cfUTF8Encode($r['name']).'</div>';

	$o.='<div class="uiUsersResL2">'.cfUTF8Encode(@$r['definition']['resourceTypeLabel']).'</div>';
	$o.='<div class="uiUsersResL3">';
	$st=$r['subType'];
	switch ($r['type']){
		case 'explorer':
			if(@$r['sharedMode']=='folder') $o.=cfUTF8Encode($r['path']);
			elseif(@$r['sharedMode']=='list') $o.=cfCaption('filesAndFolders');
			break;
		case 'bookmarks':
			if($st=='std' ||$st=='mosaic'){
				if(@$r['bookmarksType']=='Opera') $o.='Opera';
				elseif(@$r['bookmarksType']=='Firefox') $o.='Firefox';
				elseif(@$r['bookmarksType']=='IE') $o.='Internet Explorer';
				else $o.=cfCaption('bookmarksCustom');
			}
			break;
		case 'website':
			if($st=='external') $o.=cfRGetVar('url');
			if($st=='html') $o.=cfUTF8Encode(basename(cfRGetVar('path')));
			break;
	}
	$o.='</div>';

	// UI users tab: include controls
	if($uc){
		$nb=count($uc->getResources());
		$i=0;
		$pos=0;
		foreach ($uc->getResources() as $id=>$res){
			if($id==$rid) $pos=$i;
			$i++;
		}

		// Resource actually displayed at login page
		if($uc->getVar('displayResourceAtLogin')==$rid) {
			$o.='<div style="position:absolute;left:3px;top:3px;cursor:pointer" '.outToolTip('<span style=\'text-decoration:line-through\'>'.cfCaption('userUseMainpage').'</span>').'>';
			$o.=outImage(outIcon('sel'),false,'onmouseover="this.src=\''.outIcon('selCancel').'\'" onmouseout="this.src=\''.outIcon('sel').'\'" onclick="uiUsersResLogin();stopPropagation(event)"');
			$o.='</div>';
		}

		// Remove resource control
		$o.=outImage(outIcon('cancelVS12'),false,outToolTip(cfCaption('genRemove')).' onclick="uiUsersResDel(\''.$rid.'\');stopPropagation(event)" class="biOverShown"', 'position:absolute;right:3px;top:3px;cursor:pointer');

		// Set as displayed at login page
		if($uc->getVar('displayResourceAtLogin')!=$rid) $o.=outImage(outIcon('selectedVS'),false,outToolTip(cfCaption('userUseMainpage')).' class="biOverShown" onclick="uiUsersResLogin(\''.$rid.'\');stopPropagation(event)"', 'position:absolute;right:3px;bottom:3px;cursor:pointer');

		// a single res may be selected
		$opt=array();
		// ondblclick function: configure resource
		$onclick=" ondblclick=\"try{parent.uiMain.tabSelect('uiResources.php?tab=share&resSelectedId=".$rid."','resources')} catch(e){wl.UICommand('showResource:".$rid."')}\"";
	}
	else {
		// Multiple res may be selected
		$opt=array('type'=>'checkbox');
		// onclick function
		$onclick=' onmousedown="toggleRes(\''.$rid.'\')" ondblclick="selectRes(\''.$rid.'\')"';
	}

	return outBlockItem('uiUsers',$o,'id="uiUsersResBlock_'.$rid.'"'.$onclick,'padding:5px;width:170px;text-align:left;margin:2px;margin-right:0px;margin-left:0px',$opt).'&nbsp;';
}

/**
 * @desc Return code for button for opening smtp server configuration
 * @return string
 *
 */
function rcMailoutConfigButton(){
	return outBt(cfCaption('genConfigure'),false,'javascript:rcMailoutConfigure()');
}

/*
 ***************************************************************************************************************************
 * Main parameter processing / display
 ***************************************************************************************************************************
 */

/**
 * @desc Return regexp for IP address constraint on a text data
 *
 * @return string
 */
function rcConstraintIP(){return '^(((25[0-5]|2[0-4][0-9]|1[0-9]{2}|[0-9]{1,2})\\.){3}(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[0-9]{1,2})){0,1}$';}

/**
 * @desc insert an image and / or a caption depending on $controlDefinitionArray definition
 *
 * @param array $controlDefinitionArray
 *			$controlDefinitionArray is a one or two elements array of image or caption definition (icd)
 *			(if $controlDefinitionArray is a string then it is treated as a one-element array)
 *			- if icd is false, don't display anything
 *			- if documentRoot/icd or resource's directory/icd is an image, display this image
 *			- else display icd as a caption
 *
 * @param alignLabelControl or alignControlLabel $align : defines wether control is located right or left of caption/image
 *			use this parameter to insert right or left margins
 * @return string
 */
function rcDisplayLabelAndOrImage($controlDefinitionArray,$align){
	global $resourceData;
	$out='';

	if($align==alignLabelControl) $alignStyle="margin-right:0.3em;"; else $alignStyle="margin-left:0.3em;";
	$alignStyle.='cursor:pointer;vertical-align:middle';

	// Check $controlDefinitionArray format
	if(!is_array($controlDefinitionArray)) {$item1=$controlDefinitionArray;$item2=false;}
	elseif (count($controlDefinitionArray)==1) {list($item1)=$controlDefinitionArray;$item2=false;}
	elseif (count($controlDefinitionArray)==2) {list($item1,$item2)=$controlDefinitionArray;}
	else return '';

	if($item1){
		// Check if item1 is an image
		if(is_file(cfAppDocRoot().$item1)) $out.=outImage($item1,false,false,$alignStyle);
		elseif(isset($resourceData['type']) && isset($resourceData['subType']) && file_exists(cfAppDocRoot().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$item1)) $out.=outImage('/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$item1, false, $alignStyle);
		// Else item1 is a caption
		else  $out.='<span  style="'.$alignStyle.'">'.$item1.'</span>';
	}
	if($item2){
		// Check if item2 is an image
		if(file_exists(cfAppDocRoot().$item2)) $out.='<img alt="" style="'.$alignStyle.'" src="'.$item2.'">';
		elseif(isset($resourceData['type']) && file_exists(cfAppDocRoot().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$item2)) $out.='<img alt="" style="'.$alignStyle.'" src="/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$item2.'">';
		// Else item2 is a caption
		else  $out.='<span  style="'.$alignStyle.'">'.$item2.'</span>';
	}
	// Insert spacing after caption/image if needed
	$out.="\n";
	if($align==alignControlLabel && ($item1 || $item2)){
		$item1=strtolower($item1);
		$item2=strtolower($item2);
		if(($item1 && (substr($item1,-4)=='<br>' || substr($item1,-5)=='<br/>'))) return $out;
		if(($item2 && (substr($item2,-4)=='<br>' || substr($item2,-5)=='<br/>'))) return $out;
		$out.="&nbsp;&nbsp;\n";
	}
	return $out;
}

/**
 * @desc process a resource parameter :
 *		- set default value
 *		- modify value if POST parameter received
 *		- display control for user input
 *
 * @param string $dataName : name of data
 * @param integer $dataType : type of data :
 * 		- dataText : text data
 * 		- dataNumeric : numeric data
 * 		- dataList : fixed list of data
 * 		- dataBoolean boolean
 * 		- dataFolderPath : path to folder
 * 		- dataFilePath : path to file
 * 		- dataArray: array of text values
 * 		- dataMultiList: ";" list of text values separated by a ;
 * 		- dataUserRights: list of allowed groups for a resource parameter. Format: (all|none|admin|list);(check|radio|list);(separated list of ids of allowed groups).
 *
 * @param mixed $defaultValue: default value of data
 * @param mixed $constraints: authorized values for data (see below). No constraint if set to false
 * @param mixed $controlDefinition: user control definition (caption, icons,... see below). Control is not displayed if set to false
 * @param boolean $lineBreak: true to include line break at end of control
 * @param boolean $sensitiveData: true to apply warning style to captions
 * @param options $options: array of extra options:
 * 					'POSTHook': name of PHP function to be called with POST value as parameters and that return processed POST
 * 					'displayHook': name of function to be called with data value as parameters, and that return what should be displayed into text/password input
 * 					'passwordClear': display password in clear
 * 					'disabled': control is disabled, POST commands are not processed
 * 					'faded': control is faded (use with disabled)
 * 					'hideCheckbox': set to true not to display checkbox (dataBoolean only)
 * @return boolean: true if OK, false if function parameters are not correct, or HTML control definition if $options['return'] is set
 */
function rcProcessData($dataName, $dataType, $defaultValue, $constraints, $controlDefinition=false,$lineBreak=true, $sensitiveData=false, $options=array()){
	global $resourceData;
	global $modifiedValuePHPFunction;
	global $resourceNameModified;

	static $usersIds;
	static $users;

	$out='';

	// For user rights data, set a specific data name so resource data with original $dataName can be populated with computed boolean value
	// when actually accessing resource
	if($dataType==dataUserRights) {
		$dataName='userRights_'.$dataName;
		if(!in_array($defaultValue,array('none','all','admin'))) $defaultValue='none';
		$defaultValue=$defaultValue.';'.(($defaultValue=='all'||$defaultValue=='none')?'check':'radio').';';

		// Build list of users
		if(!isset($users)){
			$adminId=false;
			$users=array();

			// Build users list
			foreach (WEnv::userConfigs() as $uid=>$user){
				if(0) $user=new WUserConfig($uid); // Zend IDE Workaround
				if($user->isAdministrator()) $adminId=$uid;
				$res=$user->getResourcesLinked();
				$users[$uid]=array('ico'=>cfExtGFX($user->iconPath(),16,16),'name'=>cfUTF8Encode($user->getVar('name')),'linked'=>isset($res[$resourceData['id']]),'admin'=>$user->isAdministrator());
				$usersIds[]=$uid;
			}
			// Add direct resource access
			$users['standalone']=array('ico'=>outIcon('directLink'),'name'=>cfCaption('directResourceAccess'),'linked'=>@$resourceData['directResourceAccessEnabled'],'admin'=>0);
			$usersIds[]='standalone';
		}

		// Indicate this resource has at least one userRights parameters so boolean data value can be computed by security.php before calling resource script
		//$resourceData['hasSpecificUserRights']=1;
	}

	// Memorize previous data value
	if(isset($resourceData[$dataName])) $previousValue=$resourceData[$dataName];

	// Memorise data with controls so it can be sent to a JS array for modification detection purposes in eSaveBt function
	if($controlDefinition) $_ENV['rcSyncData'][$dataName]=@$resourceData[$dataName];

	/*
	***************************************************************************************************************************
	* SET DEFAULT VALUE
	***************************************************************************************************************************
	*/
	if(!isset($resourceData[$dataName])) {
		// Migration from a boolean value
		if($dataType==dataUserRights){
			if(isset($resourceData[str_replace('userRights_','',$dataName)])) $defaultValue=($resourceData[str_replace('userRights_','',$dataName)])?'all;check;':'none;check;';
		}

		$resourceData[$dataName]=$defaultValue;
	}

	/*
	***************************************************************************************************************************
	* PROCESS POST COMMANDS
	***************************************************************************************************************************
	*/

	// correct POST value for booleans
	if($dataType==dataBoolean && $controlDefinition && isset($_POST['commandSent']) && !isset($_POST[$dataName])) $_POST[$dataName]=false;

	// Set new value if passed
	if(isset($_POST[$dataName]) && $controlDefinition!==false && !@$options['disabled']) {
		$requestedValue=cfUTF8Decode($_POST[$dataName],true,false,false,99999999999);

		// Hook: process POST data with external function
		if(isset($options['POSTHook'])) $requestedValue=call_user_func($options['POSTHook'],$requestedValue);

		switch($dataType){
			// Text data
			case dataArray:
			case dataText:
				if($dataName=='name') $previousName=$resourceData['name'];
				// Min-max constraint
				if(is_array($constraints) && count($constraints)==2){
					list($nbMin,$nbMax)=$constraints;
					if(($nbMin===false || strlen($requestedValue)>=$nbMin) && ($nbMax===false || strlen($requestedValue)<=$nbMax)) {
						if($dataType==dataArray && $resourceData[$dataName]!=$requestedValue) $modifiedArrayValue=1; // Used below to async update display
						$resourceData[$dataName]=$requestedValue;
					}
				}
				elseif (!$constraints) {
					if($dataType==dataArray && $resourceData[$dataName]!=$requestedValue) $modifiedArrayValue=1; // Used below to async update display
					$resourceData[$dataName]=$requestedValue;
				}

				if($dataName=='name' && $previousName!=$resourceData['name']) $resourceNameModified=true;
				break;

			// List of values separated by ";"; $constraints is an array of allowed values
			case dataMultiList:
				$newValue=array();
				// Ensure all values are allowed
				foreach (explode(';',$requestedValue) as $value) {
					if(strpos($value,'"')!==false) continue; // Don't allow " in values
					if(in_array($value,$constraints)) $newValue[]=$value;
				}
				$resourceData[$dataName]=implode($newValue,';');
				break;

			// List of allowed users
			case dataUserRights:
				// Migration from a boolean value
				if(!isset($resourceData[$dataName])){
					if(isset($resourceData[str_replace('userRights_','',$dataName)])){
						$resourceData[$dataName]=($resourceData[str_replace('userRights_','',$dataName)])?'all;check;':'none;check;';
					}
				}

				list($range,$display,$list)=@explode(';',$requestedValue,3);

				// Set default values for range and display
				if(!in_array($range,array('none','all','admin','list'))) $range='none';
				if(!in_array($display,array('check','radio','list'))) $display=($range=='list')?'list':(($range=='admin')?'radio':'check');

				// Parse list, and ensure all values are allowed
				$newValue=array();
				foreach (explode(';',$list) as $value) {
					if(strpos($value,'"')!==false) continue; // Don't allow " in values
					if(in_array($value,$usersIds)) $newValue[]=$value;
				}
				$newValue=implode($newValue,';');
				$resourceData[$dataName]=$range.';'.$display.';'.$newValue;
//cfDbg($dataName.'=>'.$requestedValue.' = > '.$resourceData[$dataName]);
				break;

			// Password
			case dataPassword:
				if($requestedValue && $requestedValue!=='weezoFakePassword') $resourceData[$dataName]=cfStoredPasswordCrypt($requestedValue);
				break;

			// Numeric data
			case dataNumeric:
				if($requestedValue==='') $requestedValue=0;
				if(is_numeric($requestedValue) && is_array($constraints) && count($constraints)==2){
					list($nbMin,$nbMax)=$constraints;
					if(($nbMin===false || $requestedValue>=$nbMin) && ($nbMax===false || $requestedValue<=$nbMax)) $resourceData[$dataName]=$requestedValue;
				}
				elseif (is_numeric($requestedValue) && !$constraints) $resourceData[$dataName]=$requestedValue;
				break;
			// List
			case dataList:
				if(is_array($constraints) && in_array($requestedValue,$constraints)) $resourceData[$dataName]=$requestedValue;
				break;
			// Boolean data
			case dataBoolean:
				if($requestedValue=='true' || $requestedValue=='on') $resourceData[$dataName]=true;
				else $resourceData[$dataName]=false;
				break;
			// File or folder path
			case dataFolderPath:
			case dataFilePath:
				// Remove double "\" but keep if starting path (UNC path), and convert "\" to "/";
				if(substr($requestedValue,0,2)=="\\\\") $requestedValue="\\\\".$requestedValue;
				$requestedValue=str_replace('\\\\','/',$requestedValue);
				$requestedValue=str_replace('\\','/',$requestedValue);

				// remove trailing slash
				if($requestedValue!="/" && substr($requestedValue,-1)=='/') $requestedValue=substr($requestedValue,0,strlen($requestedValue)-1);
				if($dataType==dataFolderPath){
					if(is_dir($requestedValue)) $resourceData[$dataName]=$requestedValue;
					elseif($_POST[$dataName]==cfCaption('explorerComputerRoot') && $constraints!='noComputerRoot') $resourceData[$dataName]='computerRoot';
					elseif (cfIsAsync()) rcAddAsyncResponse(cfAsyncXMLJSaction('dgn("'.$dataName.'").style.color="red"'));
				}
				elseif ($dataType==dataFilePath){
					if(is_file($requestedValue)) $resourceData[$dataName]=$requestedValue;
					elseif (cfIsAsync()) rcAddAsyncResponse(cfAsyncXMLJSaction('dgn("'.$dataName.'").style.color="red"'));
				}
				if(cfIsSubDir(cfAppDataRootDir(),$resourceData[$dataName])) $resourceData[$dataName]='*appDir*'.substr($resourceData[$dataName],strlen(cfAppDataRootDir()));
				break;
			/*
			// Array of strings
			case dataArray:
				if(cfCmpLeft($requestedValue,'add:')) $operation='add';
				elseif(cfCmpLeft($requestedValue,'del:')) $operation='del';
				else break;
				$requestedValue=substr($requestedValue,4);
				$requestedValue=str_replace("\t",' ',$requestedValue); // Replace tabs by spaces as it's used as a separator
				// Remove an entry
				if($operation=='del'){
					$values=explode("\t",$resourceData[$dataName]);
					if($k=array_search($requestedValue,$values)){
						unset($values[$k]);
						$resourceData[$dataName]=$values;
						$modifiedArrayValue=1; // Used below to async update display
					}
				}
				// Add an entry
				else{
					if($constraints && $constraints=='url'){
						// Check URL
						if(!cfCmpLeft(strtolower($requestedValue),'http://') && !cfCmpLeft(strtolower($requestedValue),'https://')) $requestedValue.='http://';
						if(!($h=cfSocketHTTPRequest($requestedValue,7,false,$false,$false,false,true))) break;
						fclose($h);
					}
					// Concatenate with a tab separator
					if($resourceData[$dataName]) $resourceData[$dataName].="\t";
					$resourceData[$dataName].=$requestedValue;

					$modifiedArrayValue=1; // Used below to async update display
				}

				break;
			*/
		}
	}

	/*
	***************************************************************************************************************************
	* DISPLAY CONTROL
	***************************************************************************************************************************
	*/
	if($controlDefinition){
		if($sensitiveData && $dataType!=dataUserRights) $out.='<span class="warning">';
		$disabledControl=(@$options['disabled'])?' disabled="disabled"':'';

		switch($dataType){
			// TEXT OR NUMERIC DATA
			// Control definition :
			// 	array(input lenght, $controlDefinitionArray) : text input (size=input lenght) with image and/or caption
			// 	array($controlDefinitionArray,input lenght) : image and/or caption with text input (size=input lenght)
			// 	array($controlDefinitionArray,input lenght, $controlDefinitionArray) : image and/or caption with text input (size=input lenght) and image and/or caption
			case dataText:
			case dataNumeric:
				$inputType='text';
			case dataPassword:
				if(!isset($inputType)) {
					// Show password
					if(isset($options['passwordClear'])){
						$inputValue=cfUTF8Encode(cfStoredPasswordDecrypt($resourceData[$dataName]));
						$inputType='text';
					}
					// Don't show password
					else{
						$inputValue=(cfStoredPasswordDecrypt($resourceData[$dataName]))?'weezoFakePassword':'';
						$inputType='password';
					}
				}
				else $inputValue=str_replace('"','&quot;',cfUTF8Encode($resourceData[$dataName]));

				$style=''; // <input> style, used to hide main input when an apply button is set

				// Constraints on entered value
				$extra='';
				if($constraints){
					// Min/max
					if(count($constraints)==2 && $dataType==dataNumeric) $extra=' constraints="range:'.$constraints[0].'-'.$constraints[1].'"';
					else if(count($constraints)==2) $extra=' constraints="length:'.$constraints[0].'-'.$constraints[1].'"';
					// Regex
					if(count($constraints)==1) $extra=' constraints="regex:'.htmlentities($constraints).'"';
				}

				// If modifications must be validated through an "Apply" button: hide main input and show alternative input & button
				if(isset($options['apply'])) $style='display:none';
				if(@$options['faded']) $style=outCssOpacity(50);
				if($style) $style=' style="'.$style.'"';


				// Hook: process value to be displayed by external function
				if(isset($options['displayHook'])) $inputValue=call_user_func($options['displayHook'],$resourceData[$dataName]);

				if(!is_array($controlDefinition)) $controlDefinition=array($controlDefinition,45); // default control : set size at 45
				if(count($controlDefinition)==2){
					list($item1,$item2)=$controlDefinition;
					// Textbox - caption
					if(is_numeric($item1)){
						$size=$item1;
						if(cfGetHostOS()=='osx') $size=ceil($size*2); // OSX Webkit WA
						$out.='<input onKeyUp="eSaveBt(\''.$dataName.'\',this.value)" onChange="eSaveBt(\''.$dataName.'\',this.value)" onBlur="this.focused=0;eSaveBt(\''.$dataName.'\',this.value)" onFocus="this.focused=1" type="'.$inputType.'" class="textInput" name="'.$dataName.'" size="'.$size.'" value="'.$inputValue.'"'.$disabledControl.$style.$extra.'>';
						$out.=rcDisplayLabelAndOrImage($item2,alignControlLabel);
					}
					// Caption - Textbox
					elseif (is_numeric($item2)){
						$size=$item2;
						if(cfGetHostOS()=='osx') $size=ceil($size*2); // OSX Webkit WA
						$out.=rcDisplayLabelAndOrImage($item1,alignLabelControl);
						$out.='<input onKeyUp="eSaveBt(\''.$dataName.'\',this.value)" onChange="eSaveBt(\''.$dataName.'\',this.value)" onBlur="this.focused=0;eSaveBt(\''.$dataName.'\',this.value)" onFocus="this.focused=1" type="'.$inputType.'" class="textInput" name="'.$dataName.'" size="'.$size.'" value="'.$inputValue.'"'.$disabledControl.$style.$extra.'>';
					}
				}
				// Caption - Textbox - Caption
				elseif (count($controlDefinition)==3 && is_numeric($controlDefinition[1])){
						list($item1,$item2,$item3)=$controlDefinition;
						$size=$item2;
						if(cfGetHostOS()=='osx') $size=ceil($size*2); // OSX Webkit WA
						$out.=rcDisplayLabelAndOrImage($item1,alignLabelControl);
						$out.='<input onKeyUp="eSaveBt(\''.$dataName.'\',this.value)" onChange="eSaveBt(\''.$dataName.'\',this.value)" type="'.$inputType.'" class="textInput" name="'.$dataName.'" size="'.$size.'" value="'.$inputValue.'"'.$disabledControl.$style.$extra.'>';
						$out.=rcDisplayLabelAndOrImage($item3,alignControlLabel);
				}
				// If an "apply" button is present, display a visible copy of original input and "apply" button
				if(isset($options['apply'])) {
					$out.='<input type="'.$inputType.'" class="textInput" name="'.$dataName.'_apply" size="'.$size.'" value="'.$inputValue.'"'.$disabledControl.$extra.' onKeyUp="eSaveBt(\''.$dataName.'\',this.value,1)">';
					$out.=outBt(cfCaption('explorerAudioApply'),false,"javascript:dataApply('".$dataName."')",false,false,'style="vertical-align:middle"');
				}
			break;

			// VALUES LIST : 2 types of controls : <select> droplist or radio buttons
			// Control definition :
			// <select> :
			//		array($controlDefinitionArray, array of items, $controlDefinitionArray)
			// 		items : array(value, caption)
			//  Radio buttons:
			//		array of items
			// 		items : array(left-side $controlDefinitionArray, value, right-side $controlDefinitionArray)
			case dataList:
				if(!is_array($controlDefinition) || count($controlDefinition)<2) return false;
				if($resourceData[$dataName]===true) $resourceData[$dataName]='true';
				if($resourceData[$dataName]===false) $resourceData[$dataName]='false';
				//<select>
				if(count($controlDefinition)==3 && is_array($controlDefinition[1]) && ((!is_array($controlDefinition[0])) || count($controlDefinition[0])<3)){
					list($item1,$item2,$item3)=$controlDefinition;
					$out.=rcDisplayLabelAndOrImage($item1,alignLabelControl);
					$out.='<select name="'.$dataName.'" size="1" class="textInput" onChange="eSaveBt(\''.$dataName.'\',this.value)">';
					foreach ($item2 as $value) if(is_array($value)){
						 if(count($value)==2) $out.='<option value="'.$value[0].'"'.(($resourceData[$dataName]==$value[0])?' selected':'').'>'.$value[1]."</option>\n";
						 if(count($value)==3) $out.='<option value="'.$value[0].'"'.(($resourceData[$dataName]==$value[0])?' selected':'').' style="background:url(\''.$value[2].'\') no-repeat;padding-left:20px">'.$value[1]."</option>\n";
					}
					$out.="</select>\n";
					$out.=rcDisplayLabelAndOrImage($item3,alignControlLabel);
				}
				else{
				// Radio buttons
					$radioNb=0;

					foreach ($controlDefinition as $value){
						if(is_array($value)){
							if(count($value)==3) list($item1,$item2,$item3)=$value;
							elseif(count($value)==2) {list($item2,$item3)=$value;$item1='';}
							else continue;

							if($item1) {$out.='<span style="cursor:pointer" onclick="dgn(\''.$dataName.'\','.$radioNb.').checked=\'checked\';dgn(\''.$dataName.'\','.$radioNb.').click()">'; $out.=rcDisplayLabelAndOrImage($item1,alignControlLabel); $out.='</span>';}
							else $out.=rcDisplayLabelAndOrImage($item1,alignControlLabel);
							//$out.=rcDisplayLabelAndOrImage($item1,alignLabelControl);
							$out.='<input type="radio" name="'.$dataName.'" style="cursor:pointer;vertical-align:middle"'.$disabledControl.' onClick="eSaveBt(\''.$dataName.'\',this.value)" value="'.$item2.'" '.(($resourceData[$dataName]==$item2)?'checked="checked"':'').">\n";
							if($item3) {$out.='<span style="cursor:pointer" onclick="dgn(\''.$dataName.'\','.$radioNb.').checked=\'checked\';dgn(\''.$dataName.'\','.$radioNb.').click()">'; $out.=rcDisplayLabelAndOrImage($item3,alignControlLabel); $out.='</span>';}
							else $out.=rcDisplayLabelAndOrImage($item3,alignControlLabel);
							$radioNb++;
						}
					}
				}
			break;

			// List of non-exclusive text items. $controlDefinition is an array of allowed value=>control definition
			case dataMultiList:
				$i=0;
				$selectedValues=explode(';',$resourceData[$dataName]);

				//$out.=outTableTransparent(false,'table-layout:fixed');
				foreach ($controlDefinition as $id=>$value){
					$out.='<div style="width:150px;display:block;overflow:hidden;text-overflow:ellipsis;float:left;white-space:nowrap">';
					$out.='<input type="checkbox"'.((in_array($id,$selectedValues))?' checked="checked"':'').' onclick="rcToggleMultiListItem(\''.$dataName.'\',\''.$id.'\',this.checked)">';
					$out.=rcDisplayLabelAndOrImage($value);
					//'&nbsp;<img style="width:16px;height:16px" src="'.$data['icon'].'">&nbsp;'.$data['name'];
					//$out.="</td>\n";
					$out.='</div>';
					$i++;
				}
				//while ($i%$cols>0) {$out.='<td>&nbsp;</td>';$i++;} $out.='</tr></table>';
				$out.='<br clear="all">';
				$out.='<input type="text" name="'.$dataName.'" value="'.$resourceData[$dataName].'" style="display:none">';
				$out.='<script type="text/javascript">rcMultiLists["'.$dataName.'"]="'.$resourceData[$dataName].'"</script>';
				break;

			// List of allowed users
			case dataUserRights:
				$i=0;
				list($range,$display,$sList)=@explode(';',$resourceData[$dataName],3);
				$list=explode(';',$sList);

				// Standard icons for file access rights
				$ico='';
				$sdn=substr($dataName,strlen('userRights_'));
				if($sdn=='downloadAllowed') $ico='dl2';
				elseif($sdn=='multipleDownloadAllowed') $ico='mdli';
				elseif($sdn=='writeAllowed') $ico='write';
				elseif($sdn=='modifyAllowed') $ico='modify';
				elseif($sdn=='editionAllowed') $ico='edit';
				elseif($sdn=='createFolderAllowed') $ico='newFolder';
				elseif($sdn=='executeAllowed') $ico='exec';
				elseif($sdn=='uploadAllowed') $ico='ul';
				elseif($sdn=='remoteDownloadAllowed') $ico='rdl';
				elseif($sdn=='downloadShortcut') $ico='dlShortcut';
				elseif($sdn=='shareAllowed') $ico='share';
				elseif($sdn=='fileViewAllowed') $ico='fullsizeView';
				elseif($sdn=='showExifData') $ico='exif3';

				// Fix values
				if($range=='list' && strlen(trim($sList))==0) {$range='none';$display='check';$resourceData[$dataName]='none;check;';}
				elseif($range=='list' && $display=='radio') {$display='list';$resourceData[$dataName]='list;list;'.$sList;}
				elseif($range!='list' && $display=='list') {$display='radio';$resourceData[$dataName]=$range.';radio;'.$sList;}

				$out.='<div class="rcUserRightsGroup'.(($display=='check' || cfSimpleMode())?'':' unfolded').'" id="'.$dataName.'_outer">';
				if($ico) $out.=outImageIcon($ico,false,'filter:gray;display:none');

				// Hidden field for global value
				$out.='<input type="text" name="'.$dataName.'" value="'.$resourceData[$dataName].'" style="display:none">';

				// Call an init function to workaround an IE display bug when groups should be shown
				if($display=='list') $out.='<script type="text/javascript">connect(W,"onload",function(){rcUserRightsItemInit("'.$dataName.'")})</script>';

				// Checkbox
				$out.='<input type="checkbox" onclick="if(!this.disabled) rcToggleUserRights(\''.$dataName.'\',this.checked)" name="'.$dataName.'_check" style="vertical-align:middle" ';
				if(@$options['faded'] || ($display!='check' && !cfSimpleMode())) $out.=' disabled';
				if ($range!=='none') $out.=' checked>';
				else $out.='>';

				// More options control
				if(!cfSimpleMode() && !@$options['faded']) $out.='<a href="javascript:rcToggleUserRightsDisplay(\''.$dataName.'\')"><img src="'.outIcon(($display=='check')?'groupsPlus':'groupsMinus').'" id="'.$dataName.'_bi" style="vertical-align:middle;'./*(($range=='admin'||$range=='list')?outCssOpacity(30):'').*/'"></a>';

				// Label
				if(!is_array($controlDefinition)) $controlDefinition=array($controlDefinition);
				if(cfSimpleMode()) $out.='<span '.(($sensitiveData)?' class="warning"':'').'>'.rcDisplayLabelAndOrImage($controlDefinition[0],alignControlLabel).'</span>';
				elseif(@$options['faded']) $out.='<span style="'.outCssOpacity(50).'"'.(($sensitiveData)?' class="warning"':'').'>'.rcDisplayLabelAndOrImage($controlDefinition[0],alignControlLabel).'</span>';
				else $out.='<span onclick="rcToggleUserRightsDisplay(\''.$dataName.'\')"'.(($sensitiveData)?' class="warning"':'').'>'.rcDisplayLabelAndOrImage($controlDefinition[0],alignControlLabel).'</span>';

				// Detail displayed when everything is folded but the range is admin or list
				$out.='<span id="'.$dataName.'_det" class="altText">';
				if($display=='check'){
					if($range=='admin') $out.='&nbsp;('.cfCaption('photoCfgCEAdmin').')';
					elseif($range=='list') {
						$tmp='';
						foreach ($list as $id)  $tmp.=', '.$users[$id]['name'];
						if($tmp) $out.='&nbsp;('.substr($tmp,2).')';
					}
				}
				//$out.=$range.' / '.$sList;
				$out.='</span>';

				// Radio options list (none, all, admin, list)
				$out.='<div id="'.$dataName.'_options"'.(($display=='check' || cfSimpleMode())?' style="display:none"':'').'>';

				$out.='<div class="explTitle">'.cfCaption('authorizedGroups').'</div>';
				$out.='<input type="radio" name="'.$dataName.'_radio" id="'.$dataName.'_radio_none" value="none" onclick="rcToggleUserRightsItem(\''.$dataName.'\',\'none\')" '.(($range=='none')?'checked':'').'>&nbsp;'.cfCaption('genNobody').'&nbsp;&nbsp;&nbsp;'."\n";
				$out.='<input type="radio" name="'.$dataName.'_radio" id="'.$dataName.'_radio_all" value="all" onclick="rcToggleUserRightsItem(\''.$dataName.'\',\'all\')" '.(($range=='all')?'checked':'').'>&nbsp;'.cfCaption('genEverybody').'&nbsp;&nbsp;&nbsp;'."\n";
				$out.='<input type="radio" name="'.$dataName.'_radio" id="'.$dataName.'_radio_admin" value="admin" onclick="rcToggleUserRightsItem(\''.$dataName.'\',\'admin\')" '.(($range=='admin')?'checked':'').'>&nbsp;'.cfCaption('photoCfgCEAdmin').'&nbsp;&nbsp;&nbsp;'."\n";
				$out.='<input type="radio" name="'.$dataName.'_radio" id="'.$dataName.'_radio_list" value="list" onclick="rcToggleUserRightsItem(\''.$dataName.'\',\'list\')" '.(($range=='list')?'checked':'').'>&nbsp;'.outImage(outIcon('users')).'&nbsp;'.cfCaption('genGroups').'<br>'."\n";

				// List of groups
				$out.='<div id="'.$dataName.'_list"'.(($display=='list')?'':' style="display:none"').'>';
				foreach ($users  as $id=>$value){
					if(@$value['admin']) $adminId=$id;
					$out.='<div id="'.$dataName.'_'.$id.'_list" class="rcUsersRightGroupBlock" style="'.(($value['linked'])?'':outCssOpacity(30)).'">';
					$out.='<input name="rcUserRightsItem" id="'.$dataName.'_item_input_'.$id.'" type="checkbox"'.((in_array($id,$list))?' checked="checked"':'').' onclick="rcToggleUserRightsItem(\''.$dataName.'\',\''.$id.'\',this.checked)" style="vertical-align:middle">';
					$out.='<img src="'.$value['ico'].'" style="vertical-align:middle;margin-right:0.5em">'.$value['name'];
					$out.='</div>';
					$i++;
				}
				$out.='<br clear="all">'; // End of groups
				$out.='</div>'; // End of groups
				$out.='</div>'; // End of options
				$out.='</div>'; // End of .dataUserRightsGroup

				// Set Javascript code that will give admin group id (needed by user rights data links)
				static $adminUserIdJSSet;
				if(!isset($adminUserIdJSSet) && !cfIsAsync()) {
					$out.='<script type="text/javascript">var rcUserRightsAdminId="'.$adminId.'"</script>';
					$adminUserIdJSSet=1;
				}


				break;


			// BOOLEAN DATA
			case dataBoolean:
				// Checkbox
				// Control definition :
				// 	array($controlDefinitionArray) : checkbox with image and/or caption
				//	array($controlDefinitionArray,$controlDefinitionArray) : 2 radio buttons with image and/or caption
				if(!is_array($controlDefinition)) $controlDefinition=array($controlDefinition);
				// Checkbox
				if(count($controlDefinition)==1){
					$out.='<input type="checkbox" onChange="eSaveBt(\''.$dataName.'\',this.checked)" onClick="eSaveBt(\''.$dataName.'\',this.checked)" name="'.$dataName.'" '.(($resourceData[$dataName])?'checked':'').' style="vertical-align:middle'.((@$options['hideCheckbox'])?';display:none':'').'"'.$disabledControl.'>';
					$out.='<span onclick="obj=dgn(\''.$dataName.'\');if(obj.disabled) return; obj.checked=!obj.checked;eSaveBt(\''.$dataName.'\',obj.checked);">';
					if($controlDefinition[0]!=' ') $out.=rcDisplayLabelAndOrImage($controlDefinition[0],alignControlLabel);
					$out.='</span>';
				}
				// Radio button (Warning : first item is always true, second is always false)
				elseif(count($controlDefinition)==2){
					$first=true;
					foreach ($controlDefinition as $value){
						// radio button

						// "true" button
						if($first) $out.='<input onChange="eSaveBt(\''.$dataName.'\',this.value)" onClick="eSaveBt(\''.$dataName.'\',this.value)" type="radio" name="'.$dataName.'" value="true" '.(($resourceData[$dataName])?'checked':'').' style="vertical-align:middle"'.$disabledControl.'>';
						// "false" button
						else $out.='<input onChange="eSaveBt(\''.$dataName.'\',!this.value)" onClick="eSaveBt(\''.$dataName.'\',!this.value)" type="radio" name="'.$dataName.'" value="false" '.((!$resourceData[$dataName])?'checked':'').' style="vertical-align:middle"'.$disabledControl.'>';

						// Image or label
						$out.='<span onclick="obj=D.getElementsByName(\''.$dataName.'\')['.(($first)?'0':'1').'];if(obj.disabled) return;obj.checked=true;eSaveBt(\''.$dataName.'\','.(($first)?'!':'').'this.checked);">';
						$out.=rcDisplayLabelAndOrImage($value,alignControlLabel);
						$out.='</span>';
						if($first) $first=false;
					}
				}
			break;

			// FILE OR DIRECTORY
			// Control definition :
			//	optional string : caption
			case dataFolderPath:
			case dataFilePath:
				// Corrects file/path case
				$path=cfCorrectFilenameCase(str_replace('*appDir*',cfAppDataRootDir(),$resourceData[$dataName]),true);
				if(cfIsSubDir(cfAppDataRootDir(),$path)) $resourceData[$dataName]='*appDir*'.substr($path,strlen(cfAppDataRootDir())); else $resourceData[$dataName]=$path;

				if(is_array($controlDefinition)||substr($controlDefinition,0,4)!='<img') $out.='<img alt="" src="'.(($dataType==dataFolderPath)?outIcon('folder'):outIcon('blankFile')).'" style="vertical-align:middle; margin-right:0.5em;">';
				// Caption
				if($controlDefinition!==true && !is_array($controlDefinition)) $out.=$controlDefinition;
				elseif($dataType==dataFolderPath) $out.='<b>'.cfCaption('genDirectory').'</b>';
				else $out.=cfCaption('genFile');

				// Input text box
				$size=32;
				if(cfGetHostOS()=='osx') $size=ceil($size*2); // OSX Webkit WA
				$out.='&nbsp;&nbsp;';
				$out.='<input class="textInput" type="text" name="'.$dataName.'" id="'.$dataName.'" ';
				$out.='onKeyUp="eSaveBt(\''.$dataName.'\',this.value);rcResetColor(W.event,this)" ';
				$out.='onchange="eSaveBt(\''.$dataName.'\',this.value);" ';
				$out.='size="'.$size.'" style="'.(($path!='computerRoot' && !file_exists($path))?'color:red;':'').'" ';
				$out.='value="'.(($path=='computerRoot')?cfCaption('explorerComputerRoot'):cfUTF8Encode($path)).'"'.$disabledControl.'>&nbsp;&nbsp;'; // Note: &nbsp; margins is an IE bug WA

				if($dataType==dataFolderPath){
					// Folder selection button (application environment only)
					if(cfIsInApp()){
						$out.=outBt(cfCaption('genBrowse'),outIcon('fi/folder'),"javascript:wl.UICommand('openFolder:'+encodeURIComponent(dgn('".$dataName."').value),'".$dataName."')",$dataName.'_browseBt',cfCaption("genDirectory"));
					}
					// Computer root button
					if($constraints!='noComputerRoot' && !cfSimpleMode()) {
						// On windows: fill value with computer root caption
						if(cfGGetVar('OSFamily')=='windows') $out.=outBt('', outIcon('computerRoot'),'javascript:dgi(\''.$dataName.'\').value=\''.cfCaption('explorerComputerRoot').'\';eSaveBt(\''.$dataName.'\',this.value);',cfCaption("explorerComputerRoot"));
						// On other systems, just set "/"
						else $out.=outBt('',outIcon('computerRoot'),"javascript:dgi('".$dataName."').value='/';eSaveBt('".$dataName.'\',this.value);',cfCaption("explorerComputerRoot"));
					}
					// Special : 'path' data is commonly used as base directory for explorer resources - show help button
					if($dataName=='path') $out.='&nbsp;&nbsp;&nbsp;'.outHelpButton('helpPath');
				}
				else{
					// File selection button (application environment only)
					if(cfIsInApp())
						$out.=outBt(cfCaption('genBrowse'),outIcon('explorer'),"javascript:wl.UICommand('openFile','".$dataName.";".$constraints."')",$dataName.'_browseBt',false,'onclick="eSaveBt(\''.$dataName.'\',this.value)"');
				}

			break;
			// Array of values
			case dataArray:
				$l='';

				// 1 item: use it as left-side item
				if(!is_array($controlDefinition)) $controlDefinition=array($controlDefinition);
				if(count($controlDefinition)==1 && $controlDefinition[0]!=' ') $l1=$l2=rcDisplayLabelAndOrImage($controlDefinition[0],alignControlLabel);
				// 2 items: use 1st for existing entries and 2nd for new entry
				elseif(count($controlDefinition)==2) {
					$l1=rcDisplayLabelAndOrImage($controlDefinition[0],alignControlLabel);
					$l2=rcDisplayLabelAndOrImage($controlDefinition[1],alignControlLabel);
				}

				$out.='<div id="'.$dataName.'_outer">';
				$tmpOut='';

				foreach (explode("\t",$resourceData[$dataName]) as $k=>$v) if($v)
					$tmpOut.=$l1.'<input type="text" class="textInput" value="'.str_replace('"','',cfUTF8Encode($v)).'" style="width:20em;vertical-align:bottom" disabled>'.outButton(false,"javascript:rcArrayItemRemove('".$dataName."','".$k."')",outIcon('minus')).'<br>';

				$tmpOut.=$l2.'<input id="'.$dataName.'_input" class="textInput" style="width:20em;vertical-align:bottom" onkeyup="if(wl.eventKC(W.event)==13) rcArrayItemAdd(\''.$dataName.'\')">'.outButton(false,"javascript:rcArrayItemAdd('".$dataName."')",outIcon('plus'),false,false,'style="position:absolute;right:0"');

				if(isset($modifiedArrayValue)) rcAddAsyncResponse(cfAsyncXMLInnerHTMLbyId($tmpOut,$dataName.'_outer').cfAsyncJSaction('')); else $out.=$tmpOut;

				$out.='</div>';

				$inputValue=str_replace('"','&quot;',cfUTF8Encode($resourceData[$dataName]));
				$out.='<input type="text" name="'.$dataName.'" value="'.$inputValue.'" style="display:none">';

			break;
		}
		if($sensitiveData) $out.='</span>';

		// Line break
		if($lineBreak) $out.="<br>\n";
	}


	// If a callback function has been set (rcSetModifiedValuePHPFunction) and data has changed,
	// call callbackfunction with dataName and previousValue parameters
	if(isset($modifiedValuePHPFunction)){
		if(!isset($previousValue)) $modifiedValuePHPFunction($dataName,false);
		elseif($previousValue!=$resourceData[$dataName]) $modifiedValuePHPFunction($dataName,$previousValue);
	}

	if(!@$options['returnResult']) echo $out; else return $out;
}
?>