<?php
/**
 * Standard bookmarks configuration file
 *
 * Configuration of standard bookmarks resource
 * this script is included by /res/administration/std/resourceConfig.php
 * this script should always be accessed through /res/administration/index.php including resourceConfig.php including this script
 *
 * resourceConfig.php sets 2 variables :
 * $resourceData (array) which contains resource parameters
 * $_ENV['configurationEnvironment'] which indicates if resourceConfig.php has been called from browser ('browser')
 * or from application's embeded browser ('application')
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

require(INCLUDE_DIR.'bookmarkFunctions.php');

/**
 * @desc function called when a data has been modified
 *
 * @param string $dataName : name of changed data
 * @param mixed $previousDataValue : previous value
 */
function dataChanged($dataName, $previousDataValue){
	global $resourceDataDir;
	// If bookmarks type or path or size have changed, destroy all generated thumbnails
	if($dataName=='bookmarksPath' || $dataName=='bookmarksType' || $dataName=='thumbnailMaxSize'){
		if(file_exists($resourceDataDir) && is_dir($resourceDataDir)){$dir=$resourceDataDir;cfUnlinkDir($dir);mkdir($dir);}
	}
}



/*
 ***************************************************************************************************************************
 * Verify access rights
 ***************************************************************************************************************************
 */
require_once(INCLUDE_DIR.'resourceConfigFunctions.php');
rcConfigRights(false);

// Set PHP function that will be called on data change
rcSetModifiedValuePHPFunction('dataChanged');

/**
 * Async bookmark import
 */
if(isset($_POST['importBookmarks'])){
	// Copy $resourceData into $_SESSION['res'][0] so functions using cfRgetVar / cfRSetVar can be used
	rcCopyResourceDataToRVar();

	// Send async response
	cfAsyncHeader();

	// Read custom bookmarks
	readBookmarks('Custom');
	$custom=cfRGetVar('bookmarks');
	cfRSetVar('bookmarks',false);
	$imported=false;

	// Import IE bookmarks
	if($_POST['importBookmarks']=='IE')	{
		$browserName=cfCaption('bookmarksIE');
		// Set bookmarksPath to IE bookmarks path to override custom bookmarks path
		cfRSetVar('bookmarksPath',browserBookmarksLocation('IE'));
		// Import IE bookmarks
		readBookmarks('IE');
		$imported=cfRGetVar('bookmarks');
	}

	// Import FF bookmarks
	elseif($_POST['importBookmarks']=='FF')	{
		$browserName=cfCaption('bookmarksFF');
		// Set bookmarksPath to FF bookmarks path to override custom bookmarks path
		cfRSetVar('bookmarksPath',browserBookmarksLocation('Firefox'));
		// Import FF bookmarks
		readBookmarks('Firefox');
		$imported=cfRGetVar('bookmarks');
	}

	// Import Opera bookmarks
	elseif($_POST['importBookmarks']=='Opera')	{
		$browserName=cfCaption('bookmarksOpera');
		// Set bookmarksPath to Opera bookmarks path to override custom bookmarks path
		cfRSetVar('bookmarksPath',browserBookmarksLocation('Opera'));
		// Import Opera bookmarks
		readBookmarks('Opera');
		$imported=cfRGetVar('bookmarks');
	}

	// Failed to import bookmarks
	if(!$imported) echo cfAsyncXMLJSaction('alert("'.cfCaption('transfersFailed').'")');

	// Merge previous bookmark and imported bookmarks
	$tail=array_pop($custom);
	$nb=0; // Number of imported bookmarks
	foreach ($imported as $key=>$value) if(isset($value['url'])){
		$existing=false;
		foreach ($custom as $key2=>$value2){
			if($value2['type']=='b' && $value2['url']==$value['url']) {$existing=true;break;}
		}
		if(!$existing) {
			$nb++;
			$custom[]=$value;
		}
	}
	$custom[]=$tail;

	// Generate custom bookmarks file
	$output=''; $id=1;
	foreach ($custom as $key=>$value) if(isset($value['url'])){
		$output.="#URL\n	ID=".$id."\n	NAME=".cfUTF8Encode($value['name'])."\n	";
		if(isset($value['comments'])) $output.='DESCRIPTION='.cfUTF8Encode($value['comments'])."\n	";
		$output.='URL='.$value['url']."\n\n";
		$id++;
	}

	// Save file
	if(file_put_contents(str_replace('*appDir*',cfAppRootDir(),$resourceData['bookmarksPath']).'/bookmarks.adr',$output))
		echo cfAsyncXMLJSaction('alert("'.cfCaption('configImportOK',$browserName).' ('.$nb.')")');
	else
		echo cfAsyncXMLJSaction('alert("'.cfCaption('transfersFailed').'")');

	cfAsyncFooter();
}


/*
 ***************************************************************************************************************************
 * Display configuration form
 ***************************************************************************************************************************
 */

// Insert Javascript function needed for save button activation, and <form> with hidden fields.
rcInsertScriptAndForm();

// Resource icon & name
rcInsertIconName($resourceData);

// Bookmarks type (IE/Opera/Custom)

if(cfGGetVar('OSFamily')=='windows'){
	$options=array('IE','Opera','Firefox','Custom');
	$controlDefinition=array(
		array(outIcon('iconIE.png'),'IE',cfCaption('bookmarksIE').'&nbsp;&nbsp;&nbsp;'),
		array(outIcon('iconFF.png'),'Firefox',cfCaption('bookmarksFF').'&nbsp;&nbsp;'),
		array(outIcon('iconOpera.png'),'Opera',cfCaption('bookmarksOpera').'<br>'),
		array('<div class="frame3"><img src="'.rcResourceImage('iconCustom.png').'" style="vertical-align:bottom">','Custom',cfCaption('bookmarksCustom').''));
}
else{
	$options=array('Opera','Firefox','Custom');
	$controlDefinition=array(
		array(outIcon('iconFF.png'),'Firefox',cfCaption('bookmarksFF').'&nbsp;&nbsp;'),
		array(outIcon('iconOpera.png'),'Opera',cfCaption('bookmarksOpera').'&nbsp;'),
		array('<img src="'.rcResourceImage('iconCustom.png').'" style="margin-left:-10px;vertical-align:bottom">','Custom',cfCaption('bookmarksCustom').''));
}

echo '<div class="frame2">';
echo '<div class="frame2Header">'.cfCaption('bookmarksType').'</div>';
rcProcessData('bookmarksType',dataList,'Custom',$options, $controlDefinition);
echo '<b>'.cfCaption('genImport').cfCaption('genSeparator').'</b>&nbsp;';
if(cfGGetVar('OSFamily')=='windows') echo outBt('',outIcon('iconIE.png'),'javascript:importBookmarks(\'IE\')',cfCaption('bookmarksIE'));
echo outBt('',outIcon('iconFF.png'),'javascript:importBookmarks(\'FF\')',cfCaption('bookmarksFF'));
echo outBt('',outIcon('iconOpera.png'),'javascript:importBookmarks(\'Opera\')',cfCaption('bookmarksOpera'));
echo '</div><br>';


// Separator ***************************************************************************
echo outConfigSeparator();


// Bookmarks path
rcProcessData('bookmarksPath',dataFolderPath,cfAppDataRootDir().'/shared',false,cfCaption('bookmarksPath'));
echo '</div>';


// Separator ***************************************************************************
echo outConfigSeparator();


// Thumbnails
echo '<div class="frame2">';
echo '<div class="frame2Header">'.cfCaption('genThumbnails').'</div>';
if(!$limitedConfig) echo outButton(cfCaption('genClear'),'javascript:clearThumbnails()',outIcon('eraser'),false,false,'style="float:right"');
echo cfCaption('bookmarksDisplayThumbnails').'&nbsp;&nbsp;&nbsp;';
rcProcessData('displayThumbnails',dataBoolean,true,false,array(cfCaption('genYes'),cfCaption('genNo')));

// Full-size thumbnails size
rcProcessData('thumbnailSize',dataNumeric,640,false,false);
rcProcessData('thumbnailJPGQuality',dataNumeric,60,false,false); // Thumbnails jpg quality


// Thumbnails refresh rate
rcProcessData('refreshRate',dataNumeric,15,array(0,999999),array(cfCaption('genRefreshRate'),2,cfCaption('genDays'))); // Thumbnails refresh rate
echo '</div>';


// Separator ***************************************************************************
echo outConfigSeparator();



// Thumbnails edition
echo '<div class="frame2">';
echo '<div class="frame2Header">'.cfCaption('genEdit').'</div>';

rcProcessData('editionAllowed',dataUserRights,'all',false,cfCaption('bookmarksModificationAllowed'));

echo '</div>';

rcProcessData('userTempDirAccessAllowed',dataBoolean,false,false,false/*do not display control*/); // Allow/forbid access to user temp directory (data/sessiondata/(session_id)/)
rcProcessData('resourceDataDirAccessAllowed',dataBoolean,true,false,false/*do not display control*/); // Allow/forbid access to resource directory (data/res/(res type)/(res subType)/(res file name)/)

// Separator ***************************************************************************
echo outConfigSeparator();



// Resource theme
echo rcThemeSelectionControl();

// Save / cancel buttons
echo '<br/>'.rcButtonSaveCancel();

// If resource need access to resource data dir, create it
rcCreateResourceDataDir();

// Indicate that resource has been configured
rcProcessData('resourceConfigured',dataBoolean,true,false,false/*do not display control*/);

// Save parameters
rcWriteResourceFile($resourceData, $fileName);

// Enable / disable bookmarks edition by admin/everybody on "allow edition" button value
?>
<script language="javascript" type="text/javascript">
var prevType="<?php echo $resourceData['bookmarksType'];?>";
var FF3=<?php if(file_exists(browserBookmarksLocation('Firefox').'/places.sqlite')) echo 1; else echo 0; ?>;
<?php

// existing IE default bookmarks location ?
echo 'var bookmarksPahtIEOK='.((browserBookmarksLocation('IE'))?'1':'0')."\n";

// existing Firefox default bookmarks location ?
echo 'var bookmarksPahtFFOK='.((browserBookmarksLocation('Firefox'))?'1':'0')."\n";
// existing Opera default bookmarks location ?
echo 'var bookmarksPahtOperaOK='.((browserBookmarksLocation('Opera'))?'1':'0')."\n";
?>
function dataChanged(dataName,dataValue){
	if(dataName=="displayThumbnails"){
		dataValue=dgn(dataName).checked; // Dirty fix, dataValue is not OK
		D.getElementsByName("refreshRate")[0].disabled=!dataValue;
	}
	if(dataName=="editAllowed"){
		if(!dataValue) {
			D.getElementsByName("everybodyCanEdit")[0].disabled=true;
			D.getElementsByName("everybodyCanEdit")[1].disabled=true;
		}
		else{
			D.getElementsByName("everybodyCanEdit")[0].disabled=false;
			D.getElementsByName("everybodyCanEdit")[1].disabled=false;
		}
	}
	else if(dataName=="bookmarksType" && dataValue!=prevType){
		if(dataValue=="IE") {
			dgn("bookmarksPath").value="<?php echo browserBookmarksLocation('IE',1);?>";
			if(bookmarksPahtIEOK) dgn("bookmarksPath").style.color='';else  dgn("bookmarksPath").style.color='red';
		}
		if(dataValue=="Firefox") {
			dgn("bookmarksPath").value="<?php echo browserBookmarksLocation('Firefox',1);?>";
			if(bookmarksPahtFFOK) dgn("bookmarksPath").style.color='';else  dgn("bookmarksPath").style.color='red';
			if(!FF3){
				D.getElementsByName("editAllowed")[0].disabled=true;
				D.getElementsByName("editAllowed")[1].disabled=true;
				D.getElementsByName("editAllowed")[1].checked=0;
				D.getElementsByName("editAllowed")[1].checked=1;
				D.getElementsByName("everybodyCanEdit")[0].disabled=true;
				D.getElementsByName("everybodyCanEdit")[1].disabled=true;
			}
		}
		if(dataValue=="Opera") {
			dgn("bookmarksPath").value="<?php echo browserBookmarksLocation('Opera',1);?>";
			if(bookmarksPahtOperaOK) dgn("bookmarksPath").style.color=''; else  dgn("bookmarksPath").style.color='red';
		}
		if(dataValue=="Custom") {
			D.getElementsByName("bookmarksPath")[0].value="<?php echo browserBookmarksLocation('Custom');?>";
			return;
		}
		if(dataValue!="Firefox"){
			D.getElementsByName("editAllowed")[0].disabled=false;
			D.getElementsByName("editAllowed")[1].disabled=false;
			if(D.getElementsByName("editAllowed")[0].checked){
				D.getElementsByName("everybodyCanEdit")[0].disabled=false;
				D.getElementsByName("everybodyCanEdit")[1].disabled=false;
			}
			else{
				D.getElementsByName("everybodyCanEdit")[0].disabled=true;
				D.getElementsByName("everybodyCanEdit")[1].disabled=true;
			}
		}
		prevType=dataValue;
	}

	if(dgn("bookmarksPath").style.color=='red') return false;
}

function importBookmarks(browser){
	if(browser=='IE') if(!confirm("<?php echo cfCaption('genImport').cfCaption('genSeparator').'\\n'.cfCaption('bookmarksIE');?>")) return;
	if(browser=='FF') if(!confirm("<?php echo cfCaption('genImport').cfCaption('genSeparator').'\\n'.cfCaption('bookmarksFF');?>")) return;
	if(browser=='Opera') if(!confirm("<?php echo cfCaption('genImport').cfCaption('genSeparator').'\\n'.cfCaption('bookmarksOpera');?>")) return;
	requestMethod="POST";
	sendData("importBookmarks="+browser,undefined,true,3);
}

function dataInitForm(){
	dataChangedNV("editAllowed");
	dataChanged("displayThumbnails");
<?php
if($resourceData["bookmarksType"]=='Firefox' && !file_exists($resourceData['bookmarksPath'].'/places.sqlite')){
?>
D.getElementsByName("editAllowed")[0].disabled=true;
D.getElementsByName("editAllowed")[1].disabled=true;
D.getElementsByName("editAllowed")[1].checked=0;
D.getElementsByName("editAllowed")[1].checked=1;
D.getElementsByName("everybodyCanEdit")[0].disabled=true;
D.getElementsByName("everybodyCanEdit")[1].disabled=true;
<?php
}
?>
}
function clearThumbnails(browser){wl.mask.show(0,0); sendData("clearThumbnails=1",undefined,true,3);}
dataInitForm();
</script>